/*
** Module   : fwd-embedded-driver.p
** Abstract : FWD driver for running ADM windows in embedded mode.
**
** Copyright (c) 2017, Golden Code Development Corporation.
**
** _#_ _I_ __Date__ ________________________________Description___________________________________
** 001 CA  20170825 Created initial version.
*/

/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

/*
&GLOB ADM-VERSION ADM1.1
&GLOB ADM-VERSION ADM2.2
*/

/* replace the __ADM_VERSION__ string with your ADM version */
&GLOB ADM-VERSION __ADM_VERSION__

/* allow both explicit and implicit (via p2j.embedded.js) run of this program. */
def new global shared var fwd-driver-handle as handle.

def new global shared temp-table fwd-embedded-window 
   field pname as char
   field phandle as handle.
   
def var lastWindow as char init ?.

if valid-handle(fwd-driver-handle) 
then do:
   /* if the driver was already loaded, don't run the external program code.  the APIs, even if
      invoked via this instance, it will re-use the globally shared temp-table */
   return.
end.

fwd-driver-handle = this-procedure.

procedure inMsg_logout.
   quit.
end.

procedure inMsg_fwdDestroyWindow.
   def input param pfile as char.

   def var pnum as int.
   def var phandle as handle.
   
   find first fwd-embedded-window where fwd-embedded-window.pname = pfile no-error.

   if not avail(fwd-embedded-window) then return.

   phandle = fwd-embedded-window.phandle.
   
   do on stop undo, leave:
   &IF "{&ADM-VERSION}" >= "ADM2.2"
   &THEN
      run disable_UI in phandle no-error.
   &ELSE
      run adm-disable in phandle no-error.
   &ENDIF
   end.
   
   delete object phandle no-error.

   delete fwd-embedded-window.
end.

procedure inMsg_fwdShowWindow.
   def input param pfile as char.
   def input param pnum as int.
   def input param hiddenWidgets as char.

   def var phandle as handle.
   def var hframe as handle.
   def var hwin as handle.
   
   if lastWindow <> pfile 
   then do:
      run inMsg_fwdDestroyWindow(lastWindow).
   end.
   lastWindow = pfile.
   
   find first fwd-embedded-window where fwd-embedded-window.pname = pfile no-error.
   if not avail fwd-embedded-window 
   then 
   do transaction:
      run value(pfile) persistent set phandle.

      hwin = dynamic-function("fwdGetWindow" in phandle).
      
      run initEmbeddedWindow(hwin).
      
      &IF "{&ADM-VERSION}" >= "ADM2.2"
      &THEN
         run initializeObject in phandle.
      &ELSE
         run dispatch in phandle ("initialize") no-error.
      &ENDIF

      create fwd-embedded-window.
      fwd-embedded-window.pname = pfile.
      fwd-embedded-window.phandle = phandle.
   end.
   else do:
      phandle = fwd-embedded-window.phandle.
   end.
   
   hwin = dynamic-function("fwdGetWindow" in phandle).
   hframe = dynamic-function("fwdGetMainFrame" in phandle).
   hframe:visible = false.
   hframe:hidden = true.
   
   &IF "{&ADM-VERSION}" >= "ADM2.2"
   &THEN
      run selectPage in phandle (pnum) no-error.
   &ELSE
      run select-page in phandle (pnum) no-error.
   &ENDIF
   
   run initEmbeddedWidgets (input pnum, input phandle, input hiddenWidgets, input hwin, input hframe).
   
   hframe:visible = true.
end.

procedure initEmbeddedWindow private.
   def input param hwin as handle.
   
   /* remove any menu attached to the window */
   hwin:menubar = ?.
   
   /* remove status-area and message-area */
   hwin:status-area = no.
   hwin:message-area = no.
   
   /* move the window to top */
   hwin:move-to-top().
end.

function isPageFrame returns logical(input hwid as handle, input pageNum as int, input hpages as handle extent 100).
   def var i as int.
   
   do i = 1 to pageNum:
      if hpages[i] = hwid:instantiating-procedure
      then do:
         return true.
      end.
   end.
   
   return false.
end.

procedure initEmbeddedWidgets private.
   def input param pnum as int.
   def input param phandle as handle.
   def input param hiddenWidgets as char.
   def input param hwin as handle.
   def input param hframe as handle.

   def var hwid as handle.
   def var h as handle.
   def var widname as char.
   def var idx as int.

   def var hpages as handle extent 100.
   def var pageNum as int.
   run getPageHandles in phandle (input pnum, output hpages, output pageNum).

   /* go through the frames in the window and hide all frames except the main one */
   h = hwin:first-child.
   do while h <> ?:
      if h <> hframe
      then do:
         hide h no-pause.
      end.
      
      h = h:next-sibling.
   end.
   
   /* hide all widgets which are not visible in embedded mode */
   idx = 1.
   do while idx <= num-entries(hiddenWidgets, ","):
      widname = entry(idx, hiddenWidgets, ",") no-error.
      if not error-status:error
      then do:
         hwid = hframe:first-child:first-child.
         do while hwid <> ?:
            if hwid:name = widname 
            then do:
               hwid:hidden = true.
            end.
            hwid = hwid:next-sibling.
         end.
      end.
      idx = idx + 1.
   end.

   hwid = hframe:first-child:first-child.
   do while hwid <> ?:
      if hwid:type = "frame" and hwid <> hframe and not isPageFrame(hwid, pageNum, hpages) 
      then do:
         hwid:hidden = true.
      end.
      hwid = hwid:next-sibling.
   end.
   
   def var wrow as dec.
   def var wcol as dec.
   def var mrow as dec.
   def var mcol as dec.
   
   mrow = ?.
   mcol = ?.
   
   /* go through all widgets and compute min row/col from non-hidden ones */
   hwid = hframe:first-child:first-child.
   do while hwid <> ?:
      if not hwid:hidden
      then do:
         wrow = hwid:row.
         wcol = hwid:col.

         if mrow = ? or mrow > wrow then mrow = wrow.
         if mcol = ? or mcol > wcol then mcol = wcol.
      end.

      hwid = hwid:next-sibling.
   end.
   
   /* re-adjust the widget location */
   hwid = hframe:first-child:first-child.
   do while hwid <> ?:
      if not hwid:hidden
      then do:
         wrow = hwid:row.
         wcol = hwid:col.
         hwid:row = wrow - mrow + 1.
         hwid:col = wcol - mcol + 1.
      end.

      hwid = hwid:next-sibling.
   end.
   
   /* set the frame to top-left corner */
   hframe:row = 1.
   hframe:col = 1.
end.
