/*
** Module   : JarOperationResult.java
** Abstract : of JAR-related or API-related operation.
**
** Copyright (c) 2011-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- ----------------Description----------------------
** 001 SVL 20111209 Created initial version.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/
package com.goldencode.p2j.admin;

/**
 * Result of JAR-related or API-related operation. Indicates success or
 * specific failure. In addition to other codes, contains all error codes from
 * ErrorCode {@link com.goldencode.p2j.classloader.ErrorCode}.
 */
public enum JarOpErrorCode
{
   SUCCESS("No error."),

   NOT_FOUND("Jar file could not be found."),

   IN_CLASSPATH("Jar is already in classpath."),

   ALREADY_REGISTERED("Class loader for jar already registered."),

   INIT_ERROR("Could not initialize class loader for jar."),

   NOT_REGISTERED("Could not remove jar - class loader not registered for it."),

   CLOSE_ERROR("Error closing class loader for jar."),

   BAD_LOADER("The system class loader is not set to MultiClassLoader !"),


   INSUFFICIENT_RIGHTS("Insufficient access rights."),

   ACTIVE_SESSIONS_PRESENT("There are related active sessions."),

   ERROR_TERMINATING_SESSIONS("Could not terminate related sessions."),

   ERROR_DEREG_NETWORK_SERVER("Could not deregister static network server."),

   ERROR_LOADING_HOOK("Could not load hook."),

   ERROR_RUN_HOOK("Could not run hook."),

   ERROR_LOADING_API_IFACE("Could not load API interface class."),

   ERROR_LOADING_API_IMPL("Could not load API implementation class."),

   CANNOT_OPEN_DIRECTORY_BRANCH("Could not open directory branch."),

   CANNOT_MODIFY_DIRECTORY_NODE("Could not modify/create directory node."),

   CANNOT_COMMIT_DIRECTORY("Could not commit directory changes."),

   ERROR_DELETING_NODE("Could not delete target directory node.");

   /** The message for this error code. */
   private String msg;

   /**
    * C'tor for the error code.
    *
    * @param   msg
    *          The message.
    */
   JarOpErrorCode(String msg)
   {
      this.msg = msg;
   }

   /**
    * Returns the operation result description that can be displayed in the
    * UI messages.
    *
    * @return the operation result description.
    */
   public String toString()
   {
      return msg;
   }
}