/*
** Module   : AboutDialogView.java
** Abstract : AboutDialogView represents a custom about dialog to show the application details.
**
** Copyright (c) 2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- ----------------Description----------------------
** 001 SBI 20170601 Created initial version.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/


package com.goldencode.p2j.admin.client.application.home;

import org.gwtbootstrap3.client.ui.Button;
import org.gwtbootstrap3.client.ui.Modal;
import org.gwtbootstrap3.client.ui.html.Paragraph;
import org.gwtbootstrap3.client.ui.html.Span;

import com.goldencode.p2j.admin.client.widget.dialog.ModalViewWithUiHandlers;
import com.google.gwt.uibinder.client.UiBinder;
import com.google.gwt.uibinder.client.UiField;
import com.google.gwt.user.client.ui.Image;
import com.google.gwt.user.client.ui.Widget;
import com.google.inject.Inject;
import com.gwtplatform.mvp.client.UiHandlers;


/**
 * AboutDialogView represents a custom about dialog to show the application details.
 */
public class AboutDialogView
extends ModalViewWithUiHandlers<UiHandlers>
{
   /** GWT UI Creator */
   interface Binder
   extends UiBinder<Widget, AboutDialogView>
   {}
   
   /** The underlined modal dialog */
   @UiField
   Modal modal;
   
   /** The Title span */
   @UiField
   Span title;
   
   /** The Logo image */
   @UiField
   Image logo;
   
   /** The About paragraph */
   @UiField
   Paragraph about;
   
   /** The Close button */
   @UiField
   Button closeAction;
   
   /**
    * Creates the About Dialog View.
    * 
    * @param    binder
    *           The injected GWT UI Creator
    */
   @Inject
   public AboutDialogView(Binder binder)
   {
      initWidget(binder.createAndBindUi(this));
   }

   /**
    * Provides an access to the target modal dialog.
    * 
    * @return   The underlined modal dialog
    */
   @Override
   protected Modal getModal()
   {
      return modal;
   }

   /**
    * Sets the title for this dialog.
    * 
    * @param    text
    *           The dialog title
    */
   public void setTitle(String text)
   {
      this.title.setText(text);
   }
   
   /**
    * Sets an application logo image.
    * 
    * @param    url
    *           The url to the target logo.
    */
   public void setLogo(String url)
   {
      this.logo.setUrl(url);
   }
   
   /**
    * Sets an application details.
    * 
    * @param    about
    *           The text about the target application
    */
   public void setAbout(String about)
   {
      this.about.setText(about);
   }
   
   /**
    * Sets an application details.
    * 
    * @param    html
    *           The html text about the target application
    */
   public void setAboutHtml(String html)
   {
      this.about.setHTML(html);
   }
}
