/*
** Module   : PresenterWithStateMachine.java
** Abstract : PresenterWithStateMachine is a base controller that implements change and restore
**            view states methods.
**
** Copyright (c) 2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- ----------------Description----------------------
** 001 SBI 20170601 Created initial version.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/


package com.goldencode.p2j.admin.client.application.home;

import java.util.Arrays;
import java.util.logging.Level;

import com.google.gwt.event.shared.GwtEvent;
import com.google.web.bindery.event.shared.EventBus;
import com.gwtplatform.mvp.client.View;
import com.gwtplatform.mvp.client.proxy.PlaceManager;
import com.gwtplatform.mvp.client.proxy.Proxy;
import com.gwtplatform.mvp.client.proxy.RevealContentHandler;


/**
 * PresenterWithStateMachine is a base controller that implements change and restore view's states
 * methods.
 *
 * @param    <V>
 *           A view type
 * @param    <Proxy_>
 *           A controller's proxy type
 */
public abstract class PresenterWithStateMachine<V extends View, Proxy_ extends Proxy<?>>
extends BasePresenter<V, Proxy_>
{
   /** The current view state machine associated with this view presenter */
   private final ViewStateMachine viewStateMachine;
   
   /** The saved state */
   private ViewStateMachine.State savedState;
   
   
   /**
    * Constructs this controller based on the given data.
    * 
    * @param    eventBus
    *           The event bus
    * @param    view
    *           The associated view
    * @param    proxy
    *           The associated controller's proxy
    * @param    slot
    *           The slot
    * @param    placeManager
    *           The palce manager
    * @param    viewStateMachine
    *           The view state machine
    */
   public PresenterWithStateMachine(EventBus eventBus, V view, Proxy_ proxy,
            GwtEvent.Type<RevealContentHandler<?>> slot,
            PlaceManager placeManager,
            ViewStateMachine viewStateMachine)
   {
      super(eventBus, view, proxy, slot, placeManager);
      this.viewStateMachine = viewStateMachine;
   }
   
   /**
    * Change a current view according to new provided transition state.
    * 
    * @param    transitionState
    *           The new transition state
    */
   public void changeState(ViewStateMachine.TransitionState transitionState)
   {
      this.savedState = this.viewStateMachine.apply(transitionState);
      logger.log(Level.INFO, transitionState + " -> " + Arrays.toString(this.savedState.getState()));
   }
   
   /**
    * Setups a current view's state.
    * 
    * @param    state
    *           The new view's state.
    */
   public void setState(ViewStateMachine.State state)
   {
      this.viewStateMachine.init(state);
      logger.log(Level.INFO, "setState -> " + state + " " + Arrays.toString(state.getState()));
   }
   
   /**
    * Restores the current state.
    */
   public void restoreState()
   {
      if (this.savedState != null)
      {
         this.viewStateMachine.init(this.savedState);
         logger.log(Level.INFO, "restoreState -> " + Arrays.toString(this.savedState.getState()));
      }
   }
}
