/*
** Module   : AccountsPresenter.java
** Abstract : AccountsPresenter represents a based controller that shares common methods to be
**            used with system accounts.
**
** Copyright (c) 2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- ----------------Description----------------------
** 001 SBI 20170601 Created initial version.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.admin.client.application.home.accounts;

import com.goldencode.p2j.admin.TaggedName;
import com.goldencode.p2j.admin.client.application.home.PresenterWithStateMachine;
import com.goldencode.p2j.admin.client.application.home.ViewStateMachine;
import com.goldencode.p2j.admin.client.application.home.ViewStateMachine.TransitionState;
import com.goldencode.p2j.admin.client.application.home.acl.*;
import com.google.gwt.event.shared.GwtEvent.Type;
import com.google.web.bindery.event.shared.EventBus;
import com.gwtplatform.mvp.client.View;
import com.gwtplatform.mvp.client.proxy.PlaceManager;
import com.gwtplatform.mvp.client.proxy.Proxy;
import com.gwtplatform.mvp.client.proxy.RevealContentHandler;

/**
 * AccountsPresenter represents a based controller that shares common methods to be used
 * with system accounts.
 *
 * @param    <V>
 *           An MPV view class
 * @param    <Proxy_>
 *           An MPV proxy class
 */
public abstract class AccountsPresenter<V extends View, Proxy_ extends Proxy<?>>
extends PresenterWithStateMachine<V, Proxy_>
{
   private ACLPopupPresenter aclPopupPresenterProvider;

   /**
    * Creates this controller.
    * 
    * @param    eventBus
    *           The event bus
    * @param    view
    *           The associated view
    * @param    proxy
    *           The associated controller's proxy
    * @param    slot
    *           The slot
    * @param    placeManager
    *           The palce manager
    * @param    viewStateMachine
    *           The view state machine
    */
   public AccountsPresenter(EventBus eventBus,
                            V view,
                            Proxy_ proxy,
                            Type<RevealContentHandler<?>> slot,
                            PlaceManager placeManager,
                            ViewStateMachine viewStateMachine,
                            ACLPopupPresenter aclPopupPresenterProvider)
   {
      super(eventBus, view, proxy, slot, placeManager, viewStateMachine);
      this.aclPopupPresenterProvider = aclPopupPresenterProvider;
   }

   /**
    * Switch to ACL view opened for the target account.
    *  
    * @param    account
    *           The target account
    */
   public void switchToACLs(TaggedName account)
   {
      aclPopupPresenterProvider.setFilter(null, account.getName(), null, null);
      addToPopupSlot(aclPopupPresenterProvider);
      aclPopupPresenterProvider.show();
   }
   
   /**
    * Sets a new view state.
    * 
    * @param    transitionState
    *           A new view state
    */
   public void setCurrentState(TransitionState transitionState)
   {
      changeState(transitionState);
   }
}
