/*
** Module   : GroupsUIHandlers.java
** Abstract : GroupsUIHandlers represents a common interface to be used to manage groups.
**
** Copyright (c) 2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- ----------------Description----------------------
** 001 SBI 20170601 Created initial version.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.admin.client.application.home.accounts.groups;

import java.util.Set;

import com.goldencode.p2j.admin.GroupDef;
import com.goldencode.p2j.admin.TaggedName;
import com.goldencode.p2j.admin.client.application.home.accounts.AccountsUiHandlers;

/**
 * Represents a common interface to be used to manage groups.
 */
public interface GroupsUIHandlers extends AccountsUiHandlers
{
   /**
    * Populates the all users table if it has been empty or updates its rows with the current data. 
    */
   void populateAndRefreshAllUsers();
   
   /**
    * Retrieves the custom authentication plugins data and fills the custom authentication plugins
    * combobox.
    */
   void updateCustomAuthPlugins();
   
   /**
    * Performs the user action to refresh the all groups table.
    */
   void onRefresh();
   
   /**
    * Triggers the groups selection changed.
    * 
    * @param    selectedGroups
    *           The new set of selected groups
    */
   void onGroupSelected(Set<TaggedName> selectedGroups);
   
   /**
    * Triggers the users selection changed.
    * 
    * @param    selectedUsers
    *           The new set of selected users
    */
   void onUserSelected(Set<TaggedName> selectedUsers);
   
   /**
    * Adds new group to the group accounts represented by the all groups table.
    * 
    * @param    newGroup
    *           The new group definition object representing the new adding group
    */
   void addGroup(GroupDef newGroup);
   
   /**
    * Delete selected groups given by the array of group tagged names
    * 
    * @param    groups
    *           The array of groups to delete
    */
   void deleteSelectedGroups(TaggedName[] groups);
   
   /**
    * Clones the given group by creating the new one.
    * 
    * @param    nameFrom
    *           The name of the group to clone
    * @param    nameTo
    *           The name for the new cloned group
    * @param    acls
    *           True indicates that the ACL of target group will be cloned too, otherwise
    *           the ACL of the new created group will be empty.
    */
   void cloneGroup(String nameFrom, String nameTo, boolean acls);
   
   /**
    * Commits changes in the target group definition.
    * 
    * @param    updatedGroup
    *           The group definition to commit its changes
    */
   void updateGroup(GroupDef updatedGroup);
   
   /**
    * Removes the selected users from the given group.
    * 
    * @param    group
    *           The group tagged name
    * @param    users
    *           The array of selected users given by their tagged names
    */
   void unassignSelectedUsers(TaggedName group, TaggedName[] users);
   
   /**
    * Adds a user to the given group.
    * 
    * @param    group
    *           The group tagged name
    * @param    user
    *           The user account name
    */
   void addUserToGroup(TaggedName group, String user);
   
   /**
    * Sets the target user search template.
    * 
    * @param    filter
    *           The target search template
    */
   void setUsersFilter(String filter);
   
   /**
    * Adds the selected users to the given group.
    * 
    * @param    group
    *           The group tagged name
    * @param    selectedUsers
    *           The array of selected users given by their tagged names
    */
   void addUsersToGroup(TaggedName group, Set<TaggedName> selectedUsers);
   
   /**
    * Setups the Add Group Dialog.
    */
   void setupAddGroupDialog();
   
   /**
    * Setups the Deleting Groups Alert dialog.
    * 
    * @param    groups
    *           The array of the selected groups to delete
    */
   void setupDeletingGroupsAlert(TaggedName[] groups);
   
   /**
    * Setups the Clone Group Dialog.
    * 
    * @param    clonedGroup
    *           The target group to clone
    */
   void setupCloneGroupDialog(TaggedName clonedGroup);
   
   /**
    * Setups the Edit Group Dialog.
    * 
    * @param    updatedGroup
    *           The group definition to edit
    */
   void setupEditGroupDialog(TaggedName updatedGroup);
   
   /**
    * Setups the Unassigning Users Alert dialog.
    * 
    * @param    group
    *           The group tagged name
    * @param    users
    *           The array of selected users given by their tagged names
    */
   void setupUnassigningUsersAlert(TaggedName group, TaggedName[] users);

   /**
    * Cancels the Add Group Dialog.
    */
   void cancelAddGroupDialog();
   
   /**
    * Cancels the Deleting Groups Alert dialog.
    */
   void cancelDeletingGroupsAlert();
   
   /**
    * Cancels the Clone Group Dialog.
    */
   void cancelCloneGroupDialog();
   
   /**
    * Cancels the Edit Group Dialog.
    */
   void cancelEditGroupDialog();
   
   /**
    * Cancels the Unassigning Users Alert dialog.
    */
   void cancelUnassigningUsersAlert();
}
