/*
** Module   : CloneUserAccountView.java
** Abstract : CloneUserAccountView is a view to clone a selected user account.
**
** Copyright (c) 2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- ----------------Description----------------------
** 001 SBI 20170601 Created initial version.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/


package com.goldencode.p2j.admin.client.application.home.accounts.users;

import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;

import org.gwtbootstrap3.client.ui.Button;
import org.gwtbootstrap3.client.ui.CheckBox;
import org.gwtbootstrap3.client.ui.Modal;
import org.gwtbootstrap3.client.ui.Radio;
import org.gwtbootstrap3.client.ui.TextBox;
import org.gwtbootstrap3.client.ui.form.error.BasicEditorError;
import org.gwtbootstrap3.client.ui.form.validator.BlankValidator;
import org.gwtbootstrap3.client.ui.form.validator.Validator;

import com.goldencode.p2j.admin.TaggedName;
import com.goldencode.p2j.admin.client.application.home.accounts.PasswordOptions;
import com.goldencode.p2j.admin.client.widget.EnumRadioGroup;
import com.goldencode.p2j.admin.client.widget.dialog.ModalViewWithUiHandlers;
import com.google.gwt.dom.client.Document;
import com.google.gwt.editor.client.Editor;
import com.google.gwt.editor.client.EditorError;
import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.KeyCodes;
import com.google.gwt.event.dom.client.KeyPressEvent;
import com.google.gwt.event.logical.shared.ValueChangeEvent;
import com.google.gwt.event.logical.shared.ValueChangeHandler;
import com.google.gwt.uibinder.client.UiBinder;
import com.google.gwt.uibinder.client.UiField;
import com.google.gwt.uibinder.client.UiHandler;
import com.google.gwt.user.client.ui.Widget;
import com.google.inject.Inject;

/**
 * CloneUserAccountView is a view to clone a selected user account.
 */
public class CloneUserAccountView
extends ModalViewWithUiHandlers<UsersUIHandlers>
{
   /**
    * GWT UI creator
    */
   interface Binder
   extends UiBinder<Widget, CloneUserAccountView>
   {}
   
   /** The underlined modal dialog */
   @UiField
   Modal modal;
   
   /** The account name text field */
   @UiField
   TextBox name;
   
   /** The With ACLs check box */
   @UiField
   CheckBox acl;

   /** The password options to clone the existing or to setup new one*/
   @UiField
   EnumRadioGroup<PasswordOptions> passwordOptionsSelector;
   
   /** The New Password text box */
   @UiField(provided = true)
   TextBox password =  new TextBox(Document.get().createPasswordInputElement());
   
   /** The Confirm New Password text box */ 
   @UiField(provided = true)
   TextBox password2 =  new TextBox(Document.get().createPasswordInputElement());
   
   /** The Clone account button */
   @UiField
   Button cloneAction;
   
   /** The Cancel button to discard any changes and to close this dialog */
   @UiField
   Button cancelAction;

   /** The existing cloning user account given by its tagged name */
   private TaggedName userTaggedName;
   
   /**
    * Constructs this view, used by MPV gwtplatform of ArcBees Inc.
    * 
    * @param    binder
    *           GWT UI creator
    */
   @Inject
   public CloneUserAccountView(CloneUserAccountView.Binder binder)
   {
      initWidget(binder.createAndBindUi(this));
      passwordOptionsSelector.addValueChangeHandler(new ValueChangeHandler<PasswordOptions>()
      {
         @Override
         public void onValueChange(ValueChangeEvent<PasswordOptions> event)
         {
            PasswordOptions popt = event.getValue();
            switch(popt)
            {
               case CLONE_PASSWORD:
                  password.setEnabled(false);
                  password2.setEnabled(false);
                  //clear warnings if there exist.
                  password.clear();
                  password2.clear();
                  password.validate(true);
                  password2.validate(true);
                  break;
               case NEW_PASSWORD:
                  password.setEnabled(true);
                  password2.setEnabled(true);
                  break;
               default:
            }
         }
      });
      
      password.addValidator(new BlankValidator<String>(UsersUIHandlers.BLANK_FIELD_WARNING)
                           {
                              @Override
                              public boolean isValid(String value)
                              {
                                 PasswordOptions popt = passwordOptionsSelector.getValue();
                                 
                                 if (popt == PasswordOptions.CLONE_PASSWORD)
                                 {
                                    return true;
                                 }
                                 
                                 return super.isValid(value);
                              }
                           });
      
      password2.addValidator(
               new Validator<String>()
               {

                  @Override
                  public int getPriority()
                  {
                     return Priority.HIGHEST;
                  }
         
                  @Override
                  public List<EditorError> validate(Editor<String> editor, String value)
                  {
                     List<EditorError> result = new ArrayList<EditorError>();
                     PasswordOptions popt = passwordOptionsSelector.getValue();
                     if (popt == PasswordOptions.CLONE_PASSWORD)
                     {
                        return result;
                     }
                     
                     if (!password.validate(true))
                     {
                        return result;
                     }
                     
                     String valueStr = value == null ? "" : value.toString();
                     
                     if (!valueStr.equals(password.getValue()))
                     {
                       result.add(new BasicEditorError(password2,
                                                       value,
                                                       UsersUIHandlers.CONFIRM_PASSWORD_WARNING));
                     }
                     
                     return result;
                  }
               });
      
      password.setValidateOnBlur(true);
      password2.setValidateOnBlur(true);
   }

   /**
    * The Clone button click handler.
    * 
    * @param    e
    *           Click event
    */
   @UiHandler("cloneAction")
   void onCloneAction(ClickEvent e)
   {
      logger.log(Level.INFO, "onCloneAction");
      if (passwordOptionsSelector.getValue() == PasswordOptions.CLONE_PASSWORD)
      {
         getUiHandlers().cloneUserAccount(userTaggedName.getName(), name.getText(), acl.getValue());
      }
      else
      {
         if (!password.validate() || !password2.validate())
         {
            e.preventDefault();
            e.stopPropagation();
         }
         
         getUiHandlers().cloneUserAccount(userTaggedName.getName(), name.getText(), acl.getValue(),
                  password.getValue());
      }
   }
   
   /**
    * The Cancel button click handler.
    * 
    * @param    e
    *           Click event
    */
   @UiHandler("cancelAction")
   void onCancelAction(ClickEvent e)
   {
      logger.log(Level.INFO, "onCancelAction");
      getUiHandlers().cancelCloneUserAccountDialog();
   }
   
   /**
    * Setups this dialog.
    * 
    * @param    userTaggedName
    *           The tagged name of the cloning user account
    */
   public void setupCloneUserAccountView(TaggedName userTaggedName)
   {
      this.userTaggedName = userTaggedName;
      // initial value
      name.setText("");
      acl.setValue(false);
      Radio radio = (Radio) passwordOptionsSelector.getWidget(PasswordOptions.NEW_PASSWORD.ordinal());
      radio.setValue(true, true);
   }
   
   /**
    * Key press event handler.
    *
    * @param   event
    *          Key press event.
    */
   @UiHandler("password")
   public void passwordKeyPress(KeyPressEvent event)
   {
      if(event.getNativeEvent().getKeyCode() == KeyCodes.KEY_ENTER)
      {
         password2.setFocus(true);
      }
   }

   /**
    * Provides an access to the target modal dialog.
    * 
    * @return   The underlined modal dialog
    */
   @Override
   protected Modal getModal()
   {
      return modal;
   }
}
