/*
** Module   : LoginView.java
** Abstract : LoginView represents a login form view.
**
** Copyright (c) 2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- ----------------Description----------------------
** 001 HC  20170228 Created initial version.
** 002 SBI 20170531 Changed text controls to use gwtbootstrap3, fixed "Enter pressed" handlers.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.admin.client.application.login;

import org.gwtbootstrap3.client.ui.Button;
import org.gwtbootstrap3.client.ui.TextBox;

import com.goldencode.p2j.security.*;
import com.google.gwt.dom.client.Document;
import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.KeyCodes;
import com.google.gwt.event.dom.client.KeyPressEvent;
import com.google.gwt.uibinder.client.UiBinder;
import com.google.gwt.uibinder.client.UiField;
import com.google.gwt.uibinder.client.UiHandler;
import com.gwtplatform.mvp.client.ViewWithUiHandlers;
import com.google.gwt.user.client.Window;
import com.google.gwt.user.client.ui.Widget;
import com.google.inject.Inject;

/**
 * Login view.
 */
public class LoginView
extends ViewWithUiHandlers<LoginUiHandlers>
implements LoginPresenter.MyView
{
   /**
    * GWT binder.
    */
   interface Binder
   extends UiBinder<Widget, LoginView>
   {}

   /** Widget */
   @UiField
   TextBox userid;

   /** Widget */
   @UiField(provided = true)
   TextBox pwd = new TextBox(Document.get().createPasswordInputElement());

   /** Widget */
   @UiField
   Button submit;

   /**
    * Ctor.
    *
    * @param   binder
    *          GWT binder.
    */
   @Inject
   public LoginView(Binder binder)
   {
      initWidget(binder.createAndBindUi(this));
   }

   /**
    * Displays login error to the user.
    *
    * @param   code
    *          The error code.
    */
   @Override
   public void reportLoginError(int code)
   {
      StringBuilder sb = new StringBuilder();
      sb.append("Login failed.");

      switch (code)
      {
         case SecurityConstants.AUTH_RESULT_INVALID_USERID:
         case SecurityConstants.AUTH_RESULT_INVALID_PASSWORD:
            sb.append(" Invalid user name or password.");
            break;
         case SecurityConstants.AUTH_RESULT_INSUFFICIENT_RIGHTS:
            sb.append(" Insufficient account rights.");
            break;
      }

      Window.alert(sb.toString());
   }

   /**
    * Report unspecified login failure to the user.
    */
   @Override
   public void reportFailure()
   {
      Window.alert("Unspecified login failure, please try again later!");
   }

   /**
    * Clears the password input box.
    */
   @Override
   public void clearPassword()
   {
      pwd.setValue("");
   }

   /**
    * Click event handler.
    *
    * @param   e
    *          Click event.
    */
   @UiHandler("submit")
   public void onClick(ClickEvent e)
   {
      login();
   }

   /**
    * Key press event handler.
    *
    * @param   event
    *          Key press event.
    */
   @UiHandler("userid")
   public void useridKeyPress(KeyPressEvent event)
   {
      if(event.getNativeEvent().getKeyCode() == KeyCodes.KEY_ENTER)
      {
         pwd.setFocus(true);
      }
   }

   /**
    * Key press event handler.
    *
    * @param   event
    *          Key press event.
    */
   @UiHandler("pwd")
   public void pwdKeyPress(KeyPressEvent event)
   {
      if(event.getNativeEvent().getKeyCode() == KeyCodes.KEY_ENTER)
      {
         login();
      }
   }

   /**
    * Method called after the view is attached to the DOM.
    * <p/>
    * You should override this method to perform any ui related initialization that needs to be
    * done after that the
    * view is attached <b>and that the presenter doesn't have to be aware of</b> (attach event
    * handlers for instance)
    */
   @Override
   protected void onAttach()
   {
      super.onAttach();
      userid.setFocus(true);
   }

   /**
    * Proceeds with login.
    */
   private void login()
   {
      getUiHandlers().onSubmitLoginInfo(userid.getValue(), pwd.getValue());
   }
}
