/*
** Module   : RightsEditorContext.java
** Abstract : Rights editor context.
**
** Copyright (c) 2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- --------------------------------Description-----------------------------------
** 001 HC  20170612 Initial version.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
**
** Additional terms under GNU Affero GPL version 3 section 7:
**
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
**
**   0. Attribution Requirement.
**
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
**
**   1. No License To Use Trademarks.
**
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
**
**   2. No Misrepresentation of Affiliation.
**
**     You may not represent yourself as Golden Code Development Corporation or FWD.
**
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
**
**   3. No Misrepresentation of Source or Origin.
**
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/
package com.goldencode.p2j.admin.client.editors;

import com.goldencode.p2j.admin.client.widget.dialog.*;
import com.goldencode.p2j.security.*;
import com.google.gwt.user.client.ui.*;

import java.util.*;

/**
 * The class holds contextual information used by the rights editors, see {@link RightsEditor}.
 */
public class RightsEditorContext
{
   /** The singleton instance */
   private static RightsEditorContext instance;

   /** The map of registered editors, the map key is the resource type name */
   private Map<String, RightsEditor> editors = new HashMap<>();

   /** The input dialog the rights editors may use for their UI */
   private InputDialog inputDialog;

   /** The parent widget the rights editors may use for their UI */
   private HasWidgets parent;

   /** The modal dialog UI service the rights editors may use for their UI */
   private ModalDialogs modalDialogs;

   /**
    * Default ctor.
    */
   private RightsEditorContext()
   {
   }

   /**
    * Singleton creator method.
    *
    * @return  returns the singleton instance
    */
   public static RightsEditorContext instance()
   {
      if (instance == null)
      {
         instance = new RightsEditorContext();
      }

      return instance;
   }

   /**
    * Returns the registered editor instance for a particular resource type.
    *
    * @param   resourceType
    *          Resource type name.
    *
    * @return  See above
    */
   public RightsEditor getEditor(String resourceType)
   {
      return editors.get(resourceType);
   }

   /**
    * Registers an editor for a particular resource type.
    *
    * @param   resourceType
    *          Resource type name.
    * @param   editor
    *          The editor being registered.
    *
    * @return  this instance
    */
   public RightsEditorContext registerEditor(String resourceType, RightsEditor editor)
   {
      editors.put(resourceType, editor);
      return this;
   }

   /**
    * Returns the input dialog the rights editors may use for their UI.
    *
    * @return  see above
    */
   public InputDialog getInputDialog()
   {
      return inputDialog;
   }

   /**
    * Sets the input dialog the rights editors may use for their UI.
    *
    * @param   inputDialog
    *          {@link InputDialog} instance.
    */
   public void setInputDialog(InputDialog inputDialog)
   {
      this.inputDialog = inputDialog;
   }

   /**
    * Returns the parent widget the rights editors may use for their UI.
    *
    * @return  see above
    */
   public HasWidgets getParent()
   {
      return parent;
   }

   /**
    * Sets the parent widget the rights editors may use for their UI.
    *
    * @param   parent
    *          Parent widget.
    */
   public void setParent(HasWidgets parent)
   {
      this.parent = parent;
   }

   /**
    * Returns the modal dialog UI service the rights editors may use for their UI.
    *
    * @return  see above
    */
   public ModalDialogs getModalDialogs()
   {
      return modalDialogs;
   }

   /**
    * Sets the modal dialog UI service the rights editors may use for their UI.
    *
    * @param   modalDialogs
    *          Modal dialog service.
    */
   public void setModalDialogs(ModalDialogs modalDialogs)
   {
      this.modalDialogs = modalDialogs;
   }
}

