/*
** Module   : ReportParameter.java
** Abstract : ReportParameter defines parameters that can be requested to build reports.
**
** Copyright (c) 2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- ----------------Description----------------------
** 001 SBI 20170601 Created initial version.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/


package com.goldencode.p2j.admin.shared;


/**
 * ReportParameter defines parameters that can be requested to build reports.
 */
public enum ReportParameters
{
   
   /** Defines a report's title. */
   TITLE("title"),
   
   /** Defines a data set selected for a report. */
   SELECTION("selection"),
   
   /** Defines a report's id. */
   REPORT_ID("report"),
   
   /** Defines a report's paper size. */
   PAPER_FORMAT("format"),
   
   /** Defines a report's page orientation. */ 
   PAPER_ORIENT("orient"),
   
   /** Defines a string that is used to select a subset of data for a report. */
   FILTER("filter"),
   
   /** Defines a prefix for additional parameters provided by report's extensions. */
   EXTENSION("ext"),
   
   /** Defines a report's type. */
   REPORT_TYPE("type"),
   
   /** Defines a page to preview if this parameter is present. */
   PREVIEW_PAGE("page"),
   
   /** Defines detailed reports if this parameter is present in the report request. */
   DETAILS("details"),
   
   /** Defines auto printed reports if this parameter is present in the report request. */
   ACTION("print"),
   
   /** The maximal pages number in reports */
   MAX_PAGES_NUMBER("max-pages"),
   
   /** The left page margin given in points */
   LEFT_PAGE_MARGIN("margin-left"),
   
   /** The right page margin given in points */
   RIGHT_PAGE_MARGIN("margin-right"),
   
   /** The top page margin given in points */
   TOP_PAGE_MARGIN("margin-top"),
   
   /** The bottom page margin given in points */
   BOTTOM_PAGE_MARGIN("margin-bottom"),
   
   /** The cell margin given in points */
   CELL_MARGIN("cell-margin"),
   
   /** The cell padding given in points */
   CELL_PADDING("cell-padding"),
   
   /** The header margin given in points */
   HEADER_MARGIN("header-margin"),
   
   /** The footer margin given in points */
   FOOTER_MARGIN("footer-margin"),
   
   /** The default cell font size */
   FONT_SIZE("font-size"),
   
   /** The default column font size */
   COLUMN_FONT_SIZE("column-font-size"),
   
   /** The default header font size */
   HEADER_FONT_SIZE("header-font-size"),
   
   /** The default footer font size */
   FOOTER_FONT_SIZE("footer-font-size"),
   
   /** The default font name */
   FONT_NAME("font"),
   
   /** The default bold font name */
   BOLD_FONT_NAME("bold-font"),
   
   ;
   
   /** The parameter name */
   private final String parameter;

   /**
    * Builds report parameter.
    *
    * @param    parameter
    *           The parameter
    */
   private ReportParameters(String parameter)
   {
      this.parameter = parameter;
   }

   /**
    * Gets the parameter.
    *
    * @return   The parameter
    */
   public String getParameter()
   {
      return parameter;
   }
}
