/*
** Module   : E4GLConstants.java
** Abstract : shared constants for E4GL processing
**
** Copyright (c) 2007-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- -T- --JPRM-- ----------------Description-----------------
** 001 GES 20070822 NEW   @34965 Shared constants for E4GL processing.
*/ 
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.e4gl;

/**
 * Shared constants for Embedded 4GL processing.
 */
public interface E4GLConstants
{
   /** Unknown compatibility mode. */  
   public static final int UNKNOWN_MODE = -1;
   
   /** WebSpeed compatibility mode. */  
   public static final int WEBSPEED = 0;
   
   /** WebSpeed strict compatibility mode. */  
   public static final int WEBSPEED_STRICT = 1;
   
   /** Blue Diamond compatibility mode. */  
   public static final int BLUE_DIAMOND = 2;
   
   /** Default compatibility mode. */  
   public static final int DEFAULT_MODE = WEBSPEED;
   
   /** Maps compatibility modes into a descriptive name. */
   public static final String[] MODE_NAMES =
   {
      "WebSpeed",
      "WebSpeed (Strict)",
      "Blue Diamond"
   };
   
   /** Option forcing include file style output. */
   public static final String OPT_INCLUDE = "include";
   
   /** Option forcing procedure/web-object file style output. */
   public static final String OPT_WEB_OBJ = "web-object";
   
   /** Option to avoid the commenting of META Content-Type elements. */
   public static final String OPT_KEEP = "keep-meta-content-type";
   
   /** Option to specify that HTML META elements contained options. */
   public static final String OPT_FOUND = "wsoptions-found";
}