/*
** Module   : InitTermListener.java
** Abstract :
**
** Copyright (c) 2009-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- --JPRM-- ----------------Description-----------------
** 001 SIY 20090507  @42110  Created initial version
** 002 SIY 20090511  @42140  Renamed to InitTermListener.
**
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.main;

/**
 * Classes which implement this interface may serve as per-session or server
 * hooks.
 * 
 * <p>
 * Once they are properly described in the directory instances of these
 * classes will be created on every successful client login. Then method
 * {@link #initialize()} is invoked before any business logic is executed.
 * Method {@link #terminate(Throwable)} is invoked when business logic has
 * been finished. {@link #terminate(Throwable)} is invoked regardless from the
 * cause of termination. The cause of business logic termination can be
 * determined by analyzing instance of {@link Throwable} passed to the
 * {@link #terminate(Throwable)} as a parameter. In some cases it can be
 * <code>null</code> and this can be considered as a normal exit, but even if
 * it is not <code>null</code> this not necessarily means that business logic
 * has been finished abnormally.
 * 
 * @version 1.0
 */
public interface InitTermListener
{
   /**
    * This method is invoked when business logic main entry point is about to
    * be invoked.
    */
   public void initialize();

   /**
    * This method is invoked when business logic entry point finished its
    * work.
    * 
    * @param    t
    *           Error or exception which caused exit from business logic. Can
    *           be <code>null</code>.
    */
   public void terminate(Throwable t);
}