/*
** Module   : WebClientConfig.java
** Abstract : WebClientConfig represents the web client resources.
**
** Copyright (c) 2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- ----------------Description----------------------
** 001 SBI 20170628 Created initial version.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/


package com.goldencode.p2j.main;

import java.io.Serializable;
import java.util.concurrent.atomic.*;

/**
 * Represents the web client resources.
 */
public class WebClientConfig
implements Serializable 
{
   /** Number of sessions opened using this web client. */
   private AtomicLong sessionsCounter; 
   
   /** web client id */
   private String uuid;
   
   /**
    * The direct host assigned to launch the spawned web client
    */
   private String host;
   
   /**
    * The direct port assigned for the spawned web client
    */
   private int port;
   
   /**
    * The web root prefix for all resources of the web client
    */
   private String webRoot;

   /**
    * The forwarded host
    */
   private String forwardedHost;
   
   /**
    * The http/https protocol used by the forwarded host
    */
   private  String forwardedProto;
   
   /**
    * Returns a web client unique identifier.
    * 
    * @return   The web client id
    */
   public String getUuid()
   {
      return uuid;
   }

   /**
    * Sets a web client unique identifier.
    * 
    * @param    uuid
    *           A unique identifier
    */
   public void setUuid(String uuid)
   {
      this.uuid = uuid;
   }

   /**
    * The host on which this web client is running
    * 
    * @return   The web client host
    */
   public String getHost()
   {
      return host;
   }

   /**
    * Gets the web client port number.
    * 
    * @return   The web client port number.
    */
   public int getPort()
   {
      return port;
   }

   /**
    * Gets the web client root.
    * 
    * @return   The web client root
    */
   public String getWebRoot()
   {
      return webRoot;
   }

   /**
    * Gets the forwarded host.
    * 
    * @return   The forwarded host
    */
   public String getForwardedHost()
   {
      return forwardedHost;
   }

   /**
    * Gets the forwarded host protocol.
    * 
    * @return   The forwarded host protocol
    */
   public String getForwardedProto()
   {
      return forwardedProto;
   }

   /**
    * Sets the web client host.
    * 
    * @param    host
    *           The host on which this web client is running
    */
   public void setHost(String host)
   {
      this.host = host;
   }

   /**
    * Sets the web client port number.
    * 
    * @param    port
    *           The web client port number
    */
   public void setPort(int port)
   {
      this.port = port;
   }

   /**
    * Sets the web client root
    * 
    * @param    webRoot
    *           The web client root
    */
   public void setWebRoot(String webRoot)
   {
      this.webRoot = webRoot;
   }

   /**
    * Sets the forwarded host.
    * 
    * @param    forwardedHost
    *           The forwarded host
    */
   public void setForwardedHost(String forwardedHost)
   {
      this.forwardedHost = forwardedHost;
   }

   /**
    * Sets the forwarded host protocol
    * 
    * @param    forwardedProto
    *           The forwarded host protocol
    */
   public void setForwardedProto(String forwardedProto)
   {
      this.forwardedProto = forwardedProto;
   }

   /**
    * Counts sessions number opened by this web client.
    * 
    * @return   The session counter.
    */
   public AtomicLong getSessionsCounter()
   {
      return sessionsCounter;
   }

   /**
    * Sets the current session counter that is used by this web client.
    * 
    * @param    sessionsCounter
    *           The current session counter
    */
   public void setSessionsCounter(AtomicLong sessionsCounter)
   {
      this.sessionsCounter = sessionsCounter;
   }
}

