/*
** Module   : Match.java
** Abstract : represents a match of a specific condition in a specific file
**            for use in calculating statistics
**
** Copyright (c) 2005-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- --JPRM-- -------------------Description--------------------
** 001 GES 20050225   @20122 Created initial version supporting the
**                           storage of a line number, column number and
**                           arbitrary text.
** 002 GES 20130206          Moved class to the pattern package.
*/ 
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.pattern;

/**
 * A simple container class to store and access file and text related  
 * information about a single specific match to a condition within a specific
 * file.
 * <p>
 * This supports the storage of a line number, column number and a string.
 *
 * @author    GES
 * @version   1.0.0
 */
public class Match
{   
   /** Line number of the beginning of the match in the file. */
   private int line = 0;
   
   /** Column number of the beginning of the match in the file. */
   private int column = 0;
   
   /** The user-defined text of the match. */
   private String text = null;   
   
   /**
    * Constructs an instance with user-defined data.
    *
    * @param    line
    *           The line number of the beginning of the match in the file.
    *
    * @param    column
    *           The column number of the beginning of the match in the file.
    *
    * @param    text
    *           Stores the text of the match.
    */
   public Match(int line, int column, String text)
   {
      this.line   = line;
      this.column = column;
      this.text   = text;
   }
   
   /**
    * Accesses the line number of the beginning of the match in the file.
    *
    * @return   The line number of the beginning of the match in the file.
    */
   public int getLine()
   {
      return line;
   }
   
   /**
    * Sets the line number of the beginning of the match in the file.
    *
    * @param    line
    *           The line number of the beginning of the match in the file.
    */
   public void setLine(int line)
   {
      this.line = line;
   }
   
   /**
    * Accesses the column number of the beginning of the match in the file.
    *
    * @return   The column number of the beginning of the match in the file.
    */
   public int getColumn()
   {
      return column;
   }
   
   /**
    * Sets the column number of the beginning of the match in the file.
    *
    * @param    column
    *           The column number of the beginning of the match in the file.
    */
   public void setColumn(int column)
   {
      this.column = column;
   }
   
   /**
    * Accesses the user-defined text for this match.
    *
    * @return   The user-defined text for this match.
    */
   public String getText()
   {
      return text;
   }
   
   /**
    * Sets the  user-defined text for this match.
    *
    * @param    text
    *           The user-defined text for this match.
    */
   public void setText(String text)
   {
      this.text = text;
   }   
}