/*
** Module   : TreeWalkException.java
** Abstract : Exception information for an error encountered during an AST walk.
**
** Copyright (c) 2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- --------------------------------Description---------------------------------
** 001 ECF 20170607 Created first version.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.pattern;

/**
 * Exception type for an error which occurs during a {@link PatternEngine} tree walk.
 */
public class TreeWalkException
extends RuntimeException
{
   /** ID of the source AST */
   private Long sourceAstId = null;
   
   /** ID of the copy AST */
   private Long copyAstId = null;
   
   /** Name of the file associated with the tree being walked */
   private String filename = null;
   
   /**
    * Constructor.
    * 
    * @param   message
    *          Error message.
    * @param   thr
    *          Root cause.
    */
   public TreeWalkException(String message, Throwable thr)
   {
      super(message, thr);
   }
   
   /**
    * Source AST ID accessor.
    * 
    * @return  ID of the source AST.
    */
   public Long getSourceAstId()
   {
      return sourceAstId;
   }
   
   /**
    * Source AST ID mutator.
    * 
    * @param   sourceAstId
    *          The source AST ID to set.
    */
   public void setSourceAstId(Long sourceAstId)
   {
      this.sourceAstId = sourceAstId;
   }
   
   /**
    * Copy AST ID accessor.
    * 
    * @return  ID of the copy AST.
    */
   public Long getCopyAstId()
   {
      return copyAstId;
   }
   
   /**
    * Copy AST ID mutator.
    * 
    * @param   copyAstId
    *          The copy AST ID to set.
    */
   public void setCopyAstId(Long copyAstId)
   {
      this.copyAstId = copyAstId;
   }
   
   /**
    * Associated file name accessor.
    * 
    * @return  File name associated with the AST.
    */
   public String getFilename()
   {
      return filename;
   }
   
   /**
    * Associated file name mutator.
    * 
    * @param   filename
    *          The filename to set.
    */
   public void setFilename(String filename)
   {
      this.filename = filename;
   }
}
