/*
** Module   : DataTarget.java
** Abstract : Describes an external target to which data is written from a buffer.
**
** Copyright (c) 2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- ---------------------------------Description---------------------------------
** 001 ECF 20170820 Created initial version.
** 002 CA  20170830 Added missing c'tor.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.persist;

import com.goldencode.p2j.util.*;

/**
 * Object which describes an external target into which data is written from a temp-table buffer
 * or data set member buffer.
 */
public class DataTarget
{
   /**
    * Constructor.
    * 
    * @param   type
    *          {@code FILE} or {@code STREAM}.
    * @param   target
    *          Name of a file or stream to which data is written.
    * @param   formatted
    *          Pretty-print the XML.
    * @param   encoding
    *          The encoding to use for the XML. Default UTF-8.
    * @param   schemaLocation
    *          The external XML schema file.
    * @param   writeXMLSchema
    *          Write the table structure as in-line schema.
    * @param   minXMLSchema
    *          Write minimum amount of schema.
    */
   public DataTarget(character type, 
                     character target,
                     logical   formatted,
                     character encoding,
                     character schemaLocation,
                     logical   writeXMLSchema,
                     logical   minXMLSchema)
   {
      UnimplementedFeature.missing("DataTarget c'tor");
   }
   /**
    * Constructor.
    * 
    * @param   type
    *          {@code FILE} or {@code STREAM}.
    * @param   target
    *          Name of a file or stream to which data is written.
    */
   public DataTarget(character type, character target)
   {
      UnimplementedFeature.missing("DataTarget c'tor");
   }
   
   /**
    * Constructor.
    * 
    * @param   type
    *          {@code FILE} or {@code STREAM}.
    * @param   target
    *          Name of a file or stream to which data is written.
    */
   public DataTarget(String type, character target)
   {
      UnimplementedFeature.missing("DataTarget c'tor");
   }
   
   /**
    * Constructor.
    * 
    * @param   type
    *          {@code FILE} or {@code STREAM}.
    * @param   target
    *          Name of a file or stream to which data is written.
    */
   public DataTarget(character type, String target)
   {
      UnimplementedFeature.missing("DataTarget c'tor");
   }
   
   /**
    * Constructor.
    * 
    * @param   type
    *          {@code FILE} or {@code STREAM}.
    * @param   target
    *          Name of a file or stream to which data is written.
    */
   public DataTarget(String type, String target)
   {
      UnimplementedFeature.missing("DataTarget c'tor");
   }
   
   /**
    * Constructor.
    * 
    * @param   target
    *          Pointer to an area of memory into which data is written.
    */
   public DataTarget(memptr target)
   {
      UnimplementedFeature.missing("DataTarget c'tor");
   }
   
   /**
    * Constructor.
    * 
    * @param   type
    *          {@code STREAM-HANDLE} or {@code HANDLE}.
    * @param   target
    *          Handle of a stream object or X-document/X-noderef object into which to write data.
    */
   public DataTarget(character type, handle target)
   {
      UnimplementedFeature.missing("DataTarget c'tor");
   }
   
   /**
    * Constructor.
    * 
    * @param   type
    *          {@code STREAM-HANDLE} or {@code HANDLE}.
    * @param   target
    *          Handle of a stream object or X-document/X-noderef object into which to write data.
    */
   public DataTarget(String type, handle target)
   {
      UnimplementedFeature.missing("DataTarget c'tor");
   }
   
   /**
    * Constructor.
    * 
    * @param   target
    *          Variable into which to write data.
    */
   public DataTarget(longchar target)
   {
      UnimplementedFeature.missing("DataTarget c'tor");
   }
}
