/*
** Module   : Common.cs
** Abstract : Utility functions for data type conversion in Functions/Operators.
**
** Copyright (c) 2014-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- -------------------------------- Description --------------------------------
** 001 OM  20140313 Created initial version. Support for conversion to IKVM wrapper and
**                  bidirectional for date and datetime.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

/** In this version we are using some deprecated API. Disable the warnings for the moment. */
#pragma warning disable 0612

using System;

/**
 * Utility class for simplifying the syntax of datatype conversions in Functions/Operators.
 */
public class Common
{
    /**
     * The constructor is private. No instances of this class are allowed. All access is done
     * using the static methods.
     */
    private Common()
    {
    }
    
    /**
     * Convert a date to IKVM equivalent.
     * NOTE: Uses deprecated API and should be rewritten.
     *
     * @params d
     *         The date to be converted.
     *
     * @returns a java.sql.Date IKVM object that holds the respective date.
     */
    public static java.sql.Date toDate(DateTime d)
    {
        return new java.sql.Date(d.Year, d.Month, d.Day);
    }
    
    /**
     * Convert a IKVM date to CLR equivalent.
     * NOTE: Uses deprecated API and should be rewritten.
     *
     * @params d
     *         The date to be converted.
     *
     * @returns a CLR object that holds the respective date.
     */
    public static DateTime? fromDate(java.sql.Date d)
    {
        if (d == null)
        {
            return null;
        }
        return new DateTime(d.getYear(), d.getMonth(), d.getDate());
    }
    
    /**
     * Convert a date/time to IKVM equivalent.
     * NOTE: Uses deprecated API and should be rewritten.
     *
     * @params t
     *         The date/time to be converted.
     *
     * @returns a java.sql.Timestamp IKVM object that holds the respective date/time.
     */
    public static java.sql.Timestamp toTimestamp(DateTime t)
    {
        return new java.sql.Timestamp(t.Year, t.Month, t.Day,
               t.Hour, t.Minute, t.Second, t.Millisecond * 1000000);
    }
    
    /**
     * Convert a IKVM date/time to CLR equivalent.
     * NOTE: Uses deprecated API and should be rewritten.
     *
     * @params t
     *         The date/time to be converted.
     *
     * @returns a CLR object that holds the respective date/time.
     */
    public static DateTime? fromTimestamp(java.sql.Timestamp t)
    {
        if (t == null)
        {
            return null;
        }
        return new DateTime(t.getYear(), t.getMonth(), t.getDate(),
                t.getHours(), t.getMinutes(), t.getSeconds(), t.getNanos() / 1000000);
    }
    
    /**
     * Convert a 32 bit integer to IKVM equivalent.
     *
     * @params value
     *         The value to be converted.
     *
     * @returns a java.lang.Integer IKVM object that holds the respective int number.
     */
    public static java.lang.Integer toInteger(Int32 value)
    {
        return java.lang.Integer.valueOf(value);
    }
    
    /**
     * Convert a java.lang.Integer IKVM to .Net nullable Int32.
     * This method is NEVER used.
     *
     * @params value
     *         The value to be converted.
     *
     * @returns a Int32 object that holds the respective int number.
     */
    public static Int32? fromInteger(java.lang.Integer value)
    {
        return value == null ? (Int32?)null : (Int32?)value.intValue();
    }
   
    /**
     * Convert a 64 bit integer to IKVM equivalent.
     *
     * @params value
     *         The value to be converted.
     *
     * @returns a java.lang.Long IKVM object that holds the respective long number.
     */
    public static java.lang.Long toLong(Int64 value)
    {
        return java.lang.Long.valueOf(value);
    }

    /**
     * Convert a java.lang.Long IKVM to .Net nullable Int64.
     *
     * @params value
     *         The value to be converted.
     *
     * @returns a Int64 object that holds the respective long number.
     */
    public static Int64? fromLong(java.lang.Long value)
    {
        return value == null ? (Int64?)null : (Int64?)value.longValue();
    }
    
    /**
     * Convert a boolean value to IKVM equivalent.
     *
     * @params value
     *         The value to be converted.
     *
     * @returns a java.lang.Boolean IKVM object that holds the respective logical value.
     */
    public static java.lang.Boolean toBoolean(Boolean? value)
    {
        return value == null 
                     ? (java.lang.Boolean) null  // unknown logical value
                     : java.lang.Boolean.valueOf((Boolean) value);
    }
    
    /**
     * Convert a java.lang.Long IKVM to .Net nullable Boolean.
     *
     * @params value
     *         The value to be converted.
     *
     * @returns a Boolean object that holds the respective logical number.
     */
    public static Boolean? fromBoolean(java.lang.Boolean value)
    {
        return value == null ? (Boolean?)null : (Boolean?)value.booleanValue();
    }
    
    /**
     * Convert a decimal number to IKVM equivalent.
     *
     * @params value
     *         The value to be converted.
     *
     * @returns a java.math.BigDecimal IKVM object that holds the respective decimal number.
     */
    public static java.math.BigDecimal toBigDecimal(Double value)
    {
        return java.math.BigDecimal.valueOf(value);
    }
    
    /**
     * Convert a java.math.BigDecimal IKVM to .Net nullable Double.
     *
     * @params value
     *         The value to be converted.
     *
     * @returns a Double object that holds the respective decimal number.
     */
    public static Double? fromBigDecimal(java.math.BigDecimal value)
    {
        return value == null ? (Double?)null : (Double?)value.doubleValue();
    }
}