/*
** Module   : ReportConstants.java
** Abstract : Constants used by source code analytics tools.
**
** Copyright (c) 2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- --------------------------------Description----------------------------------
** 001 ECF 20170514 Created first version.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.report;

/**
 * Constants used by the source code analytics tools.
 */
public interface ReportConstants
{
   /** Support level is unknown. */
   public static final int LVL_UNKNOWN         = 0x0000;
   
   /** Mask to find the conversion support portion. */
   public static final int CVT_LVL_MASK        = 0x001F;
   
   /** No conversion support. */
   public static final int CVT_LVL_NONE        = 0x0001;
   
   /** Partial conversion support. */
   public static final int CVT_LVL_PARTIAL     = 0x0002;
   
   /** Conversion support exists, but edge conditions and variants need exploration. */
   public static final int CVT_LVL_BASIC       = 0x0004;
   
   /** Full conversion support, but there are permanent restrictions. */
   public static final int CVT_LVL_FULL_RESTR  = 0x0008;
   
   /** Full conversion support, no restrictions. */
   public static final int CVT_LVL_FULL        = 0x0010;
   
   /** Mask to find the runtime support portion. */
   public static final int RT_LVL_MASK         = 0x7F00;
   
   /** No runtime support. */
   public static final int RT_LVL_NONE         = 0x0100;
   
   /** Runtime support has been stubbed out but is not functional. */
   public static final int RT_LVL_STUB         = 0x0200;
   
   /** Runtime support was implemented from documentation and real compatibility is unknown. */
   public static final int RT_LVL_UNTESTED     = 0x0400;
   
   /** Partial runtime support exists. */
   public static final int RT_LVL_PARTIAL      = 0x0800;
   
   /** Runtime support exists; edge conditions, implicit behavior and errors need exploration. */
   public static final int RT_LVL_BASIC        = 0x1000;
   
   /** Full runtime support, but there are permanent restrictions. */
   public static final int RT_LVL_FULL_RESTR   = 0x2000;
   
   /** Full runtime support, no restrictions. */
   public static final int RT_LVL_FULL         = 0x4000;
   
   /** Unknown report source type */
   public static final int SRC_UNKNOWN    = 0x0000;
   
   /** Schema file type identifier */
   public static final int SRC_SCHEMA     = 0x0001;
   
   /** Preprocessed source file type identifier */
   public static final int SRC_CODE_CACHE = 0x0002;
   
   /** Un-preprocessed source file type identifier */
   public static final int SRC_CODE_BASE  = 0x0004;
   
   /** Custom report sort type - no specific sorting */
   public static int SORT_NONE = 0;
   
   /** Custom report sort type - sort all rows */
   public static int SORT_ALL = 1;
   
   /** Custom report sort type - sort all but last rows (to avoid sorting a summary row) */
   public static int SORT_NOT_LAST = 2;
}
