/*
** Module   : XmlConstants.java
** Abstract : XML Constants used by source code analytics tools.
**
** Copyright (c) 2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- --------------------------------Description----------------------------------
** 001 ECF 20170625 Created first version.
** 002 ECF 20170923 Removed unused constant.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.report;

import java.io.File;

/**
 * Constants used to read and write analytics objects to and from XML.
 */
public interface XmlConstants
{
   /** Report export path */
   public static final String RPT_PATH = "rpt" + File.separator;
   
   /** Report definition file suffix. */
   public static final String RPT_SUFFIX = ".rpt";
   
   /** Path name spec for auto-exported XML report files. */
   public static final String AUTO_EXPORT_RPT_SPEC = RPT_PATH + "auto_export_%s_reports.xml";
   
   /** Path for search history auto-export file */
   public static final String AUTO_EXPORT_SRCH_PATH = RPT_PATH + "auto_export_search_history.xml";
   
   /** Path for user credentials auto-export file */
   public static final String AUTO_EXPORT_USER_PATH = RPT_PATH + "auto_export_users.xml";
   
   /** Path for filter profiles auto-export file */
   public static final String AUTO_EXPORT_FILTER_PATH = RPT_PATH + "auto_export_filters.xml";
   
   /** list element text. */
   public static final String ELEM_LIST = "list";
   
   /** report element text. */
   public static final String ELEM_RPT = "report";
   
   /** profile element text. */
   public static final String ELEM_PROFILE = "profile";
   
   /** filter element text. */
   public static final String ELEM_FILTER = "filter";
   
   /** search element text. */
   public static final String ELEM_SEARCH = "search";
   
   /** user element text. */
   public static final String ELEM_USER = "user";
   
   /** id attribute text. */
   public static final String ATTR_ID = "id";
   
   /** name attribute text. */
   public static final String ATTR_NAME = "name";
   
   /** admin attribute text. */
   public static final String ATTR_ADMIN = "admin";
   
   /** salt attribute text. */
   public static final String ATTR_SALT = "salt";
   
   /** password attribute text. */
   public static final String ATTR_PASSWORD = "password";
   
   /** password changed date attribute text. */
   public static final String ATTR_PASS_CHANGED = "pass-changed";
   
   /** spec attribute text. */
   public static final String ATTR_SPEC = "spec";
   
   /** wild attribute text. */
   public static final String ATTR_WILD = "wild";
   
   /** database attribute text. */
   public static final String ATTR_DB = "database";
   
   /** title attribute text. */
   public static final String ATTR_TITLE = "title";
   
   /** tag attribute text. */
   public static final String ATTR_TAG = "tags";
   
   /** condition attribute text. */
   public static final String ATTR_COND = "condition";
   
   /** user(-defined) attribute text. */
   public static final String ATTR_USER = "user";
   
   /** sort attribute text. */
   public static final String ATTR_SORT = "sort";
   
   /** dumpType attribute text. */
   public static final String ATTR_DUMPTYPE = "dumpType";
   
   /** dumpExpr attribute text. */
   public static final String ATTR_DUMPEXPR = "dumpExpr";
   
   /** dumpLevel attribute text. */
   public static final String ATTR_DUMPLVL = "dumpLevel";
   
   /** multiplexExpr attribute text. */
   public static final String ATTR_MULTIPLEX = "multiplexExpr";
   
   /** supportLvlExpr attribute text. */
   public static final String ATTR_SUPPORT = "supportLvlExpr";
}