/*
** Module   : CallGraphNode.java
** Abstract : Encodes a single call graph node (graph database vertex).
**
** Copyright (c) 2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- ---------------------------------Description----------------------------------
** 001 GES 20170530 First version.
*/

/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.report.server;

/**
 * Encodes a single call graph node (graph database vertex).
 */
public class CallGraphNode
{
   /** Graph-unique id for the node. */ 
   private String id = "";
   
   /** Progress parser token type that describes the node. */
   private int type = -1;
   
   /** Descriptive text for the node. */
   private String text = "";
   
   /** The node's tooltip. */
   private String tooltip = "";
   
   /**
    * Construct an instance.
    *
    * @param    id
    *           Graph-unique id for the node.
    * @param    type
    *           Progress parser token type that describes the node.
    * @param    text
    *           Descriptive text for the node.
    * @param    tooltip
    *           Tooltip text for the node.
    */
   public CallGraphNode(String id, int type, String text, String tooltip)
   {
      this.id   = id;
      this.type = type;
      this.text = text;
      this.tooltip = tooltip;
   }
   
   /**
    * Obtain the unique id of the node.
    *
    * @return   The node's unique id.
    */
   public String getId()
   {
      return id;
   }

   /**
    * Set the unique id of the node.
    *
    * @param    id
    *           The new id to assign.
    */
   public void setId(String id)
   {
      this.id = id;
   }

   /**
    * Obtain the token type for the node.
    *
    * @return   The token type.
    */
   public int getType()
   {
      return type;
   }

   /**
    * Set the token type for the node.
    *
    * @param    type
    *           The token type to assign.
    */
   public void setType(int type)
   {
      this.type = type;
   }

   /**
    * Obtain the descriptive text for the node.
    *
    * @return   The description of the node.
    */
   public String getText()
   {
      return text;
   }

   /**
    * Set the descriptive text for the node.
    *
    * @param    text
    *           The new descriptive text to assign.
    */
   public void setText(String text)
   {
      this.text = text;
   }

   /**
    * Obtain the tooltip text for the node.
    *
    * @return   The tooltip of the node.
    */
   public String getTooltip()
   {
      return tooltip;
   }

   /**
    * Set the tooltip text for the node.
    *
    * @param    tooltip
    *           The new tooltip text to assign.
    */
   public void setTooltip(String tooltip)
   {
      this.tooltip = tooltip;
   }
   
   /**
    * Compute the unique hashcode for this instance, using the {@link #id}.
    * fields.
    * 
    * @return   See above.
    */
   @Override
   public int hashCode()
   {
      return id.hashCode();
   }
   
   /**
    * Check if this instance is the same as the other one (if {@link #id} is the same).
    * 
    * @param    obj
    *           The instance to compare.
    * @return   See above.
    */
   @Override
   public boolean equals(Object obj)
   {
      if (!(obj instanceof CallGraphNode))
      {
         return false;
      }
      
      return id.equals(((CallGraphNode) obj).id);
   }
}