/*
** Module   : callgraph.js
** Abstract : callgraph web client java script functionality 
**
** Copyright (c) 2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- ------------------------------Description----------------------------------
** 001 GES 20170519 First version.
*/

/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

//"use strict";

/**
 * Call graph visualization functionality.
 */
// fwd.graph
define(["wsbase"], function(ws)
{
   "use strict";
   /** The top-level object which will be returned from this function. */
   var me = {};
   
   /** Size of the margin around the svg in pixels. */
   var SVG_MARGIN = 20;
   
   /** Size of the margin around the text in pixels. */
   var TEXT_MARGIN = 3;
   
   /** Rectangle rounded corners radius in pixels. */
   var RECT_ROUNDING_RADIUS = 1.5;
   
   /** Width buffer multiplier to leave space in a container rectangle. */
   var WIDTH_BUFFER = .1;
   
   /** Height buffer multiplier to leave space in a container rectangle. */
   var HEIGHT_BUFFER = .2;
   
   /** Number of pixels to leave empty at each side of a container rectangle. */
   var SIDE_BUFFER = 5;
            
   /** Number of pixels to leave empty at each side of a container rectangle. */
   var OUTER_BUFFER = 50;
            
   /** Base message ID for the callgraph API. */
   var MSG_BASE = ws.MSG_CG_BASE;
   
   /** Message types. */
   var msgTypes =
   {
      MST_READ_GRAPH       :  MSG_BASE + 1,
      MST_MISSING_TARGETS  :  MSG_BASE + 2,
      MST_CALL_SITES       :  MSG_BASE + 3,
      MST_EXTERNAL_DEP     :  MSG_BASE + 4,
      MST_DEAD_CODE        :  MSG_BASE + 5,
      MST_AMBIG_CALLSITES  :  MSG_BASE + 6,
      MST_FILE_AMBIG_CALLSITES  : MSG_BASE + 7,
      MST_DEAD_DEPENDENCIES     : MSG_BASE + 8,
      MST_VERIFY_SHEMA_TRIGGERS : MSG_BASE + 9,
      MST_REPORT_LIST_DEPENDENCIES : MSG_BASE + 10,
      MST_REPORT_DEPENDENCIES_FOR  : MSG_BASE + 11,
   };
   
   /** Direction constants. */
   var direction =
   {
      BOTH_DIRECTIONS : 0,
      DOWNSTREAM      : 1,
      UPSTREAM        : 2,
   };
   
   /** Constants for graph depth detail. */
   var graphDetail = 
   {
      FILES        : 0,
      ENTRY_POINTS : 1,
      ALL          : 2
   };
   
   /** Creates a set of named constants for token types that are directly referenced. */  
   var tokens = { };
   
   /** A nice set of colors. */
   var color = d3.scaleOrdinal(d3.schemeCategory20b);
   
   // you have to read the colors in order otherwise you can get duplicates
   for(var cs = 0; cs < 20; cs++)
   {
      color(cs);
   }
   
   /** List of all possible link types. */
   var linkTypes =
   [
      "ambiguous", 
      "calls",     
      "contains",  
      "implements",
      "includes",  
      "inherits",  
      "listens",   
      "override",  
      "raises",    
      "references",
      "registers",
   ];
   
   /** List of all possible link arrow types. */
   var markerTypes = [];
   
   /** Mapping of node types to visual properties. */
   var displayProps = {};          
                                          
   /** Current data model. */
   var data = null;
   
   /** Legend details. */
   var legendContent = [];
   
   /** Legend element bundle. */
   var legend = null;
   
   /** Toolbox gear icon (part of the SVG). */
   var toolboxIcon = null;
   
   /** Toolbox element. */
   var $toolbox = null;
   
   /** Dijit combobox for the starting points. */
   var startingPointsWidget = null;
   
   /** Dijit NumberTextBox for the nesting level. */
   var nestingLevelWidget = null;
   
   /** Dijit RadioButton for the direction BOTH. */
   var directionBothWidget = null;
   
   /** Dijit RadioButton for the direction IN. */
   var directionInWidget = null;
   
   /** Dijit RadioButton for the direction OUT. */
   var directionOutWidget = null;
   
   /** Dijit RadioButton for graph detail ALL. */
   var detailAllWidget = null;
   
   /** Dijit RadioButton for graph detail FILES. */
   var detailFilesWidget = null;
   
   /** Dijit RadioButton for graph detail ENTRY_POINTS. */
   var detailEntryPointsWidget = null;
   
   /** The last used nesting level value. */
   var nestingLevelValue = 0;
   
   /** The last used starting points. */
   var startingPointsValue = null;
   
   /** The last used profile ID. */
   var profileIdValue = 0;
   
   /** The last used direction. */
   var directionValue = direction.BOTH_DIRECTIONS;

   /** The last used graph detail value. */
   var graphDetailValue = graphDetail.ALL;

   /** Flag indicating how the links are expanded. */
   var moreLinksValue = false;
   
   /** Lookup table to find nodes by the id as a key. */
   var nodeMap = null;
   
   /** Lookup table to find links by the id as a key. */
   var linkMap = null;

   /**
     * Execute and show the 'Missing Call-Site Targets' report.
     */
   function showReportMissingTargets()
   {
      var repColumns = 
      [
         {
            field   : "type",
            title   : "Type",
            visible : false
         },
         {
            field        : "name",
            title        : "Target",
            visible      : true,
            headerFilter : true,
         },
         {
            field   : "callNum",
            title   : "Call #",
            align   : "center",
            width   : "15%",
            visible : true
         },
         {
            field   : "vertexId",
            title   : "vertexId",
            visible : false
         }
      ];

      showReportsMainView(msgTypes.MST_MISSING_TARGETS,
                          "Missing Call-Site Targets",
                          "type",
                          repColumns,
                          showCallSitesReport);
   }

   /**
     * Execute and show the 'External Dependencies' report.
     */
   function showReportExternalDependencies()
   {
      var repColumns = 
      [
         {
            field   : "type",
            title   : "Type",
            visible : false
         },
         {
            field        : "name",
            title        : "Target",
            visible      : true,
            headerFilter : true,
         },
         {
            field   : "callNum",
            title   : "Call #",
            align   : "center",
            width   : "15%",
            visible : true
         },
         {
            field   : "vertexId",
            title   : "vertexId",
            visible : false
         }
      ];

      showReportsMainView(msgTypes.MST_EXTERNAL_DEP,
                          "External Dependencies",
                          "type",
                          repColumns,
                          showCallSitesReport);
   }

   /**
     * Execute and show the 'Dead Code' report.
     */
   function showReportDeadCode()
   {
      var repColumns = 
      [
         {
            field   : "type",
            title   : "Type",
            visible : false
         },
         {
            field        : "file",
            title        : "File",
            visible      : true,
            headerFilter : true,
            tooltip      : function(field, value, data)
                           {
                              return data.file;
                           },
         },
         {
            field        : "name",
            title        : "Name",
            visible      : true,
            headerFilter : true,
         },
         {
            field   : "vertexId",
            title   : "vertexId",
            visible : false
         }
      ];

      showReportsMainView(msgTypes.MST_DEAD_CODE,
                          "Dead Code",
                          "type",
                          repColumns,
                          showDeadDependency);
   }

   /**
     * Execute and show the 'Ambiguous Call Sites' report.
     */
   function showReportAmbiguousCallSites()
   {
      var repColumns = 
      [
         {
            field        : "filename",
            title        : "File Name",
            visible      : true,
            headerFilter : true,
         },
         {
            field   : "num",
            title   : "#",
            align   : "center",
            width   : "15%",
            visible : true
         }
      ];

      showReportsMainView(msgTypes.MST_AMBIG_CALLSITES,
                          "Ambiguous Call Sites",
                          null,
                          repColumns,
                          showFileAmbigCallSites);
   }
   
   /**
     * Execute and show the 'Verify Schema Triggers' report.
     */
   function showReportVerifySchemaTriggers()
   {
      var repColumns = 
      [
         {
            field   : "type",
            title   : "Type",
            visible : false
         },
         {
            field        : "name",
            title        : "External Program",
            visible      : true,
            headerFilter : true,
         },
         {
            field   : "callNum",
            visible : false
         },
         {
            field   : "vertexId",
            title   : "vertexId",
            visible : false
         }
      ];

      showReportsMainView(msgTypes.MST_VERIFY_SHEMA_TRIGGERS,
                          "Verify Schema Triggers",
                          "type",
                          repColumns,
                          showCallSitesReport);
   }
      
   /**
     * Execute and show the 'Dependencies' report.
     */
   function showReportDependencies()
   {
      var repColumns = 
      [
         {
            field   : "type",
            title   : "Type",
            visible : false
         },
         {
            field        : "name",
            title        : "File",
            visible      : true,
            headerFilter : true,
         },
         {
            field   : "callNum",
            title   : "Dependency #",
            visible : true,
         },
         {
            field   : "vertexId",
            title   : "vertexId",
            visible : false
         }
      ];

      showReportsMainView(msgTypes.MST_REPORT_LIST_DEPENDENCIES,
                          "Dependencies",
                          "type",
                          repColumns,
                          showDependenciesForProgram);
   }
   
   /**
     * Main worker to show the left-side report.
     * 
     * @param    msgType
     *           The message ID of the executed report.
     * @param    title
     *           The report title.
     * @param    groupField
     *           The name of the field on which to perform row grouping.
     * @param    repColumns
     *           The report's column details.
     * @param    rightSideRep
     *           The callback executed to show the right-side report for a selected line.
     */
   function showReportsMainView(msgType, title, groupField, repColumns, rightSideRep)
   {
      var parent = $("#repCGView");
      
      parent.empty();
      
      var div = "<div></div>";
      
      var containDiv = $(div).attr("id", "containerView").addClass("container-fluid");
      parent.append(containDiv);
      
      // append attach point for first row
      var row1Div = $(div).addClass("row");
      $(containDiv).append(row1Div);
      
      // append attach point for second row
      var row2Div = $(div).addClass("row");
      $(containDiv).append(row2Div);
      
      fwd.showOverlay("Building Overview Report");
      
      ws.sendMessage(msgType, null, function(err, rows)
      {
         fwd.hideOverlay();
         
         var r1c1Div = $(div).addClass("col-md-4").css("padding", "3px");
         row1Div.append(r1c1Div);

         var r1c2Div = $(div).addClass("col-md-8").css("padding", "3px");
         row1Div.append(r1c2Div);
         
         // append title
         var tocTitleDiv = $(div).addClass("toc-heading").text(title);
         r1c1Div.append(tocTitleDiv);

         var tab1Div = $(div);
         tab1Div.attr("id", "reportToc");
         r1c1Div.append(tab1Div);

         // initialize the tabulator
         tab1Div.tabulator(
         {
            fitColumns           :  true, // fit columns to width of table
            height               :  (fwd.getAvailableHeight() - 16) + "px",
            groupBy              :  groupField,
            groupStartOpen       :  true,
            selectable           :  1,
            sortable             :  false,
            tooltipsHeader       :  true,
            columns              :  repColumns,
            rowSelectionChanged  :  function(data, rows)
                                    {
                                       // display summary report associated with this row
                                       if (rightSideRep && data.length > 0)
                                       {
                                          rightSideRep(r1c2Div, data[0]);
                                       }
                                    },
            tableBuilt           :  function()
                                    {
                                       if (rightSideRep && rows.length > 0)
                                       {
                                          var data = rows[0];
                                          rightSideRep(r1c2Div, data);
                                       }
                                    },
            data                 :  rows,
         });
         
         // re-render table on window resize
         $(window).resize(function()
         {
            $("#reportToc").tabulator("redraw");
         });
      });
   }
   
   /**
     * Execute and show the 'Ambiguous Call Sites for Program ' right-side report, associated with
     * the selected line from the left-side report.
     
     * @param    selector
     *           The parent element where the report will be attached.
     * @param    data
     *           The data for the selected line in the left-side report.
     */
   function showFileAmbigCallSites(selector, data)
   {
      if (data["filename"] == undefined)
      {
         return;
      }
      
      
      var repCols = 
      [
         {
            title          :  "Filename",
            field          :  "filename",
            visible        :  false
         },
         {
            title          :  "Call Site",
            field          :  "callSite",
            visible        :  true
         },
         {
            title          :  "Line",
            field          :  "line",
            align          :  "center",
            width          :  "15%",
            visible        :  true
         },
         {
            title          :  "Column",
            field          :  "column",
            align          :  "center",
            width          :  "15%",
            visible        :  true
         },
         {
            title          :  "Vertex ID",
            field          :  "vertexId",
            visible        :  false
         }
      ];

      showCallSitesReportMain(msgTypes.MST_FILE_AMBIG_CALLSITES,
                              selector,
                              data,
                              repCols,
                              null,
                              "Ambiguous Call Sites for Program " + data.filename,
                              [ data.filename ]);
   }
      
   /**
     * Execute and show the 'Dependencies' right-side report, associated with the selected line
     * from the left-side report.
     
     * @param    selector
     *           The parent element where the report will be attached.
     * @param    data
     *           The data for the selected line in the left-side report.
     */
   function showDependenciesForProgram(selector, data)
   {
      if (data["name"] == undefined)
      {
         return;
      }
      
      
      var repCols = 
      [
         {
            title          :  "Filename",
            field          :  "filename",
            visible        :  false,
         },
         {
            title          :  "Call Site",
            field          :  "callSite",
            visible        :  true
         },
         {
            title          :  "Line",
            field          :  "line",
            align          :  "center",
            width          :  "15%",
            visible        :  true
         },
         {
            title          :  "Column",
            field          :  "column",
            align          :  "center",
            width          :  "15%",
            visible        :  true
         },
         {
            title          :  "Vertex ID",
            field          :  "vertexId",
            visible        :  false
         }
      ];

      showCallSitesReportMain(msgTypes.MST_REPORT_DEPENDENCIES_FOR,
                              selector,
                              data,
                              repCols,
                              "filename",
                              "Dependencies for Program " + data.name,
                              [ data.vertexId ]);
   }
   
   /**
     * Execute and show the 'Call Sites' right-side report.
     
     * @param    selector
     *           The parent element where the report will be attached.
     * @param    data
     *           The data for the selected line in the left-side report.
     */
   function showCallSitesReport(selector, data)
   {
      if (data["vertexId"] == undefined)
      {
         return;
      }
      
      var repCols = 
      [
         {
            title          :  "Filename",
            field          :  "filename",
            visible        :  false
         },
         {
            title          :  "Call Site",
            field          :  "callSite",
            visible        :  true
         },
         {
            title          :  "Line",
            field          :  "line",
            align          :  "center",
            width          :  "15%",
            visible        :  true
         },
         {
            title          :  "Column",
            field          :  "column",
            align          :  "center",
            width          :  "15%",
            visible        :  true
         },
         {
            title          :  "Vertex ID",
            field          :  "vertexId",
            visible        :  false
         }
      ];

      showCallSitesReportMain(msgTypes.MST_CALL_SITES,
                              selector,
                              data,
                              repCols,
                              "filename",
                              "Call Sites",
                              [ data.vertexId ]);
   }
   
   /**
     * Execute and show the 'Dead Dependencies' right-side report, associated with a line from the
     * 'Dead Code' report.
     *
     * @param    selector
     *           The parent element where the report will be attached.
     * @param    data
     *           The data for the selected line in the left-side report.
     */
   function showDeadDependency(selector, data)
   {
      if (data["vertexId"] == undefined)
      {
         return;
      }
      
      var repCols = 
      [
         {
            title          :  "Dependency",
            field          :  "tokenText",
            visible        :  false
         },
         {
            title          :  "Entry Point",
            field          :  "callSite",
            visible        :  true
         },
         {
            title          :  "Line",
            field          :  "line",
            align          :  "center",
            width          :  "15%",
            visible        :  true
         },
         {
            title          :  "Column",
            field          :  "column",
            align          :  "center",
            width          :  "15%",
            visible        :  true
         },
         {
            title          :  "Vertex ID",
            field          :  "vertexId",
            visible        :  false
         }
      ];

      showCallSitesReportMain(msgTypes.MST_DEAD_DEPENDENCIES,
                              selector,
                              data,
                              repCols,
                              "tokenText",
                              "Dependencies",
                              [ data.vertexId ]);
   }
   
   /**
     * Main worker to show the right-side report, associated with a selected line in the left-side
     * report.
     * 
     * @param    msgType
     *           The message ID of the executed report.
     * @param    selector
     *           The parent element where the report will be attached.
     * @param    data
     *           The data for the selected line in the left-side report.
     * @param    repCols
     *           The report's column details.
     * @param    groupField
     *           The name of the field on which to perform row grouping.
     * @param    title
     *           The report title.
     * @param    parms
     *           The parameters to pass to the server-side, to execute the report.
     */
   function showCallSitesReportMain(msgType, selector, data, repCols, groupField, title, parms)
   {
      var parent = $(selector);
      
      parent.empty();
      
      var div = "<div></div>";
      
      var sumTitleDiv = $(div).addClass("toc-heading").text(title);
      parent.append(sumTitleDiv);
      
      // append attach point for tabulator
      var attachDiv = $(div);
      attachDiv.attr("id", "callSitesReport");
      
      fwd.showOverlay("Building Detail Report");
      
      ws.sendMessage(msgType, parms, function(err, rows)
      {
         fwd.hideOverlay();
         
         parent.append(attachDiv);
         
         // initialize the summary report tabulator
         attachDiv.tabulator(
         {
            fitColumns           : true, // fit columns to width of table
            height               : $("#reportToc").outerHeight() + "px",
            sortable             : false,
            pagination           : "local",
            paginationSize       : 150,
            tooltipsHeader       : true,
            groupBy              : groupField,
            columns              : repCols,
            data                 : rows,
            rowClick             : function(e, id, data, row)
            {
               if (data.vertexId)
               {
                  // display detail report associated with this row
                  fwd.rpt.showSourceAstView("#containerView",
                                            data.fid,
                                            data.filename,
                                            data.astId,
                                            data.srcType,
                                            data.line,
                                            data.column,
                                            data.callSite);
               }
            },
         });
         
         // re-render table on window resize
         $(window).resize(function()
         {
            $("#callSitesReport").tabulator("redraw");
         });
      });
   }
   
   /**
    * Initialize the call graph view.
    */
   function showCallGraphView()
   {
      $("#callGraphView").empty();
      
      // if you want to just load a single file, you can pass [ "./abl/start.p.ast" ] instead
      // of null
      var starts = Array.isArray(startingPointsValue)
                     ? startingPointsValue 
                     : startingPointsValue 
                        ? [ startingPointsValue ] 
                        : null;
      readGraph(starts, profileIdValue, directionValue, nestingLevelValue, graphDetailValue);
   }
      
   /**
    * Primary worker that actually displays the call graph based on the graph snippet sent
    * from the server.  This is a callback processed when the server up-call returns.
    */
   function loadGraph()
   {
      // we create a child container so that switching away from this view can easily clean
      // up and rmeove all our changes
      var $parent = $("<div>", {id: "graphContainer"});
      $("#callGraphView").append($parent);
      
      // calc the width of hte SVG, based on the parent container, leave room for a margin
      var width  = $parent.width() - SVG_MARGIN;
      
      // calculate the height of the container div
      var height = fwd.getAvailableHeight() - 16;
      
      // set styles for the parent container for that we center vertically (align-items) and
      // horizontally (justify-content), depends on the display mode being flex
      $parent.css(
      {
         "position"        : "relative",
         "height"          : height + "px",
         "display"         : "flex",
         "align-items"     : "center",
         "justify-content" : "center"
      });
      
      // calc the height of the SVG, leave room for the margin
      height = height - SVG_MARGIN;
      
      // store state for the containment of nodes that are not themselves otherwise contained
      var outer = {};
      
      // create the SVG
      var svg = d3.select("#graphContainer").append("svg")
                      .attr("width", width)
                      .attr("height", height)
                      .style("background-color", "#ffffff");
                     
      // enable proper resizing of the SVG and the containing div
      // TODO: make this responsive
      window.addEventListener("resize", resize);
      
      // setup some data structures and calculate some display and legend properties
      initialize();
      
      // define a marker for each edge style     
      var markers = svg.append("defs")
                       .selectAll("marker")
                       .data(markerTypes)
                       .enter()
                       .append("marker")
                       .attr("id", function(d) { return d.name; }) 
                       .attr("orient", "auto")                                            
                       .attr("fill", function(d) { return displayProps[d.type].stroke; });
         
      // create arrowheads
      markers.filter(function(d) { return d.arrow; })
             .attr("viewBox", "0 -5 10 10")
             .attr("markerUnits", "strokeWidth")
             .attr("refX", 8)
             .attr("refY", 0)
             .attr("markerWidth", 4)
             .attr("markerHeight", 4)
             .append("svg:path")
             .attr("d", "M0,-5L10,0L0,5");      
      
      // create mid-point labels
      var linkLabels = markers.filter(function(d) { return !d.arrow; })
                              .attr("viewBox", "-25 -10 50 20")
                              .attr("markerUnits", "userSpaceOnUse")
                              .attr("refX", 0)
                              .attr("refY", 0)
                              .attr("markerWidth", 30)
                              .attr("markerHeight", 10)
                              .append("g");
                              
      linkLabels.append("path")                                            
                .attr("d", "M-24,-6L24,-6L24,6L-24,6z");
                
      linkLabels.append("text")
                .text(function(d) { return displayProps[d.type].text; })
                .attr("text-align", "middle")
                .attr("x", function(d) { return -1.5 * displayProps[d.type].text.length; })
                .attr("y", 2)
                .style("font", "normal 6px sans-serif")
                .style("stroke-width", ".3")
                .style("stroke", function(d) { return (displayProps[d.type].stroke == "black") ? "white" : "black"; })
                .style("fill", function(d) { return (displayProps[d.type].stroke == "black") ? "white" : "black"; });
                // TODO: this doesn't work, not sure why; the X/Y hack above will do for now
                //.each(calcLinkTextLoc);
             
      // create a master container for everything else
      var g = svg.append("g");
      
      nodeMap = createNodeMap(data.nodes);
      
      linkMap = createLinkMap(data.links);
      
      // compute the hidden link mappings and associate them with the nodes
      computeHiddenLinks(data.nodes, data.links, nodeMap, linkMap);
      
      legend = fwd.legend.createLegend(svg, displayProps, data.nodes, data.links);
      
      setupToolbox($parent);
          
      // create the node SVG elements and join them with the node data; these are group nodes
      // that will contain other elements so that the rendering can be more attractive
      var node = g.append("g")
                  .attr("class", "nodes")
                  .selectAll(".node")
                  .data(data.nodes)
                  .enter().append("g")
                  .attr("class", "node")
                  .on("dblclick", dblclick)
                  .call(d3.drag()
                          .on("start", dragstarted)
                          .on("drag", dragged)           
                          .on("end", dragended));
                  
      // create an outer rounded rectangle for rectangle nodes
      node.filter(function(d) { return (d.shape) ? d.shape == "rect" : true; })
          .append("rect")
          .attr("rx", RECT_ROUNDING_RADIUS)
          .attr("ry", RECT_ROUNDING_RADIUS)
          .attr("stroke", calcNodeStroke)
          .attr("stroke-dasharray", calcNodeDash)
          .attr("stroke-width", calcNodeStrokeWidth)
          .attr("fill", calcNodeColor)
          .on("click", onNodeSelected)
          .on("mouseenter", onNodeEnter)
          .on("mouseleave", onNodeExit);
      
      // create a circle for some nodes nodes
      node.filter(function(d) { return (d.shape) ? d.shape == "circle" : false; })
          .append("circle")
          .attr("stroke", "dimgray")
          .attr("fill", calcNodeColor)
          .on("click", onNodeSelected)
          .on("mouseenter", onNodeEnter)
          .on("mouseleave", onNodeExit);
      
      // create the text label for each node      
      node.append("text")
          .text(calcNodeText)
          .attr("text-align", "middle")
          .style("stroke", "black")
          .each(calcBounds);

      // create an star for nodes with hidden links (after the bounds are computed)
      node.filter(function(d) { return d.hiddenLinks; })
          .append("svg:polygon")
          .attr("stroke", "black")
          .attr("fill", calcStarColor)
          .on("click", showHideLinks)
          // .on("mousenter", showHideLinks) TODO: not working very good...
          // .on("mouseleave", showHideLinks)
          .attr("points", calcStarPoints);
          
      // add a tooltip to each node     
      node.append("title")
          .text(calcNodeTooltip);
      
      // create the link SVG elements and join them with the link data (by adding these after
      // the nodes already exist, we force the natural z-order of a link to be higher than that
      // of any node, which is important to allow drawing of links when there are contained
      // nodes which overlap with their container); otherwise the link would appear behind the
      // container
      var link = g.append("g")
                  .attr("class", "links")
                  .selectAll("line")
                  .data(data.links)
                  .enter()
                  .filter(function(d) { return d.text != "contains"; })
                  .append("polyline")
                  .style("visibility", function(l) { return l.hidden ? "hidden" : "visible"; })
                  .style("stroke", calcLinkStroke)
                  .attr("stroke-width", calcLinkStrokeWidth)
                  .attr("stroke-dasharray", calcLinkDash)
                  .attr("marker-mid", function(d) { return "url(#label_" + d.text + ")" })
                  .attr("marker-end", function(d) { return "url(#arrow_" + d.text + ")" });
      
      // add a tooltip to each link
      link.append("title")
          .text(function(d) { return displayProps[d.text].text; });
          
      // create our simulation's link force (the functions get a link as a parameter)
      var linkForce = d3.forceLink(data.links)
                        .distance(120)
                        .strength(.2)
                        .id(function(d) { return d.id; });
          
      // create the force directed graph simulation itself and add the forces and the core
      // timer tick processing (the functions get a node as a parameter) 
      var simulation = d3.forceSimulation()
                         .nodes(data.nodes)
                         .force("link", linkForce)
                         .force("charge", d3.forceManyBody().strength(-120))
                         .force("snap", forceSnap)
                         .on("tick", ticked);
                         
      // add the zoom/pan support; if we associate the zoom with the g element, then we don't 
      // get the events needed to zoom, but by using the svg we do get the events
      svg.call(d3.zoom().on("zoom", function() { g.attr("transform", d3.event.transform) }));
      
      var maxDepth = 0;
      
      // we calculate containment boundaries before the model first displays
      calcContainment();
      
      // at this point the simulation just runs, processing the timer tick while the simulation
      // is recalculating positions and processing user events (clicking, dragging...) when
      // they occur
      
      /**
       * Differentially determine the (many bodies force) charge depending on the node type.
       *
       * @param    d 
       *           This is the data element bound to the node.
       *
       * @return   A negative value for repulsion and positive for attraction.
       */
      function calcAttraction(d)
      {
         return (d.type == tokens.PROCEDURE_FILE) ? 200 : -200;
      }
      
      /**
       * Calculate the fill color for a node.
       *
       * @param    d 
       *           This is the data element bound to the node.
       *
       * @return   The color to use.
       */
      function calcNodeColor(d)
      {
         if (typeof displayProps[d.type] === "undefined")
         {
            return "gold";
         }
         
         return displayProps[d.type]["color"];
      }
      
      /**
       * Calculate the stroke color for a node.
       *
       * @param    d 
       *           This is the data element bound to the node.
       *
       * @return   The color to use (empty string for none).
       */
      function calcNodeStroke(d)
      {
         if (typeof displayProps[d.type] === "undefined")
         {
            return "chartreuse";
         }
         
         return displayProps[d.type]["stroke"];
      }
      
      /**
       * Calculate the "stroke-dasharray" for a node.
       *
       * @param    d 
       *           This is the data element bound to the node.
       *
       * @return   The stroke dash type to use (empty string for none).
       */
      function calcNodeDash(d)
      {
         if (typeof displayProps[d.type] === "undefined")
         {
            return "8";
         }
         
         return displayProps[d.type]["stroke-dasharray"];
      }
      
      /**
       * Calculate the "stroke-width" for a node.
       *
       * @param    d 
       *           This is the data element bound to the node.
       *
       * @return   The stroke width to use.
       */
      function calcNodeStrokeWidth(d)
      {
         if (typeof displayProps[d.type] === "undefined")
         {
            return "4";
         }
         
         return displayProps[d.type]["stroke-width"];
      }
      
      /**
       * Calculate the text description for a node.
       *
       * @param    d 
       *           This is the data element bound to the node.
       *
       * @return   The text for the node.
       */
      function calcNodeText(d)
      {
         var tn = fwd.ast.tokenName(d.type);
         var txt = null;
         var dp = displayProps[d.type];
         
         if (d.type != tokens.MORE && dp == undefined)
         {
            console.log("Can not find display prop for AST type " + tn);
            return tn;
         }
         
         if (d.type == tokens.MORE)
         {
            txt = "More";
         }
         else if (tn == d.text)
         {                          
            txt = dp.text;
         }                                                            
         else if (dp.augmentText)
         {
            txt = dp.text + " " + d.text;                
         }
         else
         {
            txt = d.text;
         }                                                                            
         
         if (d.type != tokens.MORE &&
             d.tooltip             &&
             txt.indexOf("<") > 0  &&
             txt.indexOf(">") > 0  &&
             d.tooltip.indexOf(";") < 0) 
         {
            // check if we can replace the text with the tooltip
            var idx1 = txt.indexOf("<");
            var idx2 = txt.indexOf(">");
            
            txt = txt.substring(0, idx1) + d.tooltip + txt.substring(idx2 + 1);
         }
         
         return txt;
      }
      
      /**
       * Calculate the tooltip description for a node.
       *
       * @param    d 
       *           This is the data element bound to the node.
       *
       * @return   The text for the node.
       */
      function calcNodeTooltip(d)
      {
         var tn = fwd.ast.tokenName(d.type);
         var extra = d.tooltip ? d.tooltip : (tn == d.text) ? "" : " : " + d.text;
         
         if (displayProps[d.type] == undefined)
         {
            console.log("Can not find display prop for AST type " + tn);
            return "(" + tn + ")" + extra;
         }
         
         return displayProps[d.type].text + " (" + tn + ")" + extra;
      }
      
      /**
       * Calculate the stroke color for a link.
       *
       * @param    d 
       *           This is the data element bound to the link.
       *
       * @return   The color to use (empty string for none).
       */
      function calcLinkStroke(d)
      {
         if (typeof displayProps[d.text] === "undefined")
         {
            return "chartreuse";
         }
         
         return displayProps[d.text]["stroke"];
      }
      
      /**
       * Calculate the "stroke-width" for a link.
       *
       * @param    d 
       *           This is the data element bound to the link.
       *
       * @return   The stroke width to use.
       */
      function calcLinkStrokeWidth(d)
      {
         if (typeof displayProps[d.text] === "undefined")
         {
            return "4";
         }
         
         var w = displayProps[d.text]["stroke-width"];
         
         if (d.hiddenCount !== null)
         {
            w = w * Math.sqrt(d.hiddenCount);
         }
         
         return w;
      }
      
      /**
       * Calculate the "stroke-dasharray" for a link.
       *
       * @param    d 
       *           This is the data element bound to the link.
       *
       * @return   The stroke dash type to use (empty string for none).
       */
      function calcLinkDash(d)
      {
         if (typeof displayProps[d.text] === "undefined")
         {
            return "8";
         }
         
         return displayProps[d.text]["stroke-dasharray"];
      }
      
      /**
       * Calculate the size of a text element and move the X and Y position to center using the
       * metrics.
       *
       * @param    d 
       *           This is the datam bound to the text element.
       */
      function calcLinkTextLoc(d)
      {
         // TODO: the next code dies, not sure why
         // obtain the bounding box for the text
         var bounds = this.getBBox();
         
         // shift the text to be in the center (horizontally) of the bounding box; the vertical
         // number we get seems to be near the bottom of the text, so the "box" doesn't make much
         // sense
         var textX = bounds.x - (bounds.width / 2);
         var textY = bounds.height / 2;
         
         // save it into the text attributes
         d3.select(this)
           .attr("x", textX)
           .attr("y", textY);
      }
      
      /**
       * Calculate the color of the star associated with nodes which have links external to other
       * programs.
       * The color of the link will be a gradient from red to green, from the number of hidden
       * links (all = red) to the number of remaining hidden links (all links visible = green).
       *
       * @param    n
       *           This is the datam bound to the star element.
       */
      function calcStarColor(n)
      {
         var hid = 0;
         for (var k = 0; k < n.hiddenLinks.length; k++)
         {
            hid += (n.hiddenLinks[k].hidden ? 1 : 0);
         }
         
         if (hid == 0)
         {
            return "green";
         }
         else if (hid == n.hiddenLinks.length)
         {
            return "red";
         }
         
         var x = hid / n.hiddenLinks.length;

         if (x > 0.5)
         {
            var g = Math.round(510 * x);

            return "#" + ("000000" + (0xFF0000 | g << 8).toString(16)).substr(-6);
         }
         else
         {
            var r = Math.round(510 * (1.0 - x));

           return "#" + ("000000" + (0x00FF00 | r << 16).toString(16)).substr(-6);
         }
      }
      
      /**
       * Calculate the points associated with the star, for nodes which have links to other 
       * programs.
       *
       * @param    d
       *           This is the datam bound to the star element.
       */
      function calcStarPoints(d)
      {
         var points = "";
         
         var tips = 7;
         var angle = (2 * Math.PI) / (2 * tips);
         
         var cx = d.bounds.x;
         var cy = d.bounds.y;
         
         if (d.shape == "circle")
         {
            cx = -(d.bounds.radius * Math.cos(0.25 * Math.PI));
            cy = -(d.bounds.radius * Math.sin(0.25 * Math.PI));
         }
         
         for (var i = 0; i < (2 * tips); i++)
         {
            var x = cx + Math.cos(i * angle) * (i % 2 == 0 ? 4 : 6);
            var y = cy + Math.sin(i * angle) * (i % 2 == 0 ? 4 : 6);

            points = points + ( i > 0 ? "," : "") + x + "," + y;
         }

         return points;
      }
      
      /**
       * Set the relative position of the text SVG element and the position/size of the rectangle
       * SVG element, based on the bounding box of the text element in its relation to the parent
       * (group) node.  These relative locations/sizes only need to be set once, so this function
       * would only be called during text node creation.
       * <p>
       * Inside this function, "this" refers to the text SVG element.  The algorithm in this
       * function is dependent upon the text node having the attribute "text-align" set to
       * "middle".
       *
       * @param    d 
       *           This is the data element bound to the node.
       */
      function calcBounds(d)
      {
         // obtain the bounding box for the text
         var bounds = this.getBBox();
         
         // shift the text to be in the center (horizontally) of the bounding box; the vertical
         // number we get seems to be near the bottom of the text, so the "box" doesn't make much
         // sense
         // TODO: for the current font height this textY calc (and the rectangle Y calc below
         //       and the circle CY calc below) happens to work but this is fragile and doesn't
         //       make sense (other more obvious approaches don't center the text vertically OR
         //       they result in the calculated circleIntersection() result being Y shifted away
         //       from the node
         var textX = bounds.x - (bounds.width / 2);
         var textY = bounds.height / 2;
         
         // save it into the text node attributes
         d3.select(this)
           .attr("x", textX)
           .attr("y", textY);
           
         d.bounds = {};
         d.contained = [];
         d.maxElementWidth  = 0;
         d.maxElementHeight = 0;
         d.minWidth         = 0;
         d.minHeight        = 0;
         // d.containment = true;
           
         if (d.shape == "rect")
         {
            // get the sibling SVG rectangle node
            var rect = d3.select(this.parentNode)
                         .select("rect");
                         
            d.bounds.x      = textX - TEXT_MARGIN;
            d.bounds.y      = bounds.y + textY - TEXT_MARGIN;
            d.bounds.width  = bounds.width + TEXT_MARGIN * 2;
            d.bounds.height = bounds.height + TEXT_MARGIN * 2;
              
            // center the rectangle around the text with a small margin; the
            rect.attr("x", d.bounds.x)
                .attr("y", d.bounds.y)  // TODO: see above
                .attr("width", d.bounds.width)
                .attr("height", d.bounds.height);
                
            // cache the rectangle node so that the bounding rectangle can be found later
            // during intersection processing (which happens during each tick)
            d.svgRect = rect;
         }
         
         if (d.shape == "circle")
         {
            // get the sibling SVG circle node
            var circle = d3.select(this.parentNode)
                           .select("circle");
              
            var radius = (Math.max(bounds.height, bounds.width) / 2) + (1.5 * TEXT_MARGIN);
            
            d.bounds.x = bounds.x;
            d.bounds.y = textY - TEXT_MARGIN;
            
            // center the circle around the text with a small margin; 
            circle.attr("cx", d.bounds.x)
                  .attr("cy", d.bounds.y)  // TODO: see above
                  .attr("r", radius);
                
            // cache the radius so that the bounding circle can be calculated later
            // during intersection processing (which happens during each tick)
            d.bounds.radius = radius;
            d.bounds.width = d.bounds.height = (radius * 2);
         }
      }
      
      /**
       * Recalculate the positions of all nodes and links in the simulation.  This is called for
       * each "tick" of the simulation.  The input positions will have changed based on the
       * forces operating on the simulation. This function calculates or translates values
       * on input and sets new values on output to control the look and feel of the model
       * visualization.
       * <p>
       * A big part of this code is to calculcate the intersection points between the nodes
       * and links, so that the links are attaching to the outside of the nodes. This allows
       * markers to be drawn making the visualization of the links much improved.
       */                                                
      function ticked()
      {
         // by the time this gets called, all of our active forces are already calculated for
         // this tick, this means that anything that we assign below will take precedence and
         // will act like a "very strong force"
         
         // implement a bounding box for those nodes that are contained; after they are pinned
         // in their snap point "jail", check to see if swapping snap points would lead to
         // a more stable layout (which will have better spacing and will minimize intersecting
         // links thus being more attractive and readable)
         node//.filter(isContained)
             .each(enforceBoundingBox)
             .each(swapSnapPoints);
             
         // we have to apply a translate() for each node so that the contained node's get their
         // (x, y) coordinates shifted to the correct values (we originally set them on a 
         // relative basis, in the one and only time we called calcBounds()); this shift has
         // to happen before we calculate the intersection points below; as an alternative, we
         // could manually calculate and set (using attr()) all the contained locations to get
         // the same effect; no scaling is needed here because this is just a coordinate shift
         node.attr("transform", function(d)
         {
            return (d && d.x && d.y) ? "translate(" + d.x + "," + d.y + ")" : "";
         });
         
         // the first step here calcuates the intersection points and then the attribute setters
         // use the calculated results; in case there is a problem with the calculation, we
         // use the fallback values that are given on input
         link.each(findIntersectionPoints)
             // "line" approach
             //.attr("x1", function(d) { return (d.interSrc) ? d.interSrc.x : d.source.x; })
             //.attr("y1", function(d) { return (d.interSrc) ? d.interSrc.y : d.source.y; })
             //.attr("x2", function(d) { return (d.interTar) ? d.interTar.x : d.target.x; })
             //.attr("y2", function(d) { return (d.interTar) ? d.interTar.y : d.target.y; });
             // "polyline" approach        
             .attr("points", function(d)
             {
                var sx = (d.interSrc) ? d.interSrc.x : d.source.x;
                var sy = (d.interSrc) ? d.interSrc.y : d.source.y;
                var tx = (d.interTar) ? d.interTar.x : d.target.x;
                var ty = (d.interTar) ? d.interTar.y : d.target.y;
                
                return sx + "," + sy + " " +
                       ((sx + tx) / 2) +  "," + ((sy + ty) / 2) + " " +
                       tx + "," + ty;
             });             
      }
      
      /**
       * Re-calculate the dimension of this container's grid.  This is done by computing the 
       * minimum width and height for each column and row, so that all elements on that column or
       * row will fit (with some extra added space).
       * Once these are computed, the container's width and height will be adjusted only if it
       * has changed; in this case, the parent container's grid will be re-computed (up to the
       * root container). 
       *
       * @param    d 
       *           This is the data element bound to the node.
       */
      function calcMinGrid(d)
      {
         if (!(d.contained && d.contained.length >= 1))
         {
            return;
         }
         
         // sort elements by their snap index, to be able to compute the grid
         var dnodes = d.contained.slice();
         dnodes.sort(function(d1, d2) { return d1.snapIndex - d2.snapIndex; });

         var dwidth = SIDE_BUFFER;
         var dheight = (d.bounds.headerHeight ? d.bounds.headerHeight : 0);

         // do for the columns
         for (var c = 0; c < d.cols; c++)
         {
            var maxColWidth = Number.MIN_VALUE;
            var prevColWidth = Number.MIN_VALUE;
            
            // compute the maximum width for this column
            for (var r = 0; r < d.rows; r++)
            {
               var idx = r * d.cols + c;
               
               if (dnodes[idx])
               {
                  maxColWidth = Math.max(maxColWidth, dnodes[idx].bounds.width);
               }

               prevColWidth = Math.max(prevColWidth, d.snapPoints[idx].colWidth);
            }
            
            maxColWidth *= (1 + WIDTH_BUFFER);
            
            // leave room for the links...
            if (!d.container && maxColWidth < d.maxElementWidth / 2)
            {
               maxColWidth *= 1.5;
            }

            var delta = prevColWidth - maxColWidth;
            dwidth += maxColWidth;

            if (delta === 0)
            {
               continue;
            }
            
            // the found width is the new column's width; adjust the snap points accordingly
            for (var r = 0; r < d.rows; r++)
            {
               var idx = r * d.cols + c;

               d.snapPoints[idx].colWidth = maxColWidth;
               d.snapPoints[idx].offsetX = dwidth - maxColWidth + d.bounds.x;
               d.snapPoints[idx].x = dwidth - maxColWidth + d.bounds.x + maxColWidth / 2;
            }

            // all the right-side columns must be adjusted to decrease the X offset
            var dwidth2 = dwidth + d.bounds.x;
            for (var cr = c + 1; cr < d.cols; cr++)
            {
               var maxw2 = 0;
               for (var r = 0; r < d.rows; r++)
               {
                  var idx = r * d.cols + cr;

                  maxw2 = Math.max(d.snapPoints[idx].colWidth, maxw2);
                  
                  d.snapPoints[idx].offsetX = dwidth2;
                  d.snapPoints[idx].x = dwidth2 + d.snapPoints[idx].colWidth / 2;
               }
               
               dwidth2 += maxw2;
            }
         }

         // do for the rows
         for (var r = 0; r < d.rows; r++)
         {
            var maxRowHeight = Number.MIN_VALUE;
            var prevRowHeight = Number.MIN_VALUE;
            
            // compute the maximum height for this row
            for (var c = 0; c < d.cols; c++)
            {
               var idx = r * d.cols + c;
               
               if (dnodes[idx])
               {
                  maxRowHeight = Math.max(maxRowHeight, dnodes[idx].bounds.height);
               }

               prevRowHeight = Math.max(prevRowHeight, d.snapPoints[idx].rowHeight);
            }
            
            maxRowHeight *= (1 + HEIGHT_BUFFER);
            
            // some extra space to better see the links...
            if (maxRowHeight < d.maxElementHeight / 2)
            {
               maxRowHeight *= 1.5;
            }
            
            var delta = prevRowHeight - maxRowHeight;
            dheight += maxRowHeight;

            if (delta === 0)
            {
               continue;
            }
            
            // the found height is the new row's height; adjust the snap points accordingly
            for (var c = 0; c < d.cols; c++)
            {
               var idx = r * d.cols + c;

               d.snapPoints[idx].rowHeight = maxRowHeight;
               d.snapPoints[idx].offsetY = dheight - maxRowHeight + d.bounds.y;
               d.snapPoints[idx].y = dheight - maxRowHeight + d.bounds.y + maxRowHeight / 2;
            }

            // all the bellow rows must be adjusted to decrease the Y offset
            var dheight2 = dheight + d.bounds.y;
            for (var br = r + 1; br < d.rows; br++)
            {
               var maxh2 = 0;
               for (var c = 0; c < d.cols; c++)
               {
                  var idx = br * d.cols + c;

                  maxh2 = Math.max(maxh2, d.snapPoints[idx].rowHeight); 

                  d.snapPoints[idx].offsetY = dheight2;
                  d.snapPoints[idx].y = dheight2 + d.snapPoints[idx].rowHeight / 2;
               }
               
               dheight2 += maxh2;
            }
         }

         dwidth = Math.max(dwidth, d.bounds.headerWidth);

         if (dwidth != d.bounds.width || dheight != d.bounds.height)
         {
            // adjust the container's width and height to the new values
            d.bounds.height = dheight;
            d.bounds.width = dwidth;

            node.filter(function(n) { return (d === n); })
                .select("rect")
                .attr("height", dheight)
                .attr("width", dwidth);

            if (d.container)
            {
               // propagate this width/height change to the parent...
               calcMinGrid(d.container);
            }
         }
      }
      
      /**
       * Calculate the largest width and height of any target node that is related via a
       * "contains" edge as well as tracking the number of contained nodes. Using these
       * values, container nodes (rectangles only right now) are resized to include enough
       * space such that the contained nodes can be positioned inside and bounded to that
       * area.  The calculation results are stored in the node's datum or are saved into the
       * SVG element's attributes.
       */
      function calcContainment()
      {
         var link   = null;
         var vertex = null;
         var src    = null;
         var tar    = null;
         
         // cross-link all container/containee relationships for the entire graph
         for (var i = 0; i < data.links.length; i++)
         {
            link = data.links[i];
            
            if (link.text == "contains")
            {
               if (typeof link.source === 'string' || link.source instanceof String)
               {
                  // links that are excluded from the graph have a string source member that
                  // is the id member of the node object
                  src = nodeMap[link.source];
                  tar = nodeMap[link.target];
               }
               else
               {
                  // links that are included in the graph have their source member transformed
                  // to be a direct link to the node object
                  src = link.source;
                  tar = link.target;
               }
               
               // console.log("calcContainment CONTAINS target " + tar.text + " " + fwd.ast.tokenName(tar.type) + " " + tar.id +
               //             " source " + src.text + " " + fwd.ast.tokenName(src.type) + " " + src.id);
               
               src.contained.push(tar);
               tar.container = src;
               tar.snapIndex = src.contained.length - 1;
            }
         }
         
         // now that all containment relationships are cross-linked, we need to calculate the
         // containment depth for each node
         calcContainmentDepth();
         
         // process all containers, from the most deeply nested container to the least nested
         // and for each one accumulate the largest bounding box that can contain any of the
         // contained nodes; this must be done in this order so that any sub-containers will
         // already have their maximum size determined; after the maximum element size is
         // known, calculate the container's size
         for (var k = maxDepth; k >= 0; k--)
         {
            for (var j = 0; j < data.nodes.length; j++)
            {
               vertex = data.nodes[j];
               
               // console.log("calcContainment main sizing loop " + k + " " + vertex.text + " " + fwd.ast.tokenName(vertex.type) + " " + vertex.id +
               //             " contained " + vertex.contained.length + " depth " + vertex.containerDepth);
               
               // only process containers that are at the right depth, ignore non-containers
               if (vertex.containerDepth == k && vertex.contained.length > 0)
               {
                  // console.log("calcContainment " + vertex.text + " " + fwd.ast.tokenName(vertex.type) + " " + vertex.id + "; contains " + vertex.contained.length );
                  
                  // calculate the element sizes
                  updateMaxElementSize(vertex);
                  
                  // console.log("calcContainment maxElementWidth " + vertex.maxElementWidth + "; maxElementHeight " + vertex.maxElementHeight); 
                  
                  // now calculate the container size itself
                  calcContainerSize(vertex);
                  
                  // printBounds(vertex.bounds, "calcContainment");
               }
            }
         }
         
         // now that all containers are calculated, calculate the implicit "top-level" container
         // which is the SVG itself
         calcNonContainedNodes();

         for (var k = maxDepth; k >= 0; k--)
         {
            for (var j = 0; j < data.nodes.length; j++)
            {
               vertex = data.nodes[j];
               
               // only process containers that are at the right depth, ignore non-containers
               if (vertex.containerDepth == k && vertex.contained.length > 0)
               {
                  calcMinGrid(vertex);
               }
            }
         }
      }
      
      /**
       * Calculate (and record) the number of containers in each node's containment hierarchy.
       */
      function calcContainmentDepth()
      {
         var node = null;
         
         for (var k = 0; k < data.nodes.length; k++)
         {
            node = data.nodes[k];
            
            var cntr  = (typeof node.container === "undefined") ? null : node.container;
            var depth = 0;
            
            while (cntr != null)
            {
               depth++;
               cntr = (typeof cntr.container === "undefined") ? null : cntr.container;
            }
         
            node.containerDepth = depth;
            
            // remember the maximum depth for the graph
            maxDepth = Math.max(maxDepth, depth);
         }
      }
      
      /**
       * Report if the given node is to be rendered as a contained node.
       *
       * @param    d
       *           The node's datum to check.
       *
       * @return   <code>true</code> if the node is rendered as contained.
       */
      function isContained(d)
      {
         return (d.container && d.container.containment) ? d.container.containment : false;
      }
      
      /**
       * Calculate the size (width and height) of the container based on the maximum contained
       * element size.
       *
       * @param    d
       *           The container node's datum.
       */
      function calcContainerSize(d)
      {
         calcGrid(d, d.contained.length);
         
         // remember the original width and height which is now the header's dimension 
         d.bounds.headerHeight = d.bounds.height;
         d.bounds.headerWidth = d.bounds.width;
         
         // calculate some buffer space
         var horizBuffer = (d.maxElementWidth * WIDTH_BUFFER);
         var vertBuffer = (d.maxElementHeight * HEIGHT_BUFFER);
         
         // make sure to accommodate the largest width node which might be ourselves
         d.bounds.width = Math.max(d.bounds.width, (d.maxElementWidth + horizBuffer) * d.cols);  
         
         // calculate the height of the container with room for the header
         d.bounds.height = d.bounds.height + ((d.maxElementHeight + vertBuffer) * d.rows);
                                                                                                                      
         console.log("calcContainerSize() hbuf " + horizBuffer + "; vbuf " + vertBuffer + "; maxwidth " + d.bounds.width + "; minWidth " + d.minWidth + "; w% " + (d.minWidth / d.bounds.width) + "; maxHeight " + d.bounds.height + "; minHeight " + d.minHeight + "; h% " + (d.minHeight / d.bounds.height));  
         
         var x1 = d.bounds.x + SIDE_BUFFER;                                                                                                                                                                                                                                                              
         var x2 = x1 + d.bounds.width - SIDE_BUFFER;
         var y1 = d.bounds.y + d.bounds.headerHeight;
         var y2 = d.bounds.y + d.bounds.height - SIDE_BUFFER;
         
         // calculate the internal locations to which elements should be attracted 
         d.snapPoints = createSnapPoints(d.rows, d.cols, x1, y1, x2, y2);
         
         // circles and other shapes are poor containers, so we don't use them this way
         if (d.shape == "rect")
         {
            d.containment = true; 
            
            // change the SVG attributes so they draw correctly, because we aren't being called
            // in ticked() or some other callback, we can't use d3.select(this) anymore; instead
            // we use this trick to get the current node to match the datum
            node.filter(function(n) { return (d === n); })
              .select("rect")
              .attr("width", d.bounds.width)                           
              .attr("height", d.bounds.height);
         }
         else
         {
            d.containment = false;
         }
      }
      
      /**
       * Calculate the snap points for the nodes that are not otherwise contained. This includes
       * any containers that are not themselves contained and any non-container nodes that are
       * not contained.  The resulting state will be stored in a non-node container object that
       * represents the containment values for the outermost level of the SVG.  It is setup to
       * mimic the same state as a regular node-based container.
       */
      function calcNonContainedNodes()
      {
         var node = null;
         
         // TODO: handle the case where there is not enough space in the width and height of the
         //       viewport to store all containers
         
         outer.x                   = 0;
         outer.y                   = 0;
         outer.bounds              = {};
         outer.bounds.x            = 0;
         outer.bounds.y            = 0;
         outer.bounds.width        = width;
         outer.bounds.height       = height;
         outer.bounds.headerHeight = 0;
         outer.maxElementWidth     = 0;
         outer.maxElementHeight    = 0;
         outer.containment         = true;
         outer.contained           = [];
         
         for (var i = 0; i < data.nodes.length; i++)
         {
            node = data.nodes[i];
            
            if (!isContained(node))
            {
               outer.maxElementWidth  = Math.max(outer.maxElementWidth, node.bounds.width);
               outer.maxElementHeight = Math.max(outer.maxElementHeight, node.bounds.height);
               
               // limit snap positions to the container elements
               // if (node.containment)
               {
                  outer.contained.push(node);
                  node.container = outer;
                  node.snapIndex = outer.contained.length - 1;
                  node.drive     = true;
               }
               
               // TODO: non-container nodes should be gently pulled toward one cardinal
               //       direction to ensure they don't overlap the containers, but we don't want
               //       the strength of this to be strong enough to pull the containers out
               //       of their containment box
            }
         }
         
         calcGrid(outer, outer.contained.length);
         
         var availWidth  = width - OUTER_BUFFER;
         var availHeight = height - OUTER_BUFFER;
         
         var neededWidth  = outer.maxElementWidth * outer.cols;
         var neededHeight = outer.maxElementHeight * outer.rows;
         
         var x1 = OUTER_BUFFER;
         var x2 = x1 + availWidth;
         var y1 = OUTER_BUFFER;
         var y2 = y1 + availHeight;
         
         if (neededWidth > availWidth || neededHeight > availHeight)
         {
            // we are going to display some of our contained elements outside of the viewport,
            // just center everything in both dimensions
            var cx    = width / 2;
            var cy    = height / 2;
            var horiz = neededWidth / 2;
            var vert  = neededHeight / 2;
            
            x1 = cx - horiz;
            x2 = cx + horiz;
            y1 = cy - vert;
            y2 = cy + vert;
         }
         else
         {
            // nice! our contained nodes comfortably fit in the SVG viewport
            x1 = OUTER_BUFFER;
            x2 = x1 + availWidth;
            y1 = OUTER_BUFFER;
            y2 = y1 + availHeight;
         }
         
         // calculate the internal locations to which elements should be attracted 
         outer.snapPoints = createSnapPoints(outer.rows, outer.cols, x1, y1, x2, y2);
         
         calcMinGrid(outer);
      }
      
      /**
       * Calculate the number of rows and columns needed to hold the given number of elements
       * assuming that the resulting grid shape should be as square as possible. Bias toward
       * wider rows instead of taller columns if a square shape cannot be achieved.
       *
       * @param    d
       *           The object to update.  The rows and cols members will be added.
       * @param    num
       *           Number of elements to contain.
       */
      function calcGrid(d, num)
      {
         // use a square shape to make it easier to route links around this cluster
         var sqrt = Math.sqrt(num);
         
         // decide on the number of rows and columns in our matrix
         d.cols = Math.ceil(sqrt);
         d.rows = Math.round(sqrt);
         
         // extend the element array to hold all possible element positions
         d.contained.length = d.cols * d.rows;
         
         // we need to calculate the maximum possible row width and column height which will
         // form the minimum actual size that can be allocated
         
         var widths  = [];
         var heights = [];
         
         // populate arrays of the contained element widths/heights
         for (var q = 0; q < d.contained.length; q++)
         {
            if (typeof d.contained[q] !== "undefined")
            {            
               widths.push(d.contained[q].bounds.width);
               heights.push(d.contained[q].bounds.height);
            }
         }
         
         // sort in ascending order
         widths.sort(sortNumberAsc);
         heights.sort(sortNumberAsc);
            
         // take the elements from the end of the array, these are the largest elements
         var swidths  = widths.slice(-(d.cols));
         var sheights = heights.slice(-(d.rows));
                                                       
         swidths.forEach(function(e) { d.minWidth += (WIDTH_BUFFER + e); });
         sheights.forEach(function(e) { d.minHeight += (HEIGHT_BUFFER + e); });
      }                                                  
      
      /**
       * Comparator to sort an array of numbers in place in ascending order.
       *
       * @param    a                                                                    
       *           A number element in the array.
       * @param    b
       *           A number element in the array.
       *
       * @return   Negative number if b is greater than a, 0 if they are equal and a positive
       *           number when a is gretaer than b.
       */
      function sortNumberAsc(a, b)
      {
         return a - b; 
      }
      
      /**
       * Iterate through the container's elements and update the maximum element size.
       *
       * @param    d
       *           The container node's datum to process.
       */
      function updateMaxElementSize(d)
      {
         for (var i = 0; i < d.contained.length; i++)
         {
            if (typeof d.contained[i] !== "undefined")
            {
               maxElementSize(d, d.contained[i]);
            }
         }
      }
      
      /**
       * Update the maximum contained element size given the source container and one of the
       * contained elements.  Both max width and max height is updated.
       *
       * @param    src
       *           The container whole state is updated.
       * @param    tar
       *           The element to check.
       */
      function maxElementSize(src, tar)
      {
         src.maxElementWidth  = Math.max(src.maxElementWidth, tar.bounds.width);
         src.maxElementHeight = Math.max(src.maxElementHeight, tar.bounds.height);
      }                            
      
      /**                                                     
       * Recalculate and assign the location for contained nodes such that they render fully
       * inside an element-specific "jail" within their container. This is meant to be called
       * each tick and it will respond to the simulation's changes while still staying in the
       * hard boundary of the container's internal jail.  The jail is a "cell" centered on
       * one of the snap points in the container and it is sized large enough to fit the
       * the largest contained element (which may not be the same size as the current element).
       * <p>
       * This should only be called for contained nodes.
       *
       * @param    d
       *           The node's datum to check.
       */
      function enforceBoundingBox(d)
      { 
         if (!d || !d.container)
         {
            return;
         }
         
         var jail = calcElementJail(d);
         
         // the coordinates are forced the same way for both rect and circle elements, but the
         // jail will be computed differently for rect and circle
         d.x = Math.max(jail.minX, Math.min(jail.maxX, jail.itemX)) - jail.adjX;
         d.y = Math.max(jail.minY, Math.min(jail.maxY, jail.itemY)) - jail.adjY;
      }
      
      /**
       * Detect if two sibling nodes should swap snap points and implement the swap when
       * found.  Contained nodes that are centered around snap points (and which have an
       * individual bounding box around that point) might have links pulling them toward a side
       * of their bounding box.  This information can be used to detect if a swap may make
       * sense.
       * <p>
       * If two adjacent contained nodes are both pulling toward each other such that they are
       * each trying to "move into" the other's bounding box, then we will oblige them by
       * swapping points.
       * <p>
       * The idea is to obtain a more stable layout which will have better spacing, will
       * minimize intersecting links and thus be more attractive and readable.
       * <p>
       * This should only be called for contained nodes and only after enforceBoundingBox()
       * has already been executed for this tick.
       *
       * @param    d
       *           The node's datum to check.
       */
      function swapSnapPoints(d)
      {
         //console.log("swapSnapPoints START");
         
         // find the snap point (don't try the short cut test for undefined, because snapIndex
         // is a number so 0 will return as false bypassing this block for the first element)
         if (typeof d.snapIndex !== "undefined")
         {
            var list = d.container.snapPoints;
            
            // a single contained element can't swap
            if (list.length == 1)
               return;
            
            var idx     = d.snapIndex;
            var pt      = list[idx];
            var against = calcPushingAgainst(d);
            
            // calculate which border cells we may be in
            var leftColumn  = ((pt.column % d.container.cols) == 0);
            var rightColumn = (d.container.cols == 1) ||
                              ((pt.column % (d.container.cols - 1)) == 0);
            var topRow      = (pt.row == 0);
            var bottomRow   = (d.container.rows == 1)||
                              ((pt.row % (d.container.rows - 1)) == 0);
            
            // console.log("borders left " + leftColumn + "; top " + topRow + "; right " + rightColumn + "; bottom " + bottomRow);
            
            var adjacent   = null;
            var adjAgainst = null
            
            // check adjacent cells (which can only be an interior boundary) for their "pushing
            // against" values
            if (!leftColumn && against.left)
            {
               // same row, one column left
               adjacent   = findNodeForCell(d.container, pt.row, pt.column - 1);
               adjAgainst = calcPushingAgainst(adjacent);
               
               if (adjacent === null || adjAgainst.right)
               {
                  // swap
                  swapPointsWorker(d, adjacent, pt.row, pt.column - 1);
               }
            }
            if (!rightColumn && against.right)
            {
               // same row, one column right
               adjacent   = findNodeForCell(d.container, pt.row, pt.column + 1);
               adjAgainst = calcPushingAgainst(adjacent);
               
               if (adjacent === null || adjAgainst.left)
               {
                  // swap
                  swapPointsWorker(d, adjacent, pt.row, pt.column + 1);
               }
            }
            if (!topRow && against.top)
            {
               // row above, same column
               adjacent   = findNodeForCell(d.container, pt.row - 1, pt.column);
               adjAgainst = calcPushingAgainst(adjacent);
               
               if (adjacent === null || adjAgainst.bottom)
               {
                  // swap
                  swapPointsWorker(d, adjacent, pt.row - 1, pt.column);
               }
            }
            if (!bottomRow && against.bottom)
            {
               // row below, same column
               adjacent   = findNodeForCell(d.container, pt.row + 1, pt.column);
               adjAgainst = calcPushingAgainst(adjacent);
               
               if (adjacent === null || adjAgainst.top)
               {
                  // swap
                  swapPointsWorker(d, adjacent, pt.row + 1, pt.column);
               }
            }
         }
      }
      
      /**
       * Swap the snap points for two contained elements or if the second element is
       * <code>null</code> then move the first element into the specified target cell.
       *
       * @param    pt1
       *           First contained element. Must not be <code>null</code>.
       * @param    pt2
       *           Second contained element, may be <code>null</code>.
       * @param    targetRow
       *           When the second point is <code>null</code>, this is the target cell's row.
       * @param    targetCol
       *           When the second point is <code>null</code>, this is the target cell's column.
       */
      function swapPointsWorker(pt1, pt2, targetRow, targetCol)
      {
         var cnt = pt1.container;
         
         //console.log("SWAP! " + pt1.text + ((pt2 == null) ? (" into row " + targetRow + "and col " + targetCol)
         //                                                 : (" for " + pt2.text)));
         
         if (pt2 == null)
         {
            // move into an already empty cell
            var idx = cellToIndex(cnt, targetRow, targetCol);
            
            if (idx >= 0 && idx < cnt.contained.length)
            {
               if (typeof cnt.contained[idx] === "undefined" || cnt.contained[idx] == null)
               {
                  cnt.contained[idx] = pt1;
                  pt1.snapIndex = idx;
               }
               else
               {
                  console.log("swapPointsWorker() attempt to place " + pt1.text + " into " +
                              " non-empty index " + idx + " where " + cnt.contained[idx].text +
                              " already exists.");
               }
            }
         }
         else
         {
            var idx1 = pt1.snapIndex;
            var idx2 = pt2.snapIndex;
            
            cnt.contained[idx1] = pt2;
            pt2.snapIndex = idx1;
            
            cnt.contained[idx2] = pt1;
            pt1.snapIndex = idx2;
         }
         
         enforceBoundingBox(pt1);
         
         if (pt2 != null)
         {
            enforceBoundingBox(pt2);
         }

         calcMinGrid(cnt);
      }
      
      /**
       * Obtain the array index that matches the specified cell of the given container.
       *
       * @param    d
       *           The container node.
       * @param    row
       *           The row of the cell.
       * @param    col
       *           The column of the cell.
       *
       * @return   The array index.
       */
      function cellToIndex(d, row, col)
      {
         return (row * d.cols) + col;
      }
      
      /**
       * Obtain the node that resides in the specified cell of the given container.
       *
       * @param    d
       *           The container node.
       * @param    row
       *           The row of the cell.
       * @param    col
       *           The column of the cell.
       *
       * @return   The node in the specified cell or <code>null</code> if there is no such
       *           cell or if the cell is empty.
       */
      function findNodeForCell(d, row, col)
      {
         var node = null;
         
         var idx = cellToIndex(d, row, col);
         
         if (idx >= 0 && idx < d.contained.length)
         {
            if (typeof d.contained[idx] !== "undefined")
            {
               node = d.contained[idx];
            }
         }
         
         return node;
      }
      
      /**
       * Calculate the details of which boundaries (if any) of the element-specific jail the
       * given node is pushing against. Contained nodes that are centered around snap points
       * (and which have an individual bounding box around that point) might have links pulling
       * them toward a side of their bounding box (the "jail").
       * <p>
       * This should only be called for contained nodes and only after enforceBoundingBox()
       * has already been executed for this tick.
       *
       * @param    d
       *           The node's datum to check.
       *
       * @return   An object with jail boundaries (minX, maxX, minY, maxY), actual element
       *           boundaries (x1, y1, x2, y2) and the jail sides which are being pushed
       *           against (Boolean members left, right, top, bottom). <code>null</code> is
       *           returned if the cell is empty (which can happen because there can be
       *           more cells than elements).
       */
      function calcPushingAgainst(d)
      {
         var against = null;
         
         // find the snap point (don't try the short cut test for undefined, because snapIndex
         // is a number so 0 will return as false bypassing this block for the first element)
         if (d !== null && typeof d.snapIndex !== "undefined")
         {
            var list = d.container.snapPoints;
            
            // a single contained element can't swap
            if (list.length == 1)
               return;
            
            var idx  = d.snapIndex;
            var pt   = list[idx];
            
            // these are jail boundary points
            var minX = d.container.x + pt.offsetX;
            var minY = d.container.y + pt.offsetY;
            var maxX = minX + pt.colWidth;
            var maxY = minY + pt.rowHeight;
            
            // this is the element's internal bounds and relative position
            var item = d.bounds;
            
            var x1 = null;
            var y1 = null;
            
            if (d.shape == "circle")
            {
               // TODO: is this right?
               // console.log("calcPushingAgainst d.x " + d.x + "; d.y " + d.y + "; item.x " + item.x + "; item.y " + item.y + "; radius " + item.radius);
               x1 = d.x + item.x - item.radius;
               y1 = d.y + item.y - item.radius;
            }
            else
            {
               // these are the actual boundary points of the element
               x1 = d.x + item.x;
               y1 = d.y + item.y;
            }
            
            var x2 = x1 + item.width;
            var y2 = y2 + item.height;
            
            // calculate which sides we are "pushing against"
            var left   = (x1 <= minX);
            var top    = (y1 <= minY);
            var right  = (x2 >= maxX);
            var bottom = (y2 >= maxY);
            
            // console.log("swap " + d.text + " d.x " + (d.x + item.x) + "; minX" + minX + "; d.y " + (d.y + item.y) + "; minY" + minY +
            //             "; d.x2 " + (d.x + item.x + item.width) + "; maxX " +  maxX +
            //             "; d.y2 " + (d.y + item.y + item.height) + "; maxY " + maxY)
            // console.log("intersect left " + left + "; top " + top + "; right " + right + "; bottom " + bottom);
            
            against =
            {
               "minX"   : minX,
               "minY"   : minY,
               "maxX"   : maxX,
               "maxY"   : maxY,
               "x1"     : x1,
               "y1"     : y1,
               "x2"     : x2,
               "y2"     : y2,
               "left"   : left,
               "top"    : top,
               "right"  : right,
               "bottom" : bottom,
            };
         }
         
         return against;
      }
      
      /**
       * Calculate the element-specific "jail" within its container. The jail is a "cell"
       * (intersection of a specific row and column) centered on one of the snap points in the
       * container and it is sized large enough to fit the the largest contained element
       * (which may not be the same size as the current element).
       * <p>
       * This should only be called for contained nodes.
       *
       * @param    d
       *           The node's datum to check.
       *
       * @return   The calculated element bounding box.
       */
      function calcElementJail(d)
      {
         // box is the containing element and item is the contained element
         var box  = d.container.bounds;
         var item = d.bounds;
         var pt   = null;
         
         // find the snap point (don't try the short cut test for undefined, because snapIndex
         // is a number so 0 will return as false bypassing this block for the first element)
         if (typeof d.snapIndex !== "undefined")
         {
            var i = d.snapIndex; 
            var l = d.container.snapPoints;
            
            pt = l[i];
         }
         
         // the bounds instances represents the internal rectangle x, y, width and height which
         // is relative to the containing group node
         
         // d.container.x (and y) are the model-controlled d.x and d.y which is the location
         // of the group node; this is used later for translate() to move all group node
         // children to the right coordinates
         
         // the group node children have a relative x and y that are not necessarily 0 which
         // means they draw slightly off the group node coordinates; we have to account for
         // these differences in the min/max calcs below
         
         // if we have a snap point, find the internal snap-point x and y; otherwise fall back
         // using box.x/box.y which will yield the outside boundaries for the entire container
         var boxX = d.container.x + ((pt) ? pt.offsetX : box.x);
         var boxY = d.container.y + ((pt) ? pt.offsetY : box.y);
         
         var itemX = d.x + item.x;
         var itemY = d.y + item.x;
         
         // if we have a snap point, calculate the min/max based on the internal snap point;
         // otherwise using the box and buffer values will yield the outside boundaries for the
         // entire container
         var minX = boxX + ((pt) ? 0 : SIDE_BUFFER);
         var minY = boxY + ((pt) ? 0 : box.headerHeight);
         var maxX = boxX + ((pt) ? pt.colWidth  : box.width - SIDE_BUFFER) - item.width;
         var maxY = boxY + ((pt) ? pt.rowHeight : box.height - SIDE_BUFFER) - item.height;
         
         var isCircle = (d.shape == "circle");
         
         // for circle cases, the minimum and maximum coordinates must be expanded with its radius
         // it is required because we want to enforce the coordinates of the circle's center, 
         // and not top-left
         
         var jail =
         {
            "minX"  : minX + (isCircle ? item.radius : 0),
            "minY"  : minY + (isCircle ? item.radius : 0),
            "maxX"  : maxX + (isCircle ? item.radius : 0),
            "maxY"  : maxY + (isCircle ? item.radius : 0),
            "itemX" : itemX,
            "itemY" : itemY,
            "adjX"  : item.x,
            "adjY"  : item.y
         };
         
         return jail;
      }
      
      /**
       * Split the given rectangle into an evenly spaced set of internal snap points where
       * elements should be positioned so that if each element is assigned a different point
       * there would be no overlap.
       *
       * @param    rows
       *           The number of rows to split the height into.
       * @param    cols
       *           The number of columns to split the width into.
       * @param    x1
       *           Left coordinate.
       * @param    y1
       *           Top coordinate.
       * @param    x2
       *           Right side coordinate.
       * @param    y2
       *           Bottom coordinate.
       *
       * @return   The array of points.
       */
      function createSnapPoints(rows, cols, x1, y1, x2, y2)
      {
         var points = [];
         
         var colWidth  = (x2 - x1) / cols;
         var rowHeight = (y2 - y1) / rows;
         var colMid    = colWidth / 2;
         var rowMid    = rowHeight / 2;
         
         for (var r = 0; r < rows; r++)
         {
            for (var c = 0; c < cols; c++)
            {
               var x = x1 + Math.round(c * colWidth);
               var y = y1 + Math.round(r * rowHeight);
               
               // the points report their x and y coordinates at the mid-point because the
               // contained element should be centered; we store some additional information
               // to make related downstream bounding box calculations easier
               points.push(
               {
                  "x"        : x + colMid,
                  "y"        : y + rowMid,
                  "offsetX"  : x,
                  "offsetY"  : y,
                  "row"      : r,
                  "column"   : c,
                  "colWidth" : colWidth,
                  "rowHeight": rowHeight
               }); 
            }
         }
         
         return points;
      } 
      
      /**
       * Apply a force to modify the xand y velocity for the given node based on a snap point
       * to which the node should be attracted.
       *
       * @param    alpha
       *           The energy in the system for this tick.
       */
      function forceSnap(alpha)
      {                                   
         for (var i = 0; i < data.nodes.length; i++)
         {
            var node  = data.nodes[i];
            var drive = (node.drive) ? node.drive : false;
            
            forceSnapWorker(node, drive);
         }
      }
      
      /**
       * Apply a force to modify the xand y velocity for the given node based on a snap point
       * to which the node should be attracted.
       *
       * @param    d
       *           The node being processed.
       * @param    drive                           
       *           <code>true</code> if the position should be hard coded or <code>false</code>
       *           to just modify the velocity.
       */
      function forceSnapWorker(d, drive)
      {                                   
         var p = lookupSnapPoint(d);
      
         if (p)
         {
            // TODO: is this right?
            
            if (d.shape == "circle")
            {
               // console.log("forceSnapWorker d.width " + d.bounds.width + "; d.height " + d.bounds.height + "; p.x " + p.x + "; p.y " + p.y);
            }
            var xOffset = Math.round(d.bounds.width / 2);
            var yOffset = Math.round(d.bounds.height / 2);
            var targetX = p.x - xOffset;
            var targetY = p.y - yOffset;
                                                          
            if (drive)
            {
               // don't just nudge the velocity, drive it
               if (d.shape == "circle")
               {
                  // console.log("forceSnapWorker d.x " + d.x + "; d.y " + d.y + "; targetX " + targetX + "; targetY " + targetY + "; p.x " + p.x + "; p.y " + p.y + "; xOff " + xOffset + "; yOff " + yOffset);
                  d.x = targetX;
                  d.y = targetY;
               }
               else
               {
                  // good old rectangles
                  d.x = targetX;
                  d.y = targetY;
               }
            }
            else
            {
               // if we multiply by alpha here we are reducing the effect of the force so
               // much that it is worthless; the .3 can be considered a constant "strength"
               // value
               if (d.shape == "circle")
               {
                  // console.log("forceSnapWorker d.x " + d.x + "; d.y " + d.y + "; targetX " + targetX + "; targetY " + targetY + "; p.x " + p.x + "; p.y " + p.y + "; xOff " + xOffset + "; yOff " + yOffset);
                  d.vx += (targetX - d.x) * .3;
                  d.vy += (targetY - d.y) * .3;
               }
               else
               {
                  d.vx += (targetX - d.x) * .3; // * alpha;
                  d.vy += (targetY - d.y) * .3; // * alpha;
               }
            }
         }
      }
      
      /**
       * Calculate the snap point to which the given node should be attracted. The point will
       * be in absolute coordinates instead of relative to the container node (if the node
       * as a contained node).
       *
       * @param    d
       *           The link datum on which to base the calculation.
       *
       * @return   The point (will have x and y members).
       */
      function lookupSnapPoint(d)
      {
         var point = null;
         
         if (d.container)
         {
            // contained nodes (any level)
            if (d.snapIndex)
            {
               var i = d.snapIndex; 
               var l = d.container.snapPoints;
               var x = l[i].x + d.container.x;
               var y = l[i].y + d.container.y;
               
               point = { "x": x, "y": y };
            }
         }
         // else
         // {
         //    // top level nodes that aren't contained
         //    console.log("missing d.container " + d.text + " type " + d.type);
         // }
         
         return point;
      }
      
      /**
       * Drive the calculation for the node and line intersection calculation. This will save
       * off the results into the link datum on success.
       *
       * @param    d
       *           The link datum on which to base the calculation.
       */
      function findIntersectionPoints(d)
      {
         if (d && d.target && d.source)
         {
            var bounds = null;
            
            if (d.source.shape == "rect")
            {
               // calculate the source rectangle
               bounds = rectBounds(d.source.svgRect, d.source.x, d.source.y);
               
               // use the center point in the source rectangle and the link end point
               var sourcePoint = { "x": bounds.cx, "y": bounds.cy };
               var targetPoint = { "x": d.target.x, "y": d.target.y };
               
               // calculate the intersection with the mid-point of the closest side (source node)
               d.interSrc = rectIntersection(sourcePoint, targetPoint, bounds);
            }
            
            if (d.target.shape == "rect")
            {
               // calculate the target rectangle
               bounds = rectBounds(d.target.svgRect, d.target.x, d.target.y);
               
               // use the center point in the target rectangle and the link start point
               var sourcePoint = { "x": d.source.x, "y": d.source.y };
               var targetPoint = { "x": bounds.cx, "y": bounds.cy };
               
               // calculate the intersection with the mid-point of the closest side (target node)       
               d.interTar = rectIntersection(targetPoint, sourcePoint, bounds);
            }
            
            if (d.source.shape == "circle")
            {
               d.interSrc = circleIntersection(d.target, d.source);
            }
            
            if (d.target.shape == "circle")
            {
               d.interTar = circleIntersection(d.source, d.target);
            }
         }
      }
      
      /**
       * Calculate the intersection point between a line drawn between two non-overlapping shapes
       * where the target shape is known to be a circle.  This reports the intersection point on
       * the target circle.
       *
       * @param    source
       *           The shape that defines the starting point for the line. Must have x and y
       *           members.
       * @param    target
       *           The circle that contains the line end point. Must have x and y members.
       *
       * @return   The point on the circle that is intersected.
       */
      function circleIntersection(source, target)
      {
         var dx = target.x - source.x;
         var dy = target.y - source.y;
         
         var length = Math.sqrt(dx * dx + dy * dy);
   
         if (length === 0)
         {
            return { "x": source.x, "y": source.y };
         }
   
         var ratio = (length - target.bounds.radius) / length;
         var x = dx * ratio + source.x;
         var y = dy * ratio + source.y;
   
         return { "x": x, "y": y };
      };
      
      /**
       * Calculate the rectangle (in SVG-level coordinates) that contains the given rectangle
       * node.
       *
       * @param    node
       *           This is an SVG rectangle node, contained within a group node. This rectangle
       *           node's attributes are naturally relative to the group node.  This means they
       *           are not in SVG coordinate space, but are in the containing group node's
       *           coordinate space.
       * @param    contX
       *           The containing group node's X coordinate.
       * @param    contY
       *           The containing group node's Y coordinate.
       *
       * @return   A rectangle object with a top left point (x1, y1), bottom right point (x2, y2)
       *           and a center point (cx, cy).  All coordinate values will be in the SVG-level
       *           coordinate space.
       */
      function rectBounds(node, contX, contY)
      {
         // the node attributes are relative to the node's "g" (group) container, read them
         // from the node and translate the x and y coordinates by the containing coordinates
         var x = parseInt(node.attr("x")) + contX;
         var y = parseInt(node.attr("y")) + contY;
         var w = parseInt(node.attr("width"));
         var h = parseInt(node.attr("height"));
         
         // create the object that represents the rectangle
         var bounds =
         {                       
            "x1": x,
            "x2": (x + w),
            "y1": y,
            "y2": (y + h),
            "cx": (x + (w / 2)),
            "cy": (y + (h / 2))
         };
         
         return bounds;
      }
      
      /**
       * Calculate the intersection point between a line and a rectangle returning the middle
       * point of the rectangle side that is intersected.  This assumes that the line has
       * one end point inside the rectangle and the other end point outside. The reason the
       * mid-point is returned is that the visualization result looks better than having the
       * connection to random spots on the rectangle.
       *
       * @param    inside
       *           The line end point that is inside the rectangle. Must have x and y members.
       * @param    outside
       *           The line end point that is outside the rectangle. Must have x and y members.
       * @param    rect
       *           The rectangle instance with a top left point (x1, y1) and a bottom
       *           right point (x2, y2).
       *
       * @return   The mid-point on the side of the rectangle that is intersected.  The exact
       *           intersection point is NOT returned.
       */
      function rectIntersection(inside, outside, rect)
      {
         // ensure valid parameters
         if (!inside || !outside || !rect)
         {
            return null;
         }
         
         if (!inside.x || !inside.y || !outside.x || !outside.y)
         {
            return null;
         }
         
         // rectangle values must be regular (x1, y1) must be the top left corner and (x2, y2)
         // is the bottom right corner
         if (rect.x1 >= rect.x2 || rect.y1 >= rect.y2)
         {
            return null;
         }
         
         // the inside point must actually be inside
         if (inside.x < rect.x1 || inside.x > rect.x2 ||
             inside.y < rect.y1 || inside.y > rect.y2)
         {
            return null;
         }
         
         // the outside point must actually be outside
         if (outside.x >= rect.x1 && outside.x <= rect.x2 &&
             outside.y >= rect.y1 && outside.y <= rect.y2)
         {
            return null;
         }
         
         var width      = rect.x2 - rect.x1;
         var height     = rect.y2 - rect.y1;
         var halfWidth  = width / 2;
         var halfHeight = height / 2;

         if (outside.x === inside.x)
         {
            // if they are on the same X, then we can't compute the slope (is undefined).
            if (outside.y < inside.y)
            {
               return { x: rect.x1 + halfWidth, y: rect.y1 };
            }

            if (outside.y > inside.y)
            {
               return { x: rect.x1 + halfWidth, y: rect.y2 };
            }
            
            return null;
         }
         
         var slope      = (outside.y - inside.y) / (outside.x - inside.x);
         var horizAngle = slope * halfWidth;
         var vertAngle  = halfHeight / slope;
         
         var leftRight = (-halfHeight <= horizAngle) && (horizAngle <= halfHeight);
         var topBottom = (-halfWidth  <= vertAngle)  && (vertAngle  <= halfWidth);
         
         // left side
         if (leftRight && outside.x < inside.x)
         {
            return { x: rect.x1, y: (rect.y1 + halfHeight) };
         }
         
         // right side
         if (leftRight && outside.x > inside.x)
         {
            return { x: rect.x2, y: (rect.y1 + halfHeight) };
         }
         
         // top side
         if (topBottom && outside.y < inside.y)
         {
            return { x: (rect.x1 + halfWidth), y: rect.y1 };
         }
         
         // bottom side
         if (topBottom && outside.y > inside.y)
         {
            return { x: (rect.x1 + halfWidth), y: rect.y2 };
         }
         
         // there should have been an intersection, but if not then report that
         return null;
      }
      
      /**
       * Create useful symbolic constants for commonly referenced token types.
       */
      function initialize()
      {
         for (var t = 0; t < linkTypes.length; t++)
         {
            markerTypes.push({ "name": ("arrow_" + linkTypes[t]), "type": linkTypes[t], "arrow": true  });
            markerTypes.push({ "name": ("label_" + linkTypes[t]), "type": linkTypes[t], "arrow": false });
         }
         
         tokens.FILE_RESOURCE                 = fwd.ast.tokenType("FILE_RESOURCE");
         tokens.AST_NODE                      = fwd.ast.tokenType("AST_NODE");
         tokens.IPC_RESOURCE                  = fwd.ast.tokenType("IPC_RESOURCE");
         tokens.NATIVE_PROGRAM_RESOURCE       = fwd.ast.tokenType("NATIVE_PROGRAM_RESOURCE");
         tokens.UNKNOWN_RESOURCE              = fwd.ast.tokenType("UNKNOWN_RESOURCE");
         tokens.INCLUDES                      = fwd.ast.tokenType("INCLUDES");
         tokens.RUN_FILENAME                  = fwd.ast.tokenType("RUN_FILENAME");
         tokens.RUN_FILENAME_ON_SERVER        = fwd.ast.tokenType("RUN_FILENAME_ON_SERVER");
         tokens.RUN_INT_PROC                  = fwd.ast.tokenType("RUN_INT_PROC");
         tokens.RUN_INT_PROC_ON_SERVER        = fwd.ast.tokenType("RUN_INT_PROC_ON_SERVER");
         tokens.RUN_VALUE                     = fwd.ast.tokenType("RUN_VALUE");
         tokens.RUN_VALUE_ON_SERVER           = fwd.ast.tokenType("RUN_VALUE_ON_SERVER");
         tokens.RUN_PORT_TYPE_ON_SERVER       = fwd.ast.tokenType("RUN_PORT_TYPE_ON_SERVER");
         tokens.RUN_PORT_TYPE_VALUE_ON_SERVER = fwd.ast.tokenType("RUN_PORT_TYPE_VALUE_ON_SERVER");
         tokens.RUN_LIBRARY_REF               = fwd.ast.tokenType("RUN_LIBRARY_REF");
         tokens.RUN_LIBRARY_REF_ON_SERVER     = fwd.ast.tokenType("RUN_LIBRARY_REF_ON_SERVER");
         tokens.FUNCTION_CALL                 = fwd.ast.tokenType("FUNCTION_CALL");
         tokens.AMBIGUOUS                     = fwd.ast.tokenType("AMBIGUOUS");
         tokens.MISSING                       = fwd.ast.tokenType("MISSING");
         tokens.PROCEDURE_FILE                = fwd.ast.tokenType("PROCEDURE_FILE");
         tokens.INCLUDE_FILE                  = fwd.ast.tokenType("INCLUDE_FILE");
         tokens.SCHEMA_FILE                   = fwd.ast.tokenType("SCHEMA_FILE");
         tokens.EXTERNAL_PROCEDURE            = fwd.ast.tokenType("EXTERNAL_PROCEDURE");
         tokens.INTERNAL_PROCEDURE            = fwd.ast.tokenType("INTERNAL_PROCEDURE");
         tokens.LIBRARY_PROCEDURE             = fwd.ast.tokenType("LIBRARY_PROCEDURE");
         tokens.PORT_TYPE                     = fwd.ast.tokenType("PORT_TYPE");
         tokens.SHARED_LIBRARY                = fwd.ast.tokenType("SHARED_LIBRARY");
         tokens.NATIVE_API                    = fwd.ast.tokenType("NATIVE_API");
         tokens.NATIVE_PROCESS                = fwd.ast.tokenType("NATIVE_PROCESS");
         tokens.CHILD_PROCESS                 = fwd.ast.tokenType("CHILD_PROCESS");
         tokens.NETWORK_CONNECTION            = fwd.ast.tokenType("NETWORK_CONNECTION");
         tokens.COM_OBJECT                    = fwd.ast.tokenType("COM_OBJECT");
         tokens.DDE_SERVER                    = fwd.ast.tokenType("DDE_SERVER");
         tokens.OCX_CONTROL                   = fwd.ast.tokenType("OCX_CONTROL");
         tokens.DEFINE_PROPERTY_GET           = fwd.ast.tokenType("DEFINE_PROPERTY_GET");
         tokens.DEFINE_PROPERTY_SET           = fwd.ast.tokenType("DEFINE_PROPERTY_SET");
         tokens.MORE                          = fwd.ast.tokenType("MORE");
         tokens.FUNCTION                      = fwd.ast.tokenType("FUNCTION");
         tokens.EVENT                         = fwd.ast.tokenType("EVENT");
         tokens.TABLE                         = fwd.ast.tokenType("TABLE");
         tokens.DATABASE                      = fwd.ast.tokenType("DATABASE");
         tokens.KW_TAB_TRG                    = fwd.ast.tokenType("KW_TAB_TRG");
         tokens.CLASS_DEF                     = fwd.ast.tokenType("CLASS_DEF");
         tokens.INTERFACE_DEF                 = fwd.ast.tokenType("INTERFACE_DEF");
         tokens.CONSTRUCTOR                   = fwd.ast.tokenType("CONSTRUCTOR");
         tokens.DESTRUCTOR                    = fwd.ast.tokenType("DESTRUCTOR");
         tokens.METHOD_DEF                    = fwd.ast.tokenType("METHOD_DEF");
         tokens.METHOD_INVOCATION             = fwd.ast.tokenType("METHOD_INVOCATION");
         tokens.PROPERTY_INVOCATION           = fwd.ast.tokenType("PROPERTY_INVOCATION");
         tokens.OBJECT_INVOCATION             = fwd.ast.tokenType("OBJECT_INVOCATION");
         tokens.KW_ON                         = fwd.ast.tokenType("KW_ON");
         tokens.TRIGGER_BLOCK                 = fwd.ast.tokenType("TRIGGER_BLOCK");
         tokens.KW_MSG                        = fwd.ast.tokenType("KW_MSG");
         tokens.KW_PAUSE                      = fwd.ast.tokenType("KW_PAUSE");
         tokens.KW_WAIT_FOR                   = fwd.ast.tokenType("KW_WAIT_FOR");
         tokens.KW_READKEY                    = fwd.ast.tokenType("KW_READKEY");
         tokens.KW_INSERT                     = fwd.ast.tokenType("KW_INSERT");
         tokens.KW_APPLY                      = fwd.ast.tokenType("KW_APPLY");
         tokens.PROCESS_EVENTS                = fwd.ast.tokenType("PROCESS_EVENTS");
         tokens.KW_PRMT_FOR                   = fwd.ast.tokenType("KW_PRMT_FOR");
         tokens.KW_SET                        = fwd.ast.tokenType("KW_SET");
         tokens.KW_UPDATE                     = fwd.ast.tokenType("KW_UPDATE");
         tokens.KW_SUPER                      = fwd.ast.tokenType("KW_SUPER");
         tokens.KW_DYN_FUNC                   = fwd.ast.tokenType("KW_DYN_FUNC");
         tokens.RUN_SUPER                     = fwd.ast.tokenType("RUN_SUPER");
         tokens.DDE_INITIATE                  = fwd.ast.tokenType("DDE_INITIATE");
         tokens.CREATE_OBJECT                 = fwd.ast.tokenType("CREATE_OBJECT");
         tokens.KW_LOADCTRL                   = fwd.ast.tokenType("KW_LOADCTRL");
         tokens.KW_CONN                       = fwd.ast.tokenType("KW_CONN");
         tokens.KW_ENABLE_C                   = fwd.ast.tokenType("KW_ENABLE_C");
         tokens.KW_BTOS                       = fwd.ast.tokenType("KW_BTOS");
         tokens.KW_DOS                        = fwd.ast.tokenType("KW_DOS");
         tokens.KW_OS2                        = fwd.ast.tokenType("KW_OS2");
         tokens.KW_UNIX                       = fwd.ast.tokenType("KW_UNIX");
         tokens.KW_OS_CMD                     = fwd.ast.tokenType("KW_OS_CMD");
         tokens.KW_CTOS                       = fwd.ast.tokenType("KW_CTOS");
         tokens.KW_VMS                        = fwd.ast.tokenType("KW_VMS");
         tokens.INPUT_THRU                    = fwd.ast.tokenType("INPUT_THRU");
         tokens.INPUT_OUTPUT_THRU             = fwd.ast.tokenType("INPUT_OUTPUT_THRU");
         tokens.OUTPUT_THRU                   = fwd.ast.tokenType("OUTPUT_THRU");
         
         displayProps[tokens.FILE_RESOURCE                ] = { "color": "darkgreen"       , "stroke": "black"           , "stroke-dasharray": ""  , "stroke-width": "1"  , "augmentText" : false, "text": "File Resource" };
         displayProps[tokens.AST_NODE                     ] = { "color": "darkslategray"   , "stroke": "black"           , "stroke-dasharray": ""  , "stroke-width": "1"  , "augmentText" : false, "text": "AST Node" };
         displayProps[tokens.IPC_RESOURCE                 ] = { "color": "linen"           , "stroke": "black"           , "stroke-dasharray": ""  , "stroke-width": "1"  , "augmentText" : false, "text": "IPC Resource" };
         displayProps[tokens.NATIVE_PROGRAM_RESOURCE      ] = { "color": "lemonchiffon"    , "stroke": "black"           , "stroke-dasharray": ""  , "stroke-width": "1"  , "augmentText" : false, "text": "Native (OS) Resource" };
         displayProps[tokens.UNKNOWN_RESOURCE             ] = { "color": "darkorange"      , "stroke": "black"           , "stroke-dasharray": ""  , "stroke-width": "1"  , "augmentText" : false, "text": "Unknown Resource" };
         displayProps[tokens.INCLUDES                     ] = { "color": "dimgray"         , "stroke": "black"           , "stroke-dasharray": ""  , "stroke-width": "1"  , "augmentText" : false, "text": "Include Linkage" };
         displayProps[tokens.RUN_FILENAME                 ] = { "color": color(18)         , "stroke": ""                , "stroke-dasharray": ""  , "stroke-width": "1"  , "augmentText" : false, "text": "RUN <external_procedure>" };
         displayProps[tokens.RUN_FILENAME_ON_SERVER       ] = { "color": color(19)         , "stroke": ""                , "stroke-dasharray": ""  , "stroke-width": "1"  , "augmentText" : false, "text": "RUN <external_procedure> ON <server>" };
         displayProps[tokens.RUN_INT_PROC                 ] = { "color": "yellowgreen"     , "stroke": ""                , "stroke-dasharray": ""  , "stroke-width": "1"  , "augmentText" : false, "text": "RUN <internal_procedure>" };
         displayProps[tokens.RUN_INT_PROC_ON_SERVER       ] = { "color": color(0)          , "stroke": ""                , "stroke-dasharray": ""  , "stroke-width": "1"  , "augmentText" : false, "text": "RUN <internal_procedure> ON <server>" };
         displayProps[tokens.RUN_VALUE                    ] = { "color": color(1)          , "stroke": ""                , "stroke-dasharray": ""  , "stroke-width": "1"  , "augmentText" : false, "text": "RUN VALUE(<expr>)" };
         displayProps[tokens.RUN_VALUE_ON_SERVER          ] = { "color": color(4)          , "stroke": ""                , "stroke-dasharray": ""  , "stroke-width": "1"  , "augmentText" : false, "text": "RUN VALUE(<expr>) ON <server>" };
         displayProps[tokens.RUN_PORT_TYPE_ON_SERVER      ] = { "color": color(5)          , "stroke": ""                , "stroke-dasharray": ""  , "stroke-width": "1"  , "augmentText" : false, "text": "RUN <web_service_port_type>" };
         displayProps[tokens.RUN_PORT_TYPE_VALUE_ON_SERVER] = { "color": color(6)          , "stroke": ""                , "stroke-dasharray": ""  , "stroke-width": "1"  , "augmentText" : false, "text": "RUN VALUE(<web_service_port_type>)" };
         displayProps[tokens.RUN_LIBRARY_REF              ] = { "color": color(7)          , "stroke": ""                , "stroke-dasharray": ""  , "stroke-width": "1"  , "augmentText" : false, "text": "RUN <procedure_library_member>" };
         displayProps[tokens.RUN_LIBRARY_REF_ON_SERVER    ] = { "color": color(8)          , "stroke": ""                , "stroke-dasharray": ""  , "stroke-width": "1"  , "augmentText" : false, "text": "RUN <procedure_library_member> ON <server>" };
         displayProps[tokens.FUNCTION_CALL                ] = { "color": "lightsteelblue"  , "stroke": ""                , "stroke-dasharray": ""  , "stroke-width": "1"  , "augmentText" : false, "text": "Function Call <function name>" };
         displayProps[tokens.AMBIGUOUS                    ] = { "color": color(9)          , "stroke": "black"           , "stroke-dasharray": ""  , "stroke-width": "1"  , "augmentText" : true , "text": "Ambiguous or Unknown Entry Point" };
         displayProps[tokens.MISSING                      ] = { "color": color(10)         , "stroke": "black"           , "stroke-dasharray": ""  , "stroke-width": "1"  , "augmentText" : true , "text": "Missing Entry Point" };
         displayProps[tokens.PROCEDURE_FILE               ] = { "color": "silver"          , "stroke": "black"           , "stroke-dasharray": ""  , "stroke-width": "1.5", "augmentText" : true , "text": "Procedure Source File" };
         displayProps[tokens.INCLUDE_FILE                 ] = { "color": color(11)         , "stroke": "black"           , "stroke-dasharray": ""  , "stroke-width": "1"  , "augmentText" : false, "text": "Include File" };
         displayProps[tokens.SCHEMA_FILE                  ] = { "color": color(12)         , "stroke": "black"           , "stroke-dasharray": ""  , "stroke-width": "1"  , "augmentText" : false, "text": "Schema Definition" };
         displayProps[tokens.EXTERNAL_PROCEDURE           ] = { "color": color(2)          , "stroke": "black"           , "stroke-dasharray": "4" , "stroke-width": "1"  , "augmentText" : false, "text": "External Procedure" };
         displayProps[tokens.INTERNAL_PROCEDURE           ] = { "color": color(3)          , "stroke": "black"           , "stroke-dasharray": "4" , "stroke-width": "1"  , "augmentText" : true , "text": "Internal Procedure" };
         displayProps[tokens.LIBRARY_PROCEDURE            ] = { "color": color(13)         , "stroke": "black"           , "stroke-dasharray": "4" , "stroke-width": "1"  , "augmentText" : false, "text": "Library Procedure" };
         displayProps[tokens.PORT_TYPE                    ] = { "color": color(14)         , "stroke": "black"           , "stroke-dasharray": "4" , "stroke-width": "1"  , "augmentText" : false, "text": "Web Services Port Type" };
         displayProps[tokens.SHARED_LIBRARY               ] = { "color": color(15)         , "stroke": "black"           , "stroke-dasharray": ""  , "stroke-width": "1"  , "augmentText" : false, "text": "Shared Library" };
         displayProps[tokens.NATIVE_API                   ] = { "color": color(16)         , "stroke": "black"           , "stroke-dasharray": ""  , "stroke-width": "1"  , "augmentText" : false, "text": "Native API" };
         displayProps[tokens.NATIVE_PROCESS               ] = { "color": color(17)         , "stroke": "black"           , "stroke-dasharray": "4" , "stroke-width": "1"  , "augmentText" : false, "text": "Operating System Process" };
         displayProps[tokens.CHILD_PROCESS                ] = { "color": color(18)         , "stroke": "black"           , "stroke-dasharray": "4" , "stroke-width": "1"  , "augmentText" : false, "text": "Child Process" };
         displayProps[tokens.NETWORK_CONNECTION           ] = { "color": "darkolivegreen"  , "stroke": "black"           , "stroke-dasharray": "4" , "stroke-width": "1"  , "augmentText" : false, "text": "Network Connection" };
         displayProps[tokens.COM_OBJECT                   ] = { "color": "darkseagreen"    , "stroke": "black"           , "stroke-dasharray": ""  , "stroke-width": "1"  , "augmentText" : false, "text": "COM Automation Object" };
         displayProps[tokens.DDE_SERVER                   ] = { "color": "darkkhaki"       , "stroke": "black"           , "stroke-dasharray": ""  , "stroke-width": "1"  , "augmentText" : false, "text": "DDE Server" };
         displayProps[tokens.OCX_CONTROL                  ] = { "color": "coral"           , "stroke": "black"           , "stroke-dasharray": ""  , "stroke-width": "1"  , "augmentText" : false, "text": "OCX Control" };
         displayProps[tokens.MORE                         ] = { "color": "teal"            , "stroke": "black"           , "stroke-dasharray": ""  , "stroke-width": "1"  , "augmentText" : false, "text": "Traverse to Off Screen Node" };
         displayProps[tokens.FUNCTION                     ] = { "color": "steelblue"       , "stroke": "black"           , "stroke-dasharray": "4" , "stroke-width": "1"  , "augmentText" : true , "text": "Function Definition" };
         displayProps[tokens.EVENT                        ] = { "color": "rosybrown"       , "stroke": "black"           , "stroke-dasharray": ""  , "stroke-width": "1"  , "augmentText" : false, "text": "Event" };
         displayProps[tokens.TABLE                        ] = { "color": "sienna"          , "stroke": "black"           , "stroke-dasharray": "4" , "stroke-width": "1"  , "augmentText" : false, "text": "Table" };
         displayProps[tokens.DATABASE                     ] = { "color": "saddlebrown"     , "stroke": "black"           , "stroke-dasharray": ""  , "stroke-width": "1"  , "augmentText" : false, "text": "Database" };
         displayProps[tokens.KW_TAB_TRG                   ] = { "color": "maroon"          , "stroke": "black"           , "stroke-dasharray": ""  , "stroke-width": "1"  , "augmentText" : false, "text": "Schema Trigger" };
         displayProps[tokens.CLASS_DEF                    ] = { "color": "royalblue"       , "stroke": "black"           , "stroke-dasharray": ""  , "stroke-width": "1"  , "augmentText" : false, "text": "OO 4GL Class" };
         displayProps[tokens.INTERFACE_DEF                ] = { "color": "mediumorchid"    , "stroke": "black"           , "stroke-dasharray": ""  , "stroke-width": "1"  , "augmentText" : false, "text": "OO 4GL Interface" };
         displayProps[tokens.CONSTRUCTOR                  ] = { "color": "seashell"        , "stroke": "black"           , "stroke-dasharray": ""  , "stroke-width": "1"  , "augmentText" : false, "text": "OO 4GL Constructor" };
         displayProps[tokens.DESTRUCTOR                   ] = { "color": "antiquewhite"    , "stroke": "black"           , "stroke-dasharray": ""  , "stroke-width": "1"  , "augmentText" : false, "text": "OO 4GL Destructor" };
         displayProps[tokens.METHOD_DEF                   ] = { "color": "deepskyblue"     , "stroke": "black"           , "stroke-dasharray": ""  , "stroke-width": "1"  , "augmentText" : false, "text": "OO 4GL Method Definition" };
         displayProps[tokens.METHOD_INVOCATION            ] = { "color": "lightgreen"      , "stroke": "black"           , "stroke-dasharray": ""  , "stroke-width": "1"  , "augmentText" : false, "text": "OO 4GL Method Invocation" };
         displayProps[tokens.PROPERTY_INVOCATION          ] = { "color": "darkturquoise"   , "stroke": "black"           , "stroke-dasharray": ""  , "stroke-width": "1"  , "augmentText" : false, "text": "OO 4GL Property Invocation" };
         displayProps[tokens.OBJECT_INVOCATION            ] = { "color": "honeydew"        , "stroke": "black"           , "stroke-dasharray": ""  , "stroke-width": "1"  , "augmentText" : false, "text": "OO 4GL Object Invocation" };
         displayProps[tokens.KW_SUPER                     ] = { "color": "mediumaquamarine", "stroke": "black"           , "stroke-dasharray": ""  , "stroke-width": "1"  , "augmentText" : false, "text": "OO 4GL SUPER Statement" };
         displayProps[tokens.DEFINE_PROPERTY_GET          ] = { "color": "khaki"           , "stroke": "black"           , "stroke-dasharray": "4" , "stroke-width": "1"  , "augmentText" : false, "text": "OO 4GL Property Getter" };
         displayProps[tokens.DEFINE_PROPERTY_SET          ] = { "color": "peachpuff"       , "stroke": "black"           , "stroke-dasharray": "4" , "stroke-width": "1"  , "augmentText" : false, "text": "OO 4GL Property Setter" };
         displayProps[tokens.KW_ON                        ] = { "color": "mediumseagreen"  , "stroke": "black"           , "stroke-dasharray": ""  , "stroke-width": "1"  , "augmentText" : false, "text": "ON <events> Statement" };
         displayProps[tokens.TRIGGER_BLOCK                ] = { "color": "lightskyblue"    , "stroke": "black"           , "stroke-dasharray": ""  , "stroke-width": "1"  , "augmentText" : false, "text": "Trigger" };
         displayProps[tokens.KW_MSG                       ] = { "color": "pink"            , "stroke": "black"           , "stroke-dasharray": ""  , "stroke-width": "1"  , "augmentText" : false, "text": "MESSAGE Statement" };
         displayProps[tokens.KW_PAUSE                     ] = { "color": "deeppink"        , "stroke": "black"           , "stroke-dasharray": ""  , "stroke-width": "1"  , "augmentText" : false, "text": "PAUSE Statement" };
         displayProps[tokens.KW_WAIT_FOR                  ] = { "color": "tan"             , "stroke": "black"           , "stroke-dasharray": ""  , "stroke-width": "1"  , "augmentText" : false, "text": "WAIT-FOR Statement" };
         displayProps[tokens.KW_READKEY                   ] = { "color": "lightcoral"      , "stroke": "black"           , "stroke-dasharray": ""  , "stroke-width": "1"  , "augmentText" : false, "text": "READKEY Statement" };
         displayProps[tokens.KW_INSERT                    ] = { "color": "navajowhite"     , "stroke": "black"           , "stroke-dasharray": ""  , "stroke-width": "1"  , "augmentText" : false, "text": "INSERT Statement" };
         displayProps[tokens.KW_APPLY                     ] = { "color": "greenyellow"     , "stroke": "black"           , "stroke-dasharray": ""  , "stroke-width": "1"  , "augmentText" : false, "text": "APPLY Statement" };
         displayProps[tokens.PROCESS_EVENTS               ] = { "color": "thistle"         , "stroke": "black"           , "stroke-dasharray": ""  , "stroke-width": "1"  , "augmentText" : false, "text": "PROCESS-EVENTS Statement" };
         displayProps[tokens.KW_PRMT_FOR                  ] = { "color": "darkgoldenrod"   , "stroke": "black"           , "stroke-dasharray": ""  , "stroke-width": "1"  , "augmentText" : false, "text": "PROMPT-FOR Statement" };
         displayProps[tokens.KW_SET                       ] = { "color": "palegoldenrod"   , "stroke": "black"           , "stroke-dasharray": ""  , "stroke-width": "1"  , "augmentText" : false, "text": "SET Statement" };
         displayProps[tokens.KW_UPDATE                    ] = { "color": "goldenrod"       , "stroke": "black"           , "stroke-dasharray": ""  , "stroke-width": "1"  , "augmentText" : false, "text": "UPDATE Statement" };
         displayProps[tokens.KW_DYN_FUNC                  ] = { "color": "plum"            , "stroke": "black"           , "stroke-dasharray": ""  , "stroke-width": "1"  , "augmentText" : false, "text": "Dynamic Function Invocation <function-name>" };
         displayProps[tokens.RUN_SUPER                    ] = { "color": "lightyellow"     , "stroke": "black"           , "stroke-dasharray": ""  , "stroke-width": "1"  , "augmentText" : false, "text": "RUN SUPER Procedure" };
         displayProps[tokens.DDE_INITIATE                 ] = { "color": "orange"          , "stroke": "black"           , "stroke-dasharray": ""  , "stroke-width": "1"  , "augmentText" : false, "text": "DDE Initiate Statement" };
         displayProps[tokens.CREATE_OBJECT                ] = { "color": "crimson"         , "stroke": "black"           , "stroke-dasharray": ""  , "stroke-width": "1"  , "augmentText" : false, "text": "CREATE OBJECT Statement" };
         displayProps[tokens.KW_LOADCTRL                  ] = { "color": "darkred"         , "stroke": "black"           , "stroke-dasharray": ""  , "stroke-width": "1"  , "augmentText" : false, "text": "LoadControl Method" };
         displayProps[tokens.KW_CONN                      ] = { "color": "wheat"           , "stroke": "black"           , "stroke-dasharray": ""  , "stroke-width": "1"  , "augmentText" : false, "text": "CONNECT() Method" };
         displayProps[tokens.KW_ENABLE_C                  ] = { "color": "peru"            , "stroke": "black"           , "stroke-dasharray": ""  , "stroke-width": "1"  , "augmentText" : false, "text": "ENABLE-CONNECTIONS() Method" };
         displayProps[tokens.KW_BTOS                      ] = { "color": "orangered"       , "stroke": "black"           , "stroke-dasharray": ""  , "stroke-width": "1"  , "augmentText" : false, "text": "BTOS Shell Command Launch" };
         displayProps[tokens.KW_DOS                       ] = { "color": "orangered"       , "stroke": "black"           , "stroke-dasharray": ""  , "stroke-width": "1"  , "augmentText" : false, "text": "DOS Shell Command Launch" };
         displayProps[tokens.KW_OS2                       ] = { "color": "orangered"       , "stroke": "black"           , "stroke-dasharray": ""  , "stroke-width": "1"  , "augmentText" : false, "text": "OS2 Shell Command Launch" };
         displayProps[tokens.KW_UNIX                      ] = { "color": "orangered"       , "stroke": "black"           , "stroke-dasharray": ""  , "stroke-width": "1"  , "augmentText" : false, "text": "UNIX Shell Command Launch" };
         displayProps[tokens.KW_OS_CMD                    ] = { "color": "orangered"       , "stroke": "black"           , "stroke-dasharray": ""  , "stroke-width": "1"  , "augmentText" : false, "text": "OS-COMMAND Shell Command Launch" };
         displayProps[tokens.KW_CTOS                      ] = { "color": "orangered"       , "stroke": "black"           , "stroke-dasharray": ""  , "stroke-width": "1"  , "augmentText" : false, "text": "CTOS Shell Command Launch" };
         displayProps[tokens.KW_VMS                       ] = { "color": "orangered"       , "stroke": "black"           , "stroke-dasharray": ""  , "stroke-width": "1"  , "augmentText" : false, "text": "VMS Shell Command Launch" };
         displayProps[tokens.INPUT_THRU                   ] = { "color": "darkred"         , "stroke": "black"           , "stroke-dasharray": ""  , "stroke-width": "1"  , "augmentText" : false, "text": "INPUT-THROUGH Child Process Launch" };
         displayProps[tokens.INPUT_OUTPUT_THRU            ] = { "color": "darkred"         , "stroke": "black"           , "stroke-dasharray": ""  , "stroke-width": "1"  , "augmentText" : false, "text": "INPUT-OUTPUT-THROUGH Child Process Launch" };
         displayProps[tokens.OUTPUT_THRU                  ] = { "color": "darkred"         , "stroke": "black"           , "stroke-dasharray": ""  , "stroke-width": "1"  , "augmentText" : false, "text": "OUTPUT-THROUGH Child Process Launch" };
         displayProps["ambiguous"                         ] = { "color": ""                , "stroke": "tomato"          , "stroke-dasharray": "4" , "stroke-width": "2"  , "augmentText" : false, "text": "Ambiguous" };
         displayProps["calls"                             ] = { "color": ""                , "stroke": "black"           , "stroke-dasharray": ""  , "stroke-width": "2"  , "augmentText" : false, "text": "Calls" };
         displayProps["contains"                          ] = { "color": ""                , "stroke": "green"           , "stroke-dasharray": "2" , "stroke-width": "2"  , "augmentText" : false, "text": "Contains" };
         displayProps["implements"                        ] = { "color": ""                , "stroke": "gold"            , "stroke-dasharray": "2" , "stroke-width": "2"  , "augmentText" : false, "text": "Implements" };
         displayProps["includes"                          ] = { "color": ""                , "stroke": "chocolate"       , "stroke-dasharray": "2" , "stroke-width": "2"  , "augmentText" : false, "text": "Includes" };
         displayProps["inherits"                          ] = { "color": ""                , "stroke": "palegreen"       , "stroke-dasharray": "2" , "stroke-width": "2"  , "augmentText" : false, "text": "Inherits" };
         displayProps["listens"                           ] = { "color": ""                , "stroke": "rebeccapurple"   , "stroke-dasharray": ""  , "stroke-width": "2"  , "augmentText" : false, "text": "Listens" };
         displayProps["override"                          ] = { "color": ""                , "stroke": "darkorange"      , "stroke-dasharray": "2" , "stroke-width": "2"  , "augmentText" : false, "text": "Override" };
         displayProps["raises"                            ] = { "color": ""                , "stroke": "mediumaquamarine", "stroke-dasharray": ""  , "stroke-width": "2"  , "augmentText" : false, "text": "Raises" };
         displayProps["references"                        ] = { "color": ""                , "stroke": "mediumorchid"    , "stroke-dasharray": ""  , "stroke-width": "2"  , "augmentText" : false, "text": "References" };
         displayProps["registers"                         ] = { "color": ""                , "stroke": "gold"            , "stroke-dasharray": ""  , "stroke-width": "2"  , "augmentText" : false, "text": "Registers" };
         
         // sandybrown
         // tomato
         // darkred
         // palevioletred 
         legendContent = [];

         //legendContent.push({ "color": displayProps[tokens.FILE_RESOURCE                ].color, "text": displayProps[tokens.FILE_RESOURCE                ].text });
         //legendContent.push({ "color": displayProps[tokens.AST_NODE                     ].color, "text": displayProps[tokens.AST_NODE                     ].text });
         //legendContent.push({ "color": displayProps[tokens.IPC_RESOURCE                 ].color, "text": displayProps[tokens.IPC_RESOURCE                 ].text });
         //legendContent.push({ "color": displayProps[tokens.NATIVE_PROGRAM_RESOURCE      ].color, "text": displayProps[tokens.NATIVE_PROGRAM_RESOURCE      ].text });
         //legendContent.push({ "color": displayProps[tokens.UNKNOWN_RESOURCE             ].color, "text": displayProps[tokens.UNKNOWN_RESOURCE             ].text });
         //legendContent.push({ "color": displayProps[tokens.INCLUDES                     ].color, "text": displayProps[tokens.INCLUDES                     ].text });
         legendContent.push({ "color": displayProps[tokens.RUN_FILENAME                 ].color, "text": displayProps[tokens.RUN_FILENAME                 ].text });
         legendContent.push({ "color": displayProps[tokens.RUN_FILENAME_ON_SERVER       ].color, "text": displayProps[tokens.RUN_FILENAME_ON_SERVER       ].text });
         legendContent.push({ "color": displayProps[tokens.RUN_INT_PROC                 ].color, "text": displayProps[tokens.RUN_INT_PROC                 ].text });
         legendContent.push({ "color": displayProps[tokens.RUN_INT_PROC_ON_SERVER       ].color, "text": displayProps[tokens.RUN_INT_PROC_ON_SERVER       ].text });
         legendContent.push({ "color": displayProps[tokens.RUN_VALUE                    ].color, "text": displayProps[tokens.RUN_VALUE                    ].text });
         legendContent.push({ "color": displayProps[tokens.RUN_VALUE_ON_SERVER          ].color, "text": displayProps[tokens.RUN_VALUE_ON_SERVER          ].text });
         legendContent.push({ "color": displayProps[tokens.RUN_PORT_TYPE_ON_SERVER      ].color, "text": displayProps[tokens.RUN_PORT_TYPE_ON_SERVER      ].text });
         legendContent.push({ "color": displayProps[tokens.RUN_PORT_TYPE_VALUE_ON_SERVER].color, "text": displayProps[tokens.RUN_PORT_TYPE_VALUE_ON_SERVER].text });
         legendContent.push({ "color": displayProps[tokens.RUN_LIBRARY_REF              ].color, "text": displayProps[tokens.RUN_LIBRARY_REF              ].text });
         legendContent.push({ "color": displayProps[tokens.RUN_LIBRARY_REF_ON_SERVER    ].color, "text": displayProps[tokens.RUN_LIBRARY_REF_ON_SERVER    ].text });
         legendContent.push({ "color": displayProps[tokens.FUNCTION_CALL                ].color, "text": displayProps[tokens.FUNCTION_CALL                ].text });
         legendContent.push({ "color": displayProps[tokens.AMBIGUOUS                    ].color, "text": displayProps[tokens.AMBIGUOUS                    ].text });
         legendContent.push({ "color": displayProps[tokens.MISSING                      ].color, "text": displayProps[tokens.MISSING                      ].text });
         legendContent.push({ "color": displayProps[tokens.PROCEDURE_FILE               ].color, "text": displayProps[tokens.PROCEDURE_FILE               ].text });
         // legendContent.push({ "color": displayProps[tokens.INCLUDE_FILE                 ].color, "text": displayProps[tokens.INCLUDE_FILE                 ].text });
         legendContent.push({ "color": displayProps[tokens.SCHEMA_FILE                  ].color, "text": displayProps[tokens.SCHEMA_FILE                  ].text });
         legendContent.push({ "color": displayProps[tokens.EXTERNAL_PROCEDURE           ].color, "text": displayProps[tokens.EXTERNAL_PROCEDURE           ].text });
         legendContent.push({ "color": displayProps[tokens.INTERNAL_PROCEDURE           ].color, "text": displayProps[tokens.INTERNAL_PROCEDURE           ].text });
         legendContent.push({ "color": displayProps[tokens.FUNCTION                     ].color, "text": displayProps[tokens.FUNCTION                     ].text });
         legendContent.push({ "color": displayProps[tokens.LIBRARY_PROCEDURE            ].color, "text": displayProps[tokens.LIBRARY_PROCEDURE            ].text });
         legendContent.push({ "color": displayProps[tokens.PORT_TYPE                    ].color, "text": displayProps[tokens.PORT_TYPE                    ].text });
         legendContent.push({ "color": displayProps[tokens.SHARED_LIBRARY               ].color, "text": displayProps[tokens.SHARED_LIBRARY               ].text });
         legendContent.push({ "color": displayProps[tokens.NATIVE_API                   ].color, "text": displayProps[tokens.NATIVE_API                   ].text });
         legendContent.push({ "color": displayProps[tokens.NATIVE_PROCESS               ].color, "text": displayProps[tokens.NATIVE_PROCESS               ].text });
         legendContent.push({ "color": displayProps[tokens.CHILD_PROCESS                ].color, "text": displayProps[tokens.CHILD_PROCESS                ].text });
         legendContent.push({ "color": displayProps[tokens.NETWORK_CONNECTION           ].color, "text": displayProps[tokens.NETWORK_CONNECTION           ].text });
         legendContent.push({ "color": displayProps[tokens.COM_OBJECT                   ].color, "text": displayProps[tokens.COM_OBJECT                   ].text });
         legendContent.push({ "color": displayProps[tokens.DDE_SERVER                   ].color, "text": displayProps[tokens.DDE_SERVER                   ].text });
         legendContent.push({ "color": displayProps[tokens.OCX_CONTROL                  ].color, "text": displayProps[tokens.OCX_CONTROL                  ].text });
         legendContent.push({ "color": displayProps[tokens.DEFINE_PROPERTY_GET          ].color, "text": displayProps[tokens.DEFINE_PROPERTY_GET          ].text });
         legendContent.push({ "color": displayProps[tokens.DEFINE_PROPERTY_SET          ].color, "text": displayProps[tokens.DEFINE_PROPERTY_SET          ].text });
         legendContent.push({ "color": displayProps[tokens.MORE                         ].color, "text": displayProps[tokens.MORE                         ].text });
      }                                                                                                
      
      /**
       * All links which have the parentId field set will be marked as hidden and collected at the
       * source and target nodes, so that they can be shown on demand.
       * 
       * @param    nodes
       *           All the nodes array.
       * @param    links
       *           All the links array.
       * @param    nodeMap
       *           The node map.
       * @param    linkMap
       *           The link map.
       */
      function computeHiddenLinks(nodes, links, nodeMap, linkMap)
      {
         for (var i = 0; i < links.length; i++)
         {
            var l = links[i];

            if (l.parentId === null)
            {
               continue;
            }
            
            // this is a hidden link... mark it
            l.hidden = true
            
            // collect it at the source and target nodes
            var s = nodeMap[l.source];
            var t = nodeMap[l.target];

            if (!s.hiddenLinks)
            {
               s.hiddenLinks = [];
            }
            s.hiddenLinks.push(l);

            if (!t.hiddenLinks)
            {
               t.hiddenLinks = [];
            }
            t.hiddenLinks.push(l);
            
            // count the hidden links at the parent - it will affect its weight
            var pLink = linkMap[l.parentId];
            if (pLink.hiddenCount === undefined)
            {
               pLink.hiddenCount = 0;
            }
            pLink.hiddenCount += 1;
         }
      }
      
      /**
       * Create the lookup table of links mapped by their id as the key.
       *
       * @param    links
       *           An array of the link objects. Each element must have an id member.
       *
       * @return   The link map.
       */
      function createLinkMap(links)
      {
         var map = {};
         var key = null;
         
         for(var i = 0; i < links.length; i++)
         {
            key = links[i].id;
            
            map[key] = links[i];
         }
         
         return map;
      }
      
      /**
       * Create the lookup table of nodes mapped by their id as the key.
       *
       * @param    nodes
       *           An array of the node objects. Each element must have an id member.
       *
       * @return   The node map.
       */
      function createNodeMap(nodes)
      {
         var map = {};
         var key = null;
         
         for(var i = 0; i < nodes.length; i++)
         {
            key = nodes[i].id;
            
            map[key] = nodes[i];
            
            augmentNode(nodes[i]);
         }
         
         // TODO: get rid of this when the MORE nodes no longer are containers; we are iterating
         //        through the links and looking for any source or target nodes that are type MORE
         //        which should not happen
         for (var z = 0; z < data.links.length; z++)
         {
            var l = data.links[z];
            
            if (l.text == "contains")
            {
               var s = map[l.source];
               var t = map[l.target];
               
               if (s.type == tokens.MORE)
               {
                  console.log("FIXME: Node id " + s.id + " is of type MORE but it CONTAINS other nodes. This should NEVER HAPPEN!");
                  // force it to rectangle because otherwise nothing downstream will work
                  s.shape = "rect";
                  s.text  = s.text + ", BROKEN INPUT for link SOURCE, see console log for node id " + s.id; 
               }
               if (t.type == tokens.MORE)
               {                                                   
                  console.log("FIXME: Node id " + t.id + " is of type MORE but it is CONTAINED by another node (id " + s.id + "). This should NEVER HAPPEN!");
                  
                  t.text  = t.text + ", BROKEN INPUT for link TARGET, see console log for node id " + t.id; 
               }
            }
         }
         
         return map;
      }
      
      /**                                                   
       * Calculate and store some useful information in the node.
       *
       * @param    node
       *           The node to augment.
       */
      function augmentNode(node)
      {
         if (node.type == tokens.MORE || node.type == tokens.EVENT)
         {
            node.shape = "circle";
         }
         else
         {
            node.shape = "rect";
         }
      }
      
      /**
       * Initialize the toolbox and the toolbox icon.
       *
       * @param    $parent
       *           JQuery object to which to add the toolbox.
       */
      function setupToolbox($parent)
      {
         // create the toolbox, do this after the svg to get the nature z-order right
         $toolbox = $("<div>", { "id": "toolbox" });
         $parent.append($toolbox);
         
         // set styles for the toolbox container
         $toolbox.css(
         {
            "font"            : "normal 10px sans-serif",
            "stroke-width"    : ".3",
            "position"        : "absolute",
            "border"          : "outset 1px gray",
            "background"      : "#f8f9f9 ",
            "padding"         : "5px",
            "margin"          : "15px",
            "color"           : "#000000",
            "opacity"         : "0.8",
            "right"           : "0",
            "height"          : "95%",
            "width"           : "200px",
         });
         
         toolboxIcon = svg.append("image")
                          .attr("x", width - 45)
                          .attr("y", 10)
                          .attr("width",35)
                          .attr("height", 35)
                          .attr("xlink:href", "images/gear.svg")
                          .attr("visibility", "display")
                          .on("click", function(evt) { toggleToolbox($toolbox, evt, true); });
                          
         $toolbox.hide();
         $toolbox.on("click", function(evt) { toggleToolbox($toolbox, evt, false); });
         
         createStartingPoints($toolbox);
         $toolbox.append($("<br/>"));
         createDirection($toolbox);
         $toolbox.append($("<br/>"));
         createNestingLevel($toolbox);
         $toolbox.append($("<br/>"));
         createGraphDetail($toolbox);
         $toolbox.append($("<br/>"));
         createLoadGraph($toolbox);
         $toolbox.append($("<hr/>"));
         createMoreLinksCheck($toolbox);
      }

      /**
       * Create and attach to the toolbox the graph detail radio-set elements 
       * (ALL, FILES, ENTRY POINTS).
       *
       * @param   $toolbox
       *          The HTML toolbox element.
       */
      function createGraphDetail($toolbox)
      {
         var $detailFiles = $("<div>", {id: "detailFiles"});
         var $detailEntryPoints = $("<div>", {id: "detailEntryPoints"});
         var $detailAll  = $("<div>", {id: "detailAll"});

         $toolbox.append($("<label>Graph Detail Level: </label>"));
         $toolbox.append($("<br/>"));
         $toolbox.append($detailFiles);
         $toolbox.append($("<label for='detailFiles'>Only File Resources</label>"));
         $toolbox.append($("<br/>"));
         $toolbox.append($detailEntryPoints);
         $toolbox.append($("<label for='detailEntryPoints'>Files + Entry Points</label>"));
         $toolbox.append($("<br/>"));
         $toolbox.append($detailAll);
         $toolbox.append($("<label for='detailAll'>All</label>"));
         
         require(["dijit/form/RadioButton", "dojo/domReady!"],
         function(RadioButton)
         {
            var widget = dijit.byId("detailFiles");
            if (widget)
            {
               widget.destroy(true);
            }
            widget = dijit.byId("detailEntryPoints");
            if (widget)
            {
               widget.destroy(true);
            }
            widget = dijit.byId("detailAll");
            if (widget)
            {
               widget.destroy(true);
            }

            var rbCfg =
            {
               "name"           : "graphDetail",
               "placeHolder"    : "Files",
               "value"          : graphDetail.FILES.toString(),
               "checked"        : graphDetailValue == graphDetail.FILES
            };
         
            detailFilesWidget = new RadioButton(rbCfg, "detailFiles");
            detailFilesWidget.startup();

            rbCfg =
            {
               "name"           : "graphDetail",
               "placeHolder"    : "Entry Points",
               "value"          : graphDetail.ENTRY_POINTS.toString(),
               "checked"        : graphDetailValue == graphDetail.ENTRY_POINTS
             };
         
             detailEntryPointsWidget = new RadioButton(rbCfg, "detailEntryPoints");
             detailEntryPointsWidget.startup();

             rbCfg =
             {
                "name"           : "graphDetail",
                "placeHolder"    : "All",
                "value"          : graphDetail.ALL.toString(),
                "checked"        : graphDetailValue == graphDetail.ALL
             };
          
             detailAllWidget = new RadioButton(rbCfg, "detailAll");
             detailAllWidget.startup();
         });
      }

      /**
       * Create and attach to the toolbox the expand links checkbox element.
       *
       * @param   $toolbox
       *          The HTML toolbox element.
       */
      function createMoreLinksCheck($toolbox)
      {
         var $moreLinks = $("<div>", {id: "moreLinks"});
         $toolbox.append($moreLinks);
         $toolbox.append($("<label for='moreLinks'> Expand All Links</label>"));
         
         require(["dijit/form/CheckBox", "dojo/domReady!"],
         function(CheckBox)
         {
            var widget = dijit.byId("moreLinks");
            if (widget)
            {
               widget.destroy(true);
            }

            var cbCfg =
            {
               "name"           : "moreLinks",
               "placeHolder"    : "Expand all links",
               "checked"        : moreLinksValue,
                "onChange"      : showHideAllLinks
            };
         
            var moreLinksWidget = new CheckBox(cbCfg, "moreLinks");
            moreLinksWidget.startup();
         });
      }

      /**
       * Create and attach to the toolbox the direction radio-set elements (one for each direction,
       * OUT, IN and BOTH).
       *
       * @param   $toolbox
       *          The HTML toolbox element.
       */
      function createDirection($toolbox)
      {
         var $directionBoth = $("<div>", {id: "directionBoth"});
         var $directionOut = $("<div>", {id: "directionOut"});
         var $directionIn = $("<div>", {id: "directionIn"});

         $toolbox.append($("<label>Direction: </label>"));
         $toolbox.append($("<br/>"));
         $toolbox.append($directionBoth);
         $toolbox.append($("<label for='directionBoth'>Both Directions</label>"));
         $toolbox.append($("<br/>"));
         $toolbox.append($directionOut);
         $toolbox.append($("<label for='directionOut'>Downstream</label>"));
         $toolbox.append($("<br/>"));
         $toolbox.append($directionIn);
         $toolbox.append($("<label for='directionIn'>Upstream</label>"));
         
         require(["dijit/form/RadioButton", "dojo/domReady!"],
         function(RadioButton)
         {
            var widget = dijit.byId("directionBoth");
            if (widget)
            {
               widget.destroy(true);
            }
            widget = dijit.byId("directionIn");
            if (widget)
            {
               widget.destroy(true);
            }
            widget = dijit.byId("directionOut");
            if (widget)
            {
               widget.destroy(true);
            }

            var rbCfg =
            {
               "name"           : "direction",
               "placeHolder"    : "OUT",
               "value"          : direction.BOTH_DIRECTIONS.toString(),
               "checked"        : directionValue == direction.BOTH_DIRECTIONS
            };
         
            directionBothWidget = new RadioButton(rbCfg, "directionBoth");
            directionBothWidget.startup();

            rbCfg =
            {
               "name"           : "direction",
               "placeHolder"    : "IN",
               "value"          : direction.UPSTREAM.toString(),
               "checked"        : directionValue == direction.UPSTREAM
             };
         
             directionInWidget = new RadioButton(rbCfg, "directionIn");
             directionInWidget.startup();

             rbCfg =
             {
                "name"           : "direction",
                "placeHolder"    : "OUT",
                "value"          : direction.DOWNSTREAM.toString(),
                "checked"        : directionValue == direction.DOWNSTREAM
             };
          
             directionOutWidget = new RadioButton(rbCfg, "directionOut");
             directionOutWidget.startup();
         });
      }
      
      /**
       * Create and attach to the toolbox the nesting level textfield element.
       *
       * @param   $toolbox
       *          The HTML toolbox element.
       */
      function createNestingLevel($toolbox)
      {
         var $nestingLevel = $("<div>", {id: "nestingLevel"});
         $toolbox.append($("<label for='nestingLevel'>Level: </label>"));
         $toolbox.append($nestingLevel);
         
         require(["dijit/form/NumberTextBox", "dojo/domReady!"],
         function(NumberTextBox)
         {
            var widget = dijit.byId("nestingLevel");
            if (widget)
            {
               widget.destroy(true);
            }

            var tfCfg =
            {
               "name"           : "nestingLevel",
               "placeHolder"    : "Enter nesting level.",
               "invalidMessage" : "Enter a numeric value.",
               "value"          : (nestingLevelValue ? nestingLevelValue.toString() : "0")
            };
         
            nestingLevelWidget = new NumberTextBox(tfCfg, "nestingLevel");
            nestingLevelWidget.startup();
         });
      }
      
      /**
       * Create and attach to the toolbox the starting point combo-box element.
       *
       * @param   $toolbox
       *          The HTML toolbox element.
       */
      function createStartingPoints($toolbox)
      {
         var $startingPoints = $("<div></div>", {id: "startingPoints"});

         $toolbox.append($("<label for='startingPoints'>File Filter:</label>"));
         $toolbox.append($startingPoints);
         
         require(["dojo/store/Memory", "dijit/form/ComboBox", "dojo/domReady!"],
         function(Memory, ComboBox)
         {
            var widget = dijit.byId("startingPoints");
            if (widget)
            {
               widget.destroy(true);
            }
            
            // call the filter profile API to get the list
            fwd.flt.readFilterProfileList(function (filters, current)
            {
               if (filters && filters.length)
               {
                  var fileSets = [];
                  
                  fileSets.push({ "name": "Root List", "id": "-1" });
                  
                  for (var i = 0; i < filters.length; i++)
                  {
                     fileSets.push({ "name": filters[i].name, "id": filters[i].id + "" });
                     
                     if (profileIdValue > 0 && filters[i].id == profileIdValue)
                     {
                        startingPointsValue = filters[i].name;
                     }
                  }
                  
                  var memStore = new Memory(
                  {
                     "idProperty": "id",
                     "data"      : fileSets
                  });
                  
                  var cbCfg =
                  {
                     "name"        : "startingPoints",
                     "placeHolder" : "Pick a list of starting files or type a filename.",
                     "store"       : memStore,
                     "searchAttr"  : "name",
                     "value"       : (startingPointsValue ? startingPointsValue.toString()
                                                          : "Root List")
                  };
                  
                  startingPointsWidget = new ComboBox(cbCfg, "startingPoints");
                  startingPointsWidget.startup();
               }
            });
         });
      }
      
      function createLoadGraph($toolbox)
      {
         var $button = $("<button>Load Graph</button>", {id: "loadGraph"});
         var $p      = $("<p></p>");
         $toolbox.append($p);
         $p.append($button);
         
         $button.on("click",
         function()
         {
            var newStartingPoints = startingPointsWidget.get("value");
            var newProfileId = startingPointsWidget.item ? startingPointsWidget.item.id : 0;
            var newNestingLevel = nestingLevelWidget.get("value");
            var newDirection = directionBothWidget.get("checked")
                                 ? direction.BOTH_DIRECTIONS 
                                 : directionInWidget.get("checked") ? direction.UPSTREAM 
                                                                    : direction.DOWNSTREAM;
            var newDetail = detailAllWidget.get("checked")
                              ? graphDetail.ALL 
                              : detailFilesWidget.get("checked") ? graphDetail.FILES 
                                                                 : graphDetail.ENTRY_POINTS;

            if (newStartingPoints == "Root List")
            {
               newStartingPoints = null;
            }
            else
            {
               newStartingPoints = newStartingPoints.split(",");
            }
            
            toggleToolbox($toolbox, null, false);

            if (newStartingPoints == startingPointsValue &&
                newNestingLevel == nestingLevelValue     &&
                directionValue == newDirection           &&
                graphDetailValue == newDetail)
            {
               // nothing's changed, exit
               return;
            }
            
            nestingLevelValue = newNestingLevel;
            startingPointsValue = newStartingPoints;
            profileIdValue = newProfileId;
            directionValue = newDirection;
            graphDetailValue = newDetail;
            
            showCallGraphView();
         });
      }
      
      /**
       * Switch the visibility on the toolbox and the toolbox icon.
       *
       * @param    active
       *           Boolean value, if <code>true</code> we activate the toolbox and hide the icon,
       *           vice versa on <code>false</code>.
       */
      function toggleToolbox($toolbox, evt, active)
      {
         if (active)
         {
            $toolbox.show();
                         
            toolboxIcon.transition()
                       .duration(1500)
                       .ease(d3.easeElastic)
                       .attr("opacity", 0)
                       .attr("visibility", "hidden");
         }
         else
         {
            if (!evt || evt.target.id == "toolbox")
            {
               $toolbox.hide();
            
               toolboxIcon.transition()
                          .duration(1500)
                          .ease(d3.easeExp)
                          .attr("visibility", "display")
                          .attr("opacity", 1);
            }
         }
      }
      
      /**
       * Show or hide all the inter-program links, depending on the flag.
       *
       * @param    vis
       *           If true, make all the inter-program links visible.
       */
      function showHideAllLinks(vis)
      {
         // reset the hiddenCount
         for (var i = 0; i < data.links.length; i++)
         {
            var l = data.links[i];
            
            if (l.hiddenCount === undefined)
            {
               continue;
            }
            
            l.hiddenCount = 0;
         }
         
         for (var i = 0; i < data.nodes.length; i++)
         {
            var d = data.nodes[i];
            
            if (!d.hiddenLinks)
            {
               continue;
            }
            
            for (var j = 0; j < d.hiddenLinks.length; j++)
            {
               var dl = d.hiddenLinks[j];
               dl.hidden = !vis;

               if (!vis)
               {
                  var pl = linkMap[dl.parentId];
                  pl.hiddenCount += 1;
               }
            }
         }

         link.filter(function(l) { return !(l.hiddenCount === undefined); })
             .attr("stroke-width", calcLinkStrokeWidth)
             .style("visibility", vis ? "hidden" : "visible" );

         node.selectAll("polygon")
             .filter(function(n) { return (n.hiddenLinks && n.hiddenLinks.length > 0); })
             .attr("fill", calcStarColor);

         link.filter(function(l) { return l.text != "contains" && l.parentId !== null; })
             .style("visibility", vis ? "visible" : "hidden");
      }
      
      /**
       * Show or hide the inter-program links landing on this node, by toggling their visible
       * state.
       *
       * @param    d
       *           The datum associated with this node.
       */
      function showHideLinks(d)
      {
         if (!d.hiddenLinks)
         {
            return;
         }
         
         // walk through all its hidden links and make them visible (at both ends)
         for (var i = 0; i < d.hiddenLinks.length; i++)
         {
            var dl = d.hiddenLinks[i];
            var prevHidden = dl.hidden;
            dl.hidden = d3.event.shiftKey ? false : d3.event.ctrlKey ? true : !dl.hidden;
            
            if (prevHidden != dl.hidden)
            {
               // adjust only if state has changed
               var pl = linkMap[dl.parentId];
               pl.hiddenCount += (dl.hidden ? 1 : -1);

               link.filter(function(l) { return (l.id === dl.parentId); })
                   .attr("stroke-width", calcLinkStrokeWidth)
                   .style("visibility", function(l) { return l.hiddenCount == 0 ? "hidden" : "visible"; });

               node.selectAll("polygon")
                   .filter(function(n) { return (n.id === dl.source.id || n.id == dl.target.id); })
                   .attr("fill", calcStarColor);
               
               link.filter(function(l) { return (l.id === dl.id); })
                   .style("visibility", function(l) { return l.hidden ? "hidden" : "visible"; });
            }
         }
      }
      
      /**
       * Process the node selection event.  On shift-click we will load the source view if the
       * node is an AST.
       *
       * @param   {Object} d
       *          Node at the base of the branch to be expanded or collapsed.
       */
      function onNodeSelected(d)
      {
         if (d3.event.shiftKey)
         {
            var g = d3.select(this);
            
            if (!g.empty())
            {
               if (typeof data.asts[g.datum().id] !== "undefined")
               {
                  var callsite = data.asts[g.datum().id];
                  
                  fwd.rpt.showSourceAstView("#callGraphView",
                                            callsite.fid,
                                            callsite.filename,
                                            callsite.astId,
                                            callsite.srcType,
                                            callsite.line,
                                            callsite.column,
                                            callsite.callSite);
               }
            }
         }

         // if this is a MORE node, re-load the callgraph
         if (d.type == tokens.MORE && (d3.event.shiftKey || d3.event.ctrlKey))
         {
            var fname = d.text + ".ast";
            var files = data.rootList;

            // reset the profile ID
            profileIdValue = 0;
            
            if (d3.event.shiftKey)
            {
               // SHIFT-click means this program is root
               startingPointsValue = [ fname ];
            }
            else
            {
               // CTRL-click means this program is added to the list
               startingPointsValue = files;
               startingPointsValue.push(fname);
            }
            
            showCallGraphView();
         }
      }
      
      /**
       * Handler for the &quot;nodeenter&quot; event.  Displays AST details if available.
       * 
       * @param   {Object} e
       *          Event object.
       */
      function onNodeEnter(e)
      {
         var g = d3.select(this);
         
         if (!g.empty())
         {
            if (typeof data.asts[g.datum().id] !== "undefined")
            {
               var callsite = data.asts[g.datum().id];
               
               var ast =
               {
                  "id"    : callsite.nodeId,
                  "token" : callsite.tokenType,
                  "text"  : callsite.tokenText,
                  "l0"    : callsite.line,
                  "c0"    : callsite.column,
               };
               
               fwd.ast.showInfoPanel(svg, ast);
            }
         }
      }
      
      /**
       * Handler for the &quot;nodeexit&quot; event. Clears the AST details, if shown.
       * 
       * @param   {Object} e
       *          Event object.
       */
      function onNodeExit(e)
      {
         fwd.ast.removeInfoPanel(svg);
      }
   
      /**
       * Start drag event processing.
       *
       * @param    d
       *           The datum for the node being dragged.
       */
      function dragstarted(d)
      {
         if (!d3.event.active)
         {
            simulation.alphaTarget(0.3).restart();
         }
         
         d.fx = d.x;
         d.fy = d.y;
      }
      
      /**
       * Implement the core dragging portion of drag event processing.
       *
       * @param    d
       *           The datum for the node being dragged.
       */
      function dragged(d)
      {
         d.fx = d3.event.x;
         d.fy = d3.event.y;
      }
      
      /**
       * End the drag event processing.
       *
       * @param    d
       *           The datum for the node being dragged.
       */
      function dragended(d)
      {
         if (!d3.event.active)
         {
            simulation.alphaTarget(0);
         }
         
         // leave the dropped node at a fixed point
         d.fx = d.x;
         d.fy = d.y;
      }
      
      /**
       * Clear the fixed position values from the node. 
       *
       * @param    d
       *           The datum for the node being double clicked.
       */
      function dblclick(d)
      {
         d3.event.stopPropagation();
         d.fx = null;
         d.fy = null;
      }
      
      /**
       * Resize the SVG (height and width) and the parent container's height in response to a
       * browser window resize.
       */
      function resize()
      {
         // obtain the current parent container size (can we use padding and clientWidth here?)
         var width  = $parent.width() - SVG_MARGIN;
         var height = $(window).height() * 0.8;
         
         // force the parent container to a new height
         $parent.css( { "height" : height + "px" });
         
         // force the svg element to the new size
         svg.attr("height", height - SVG_MARGIN)
            .attr("width", width);
      }
      
      /**
       * Create a string describing the details of a point.
       *
       * @param    pt
       *           The point to dump.  There must be x and y members.
       */
      function stringifyPoint(pt)
      {
         return "(" + pt.x + ", " + pt.y + ")"
      }
      
      /**
       * Print details of a point to the console.
       *
       * @param    pt
       *           The point to dump.  There must be x and y members.
       * @param    txt
       *           Descriptive text to explain about the log entry.
       */
      function printPoint(pt, txt)
      {
         console.log(txt + " " + stringifyPoint(pt));
      }
      
      /**
       * Print details of a bounding rectangle to the console.
       *
       * @param    bounds
       *           The bounds to dump.  There must be a x, y, width and height members.
       * @param    txt
       *           Descriptive text to explain about the log entry.
       */
      function printBounds(bounds, txt)
      {
         console.log(txt + " location " + stringifyPoint(bounds) +
                           "; width " + bounds.width +
                           "; height " + bounds.height);
      }
      
      /**
       * Print details of a rectangle to the console.
       *
       * @param    rect
       *           The rectangle to dump.  There must be a top left point (x1, y1), a bottom
       *           right point (x2, y2) and optionally a center point (cx, cy).
       * @param    txt
       *           Descriptive text to explain about the log entry.
       */
      function printRect(rect, txt)
      {
         var center = (rect.cx) ? "; center " + stringifyPoint({ "x": rect.cx, "y": rect.cy })
                                : "";
         
         console.log(txt + " topLeft " + stringifyPoint({ "x": rect.x1, "y": rect.y1 }) +
                           "; bottomRight " + stringifyPoint({ "x": rect.x2, "y": rect.y2 }) +
                           center);
      }
   }
   
   me.showCallGraphView = showCallGraphView;
   me.showReportMissingTargets = showReportMissingTargets;
   me.showReportExternalDependencies = showReportExternalDependencies;
   me.showReportDeadCode = showReportDeadCode;
   me.showReportAmbiguousCallSites = showReportAmbiguousCallSites;
   me.showReportVerifySchemaTriggers = showReportVerifySchemaTriggers;
   me.showReportDependencies = showReportDependencies;
   
   
   /**
    * Obtain the specified graph from the server.
    *
    * @param    files
    *           The list of AST filenames to use as the starting point for the graph snippet
    *           or <code>null</code> to use the project's preconfigured root node list.
    * @param    profileId
    *           If greater than 0, then is the ID of a filter profile.
    * @param    direction
    *           One of the direction constants from this class for {@link #BOTH_DIRECTIONS},
    *           {@link #DOWNSTREAM} or {@link #UPSTREAM}. This controls which edges (inbound
    *           and/or outbound) are included.  UPSTREAM maps to inbound and DOWNSTREAM maps
    *           to outbound.
    * @param    levels
    *           This defines the number of cross-file levels of traversal to include in the
    *           snippet. Since some nodes are contained (using edges labeled "contains") in
    *           other nodes, using a simple calculation of edge levels would lead to inconsistent
    *           results.  Instead, we track the number of times that the graph traveral crosses
    *           from one file domain resource into another file domain resource. Use 0 to
    *           include only the starting nodes, 1 to include the starting nodes plus the nodes
    *           that can be traversed through 1 edge (in the direction specified), 2 for
    *           traversal through 2 edges and so forth. -1 means there is no limit. Use the no
    *           limit options with caution as there may be performance and memory issues with
    *           the result.
    * @param    detail
    *           The graph detail value - one of the FILES, ENTRY_POINTS or ALL values.
    *
    * @return   The specified subset of the graph.
    */
   function readGraph(files, profileId, direction, levels, detail)
   {
      if (files && files[0] && files[0] == "Include All")
      {
         files = null;
         levels = -1;
      }
      
      fwd.showOverlay("Reading Call Graph");
      
      ws.sendMessage(msgTypes.MST_READ_GRAPH,
                    [files, profileId, direction, levels, detail],
                    function(err, graph)
      {
         fwd.hideOverlay();
         
         data = graph;
         
         if (graph.nodes.length == 0)
         {
            console.log("No data returned (" + graph.nodes.length + " nodes, " +
                        graph.links.length + " links)");
         }
         
         loadGraph();
      });
   }
   
   me.readGraph = readGraph;
   
   // return our exported interface
   return me;
   
});