/*
** Module   : EncryptBase64Value.java
** Abstract : read the given base64 text, convert it to binary, encrypt it and convert it back to
**            base64
**
** Copyright (c) 2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- ---------------------------------Description----------------------------------
** 001 GES 20170203 Expose a helper routine as a static.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.security;

import java.io.*;

import com.goldencode.p2j.directory.*;

/**
 * Utility program to read the given base64 text, convert it to binary, encrypt it and convert
 * it back to base64.  The given password will be used for encryption.
 */
class EncryptBase64Value
{
   /**
    * Command line utility to encrypt text using AES encryption and a given password.
    * 
    * <pre>
    *   EncryptBase64Value base64_text_to_encrypt password
    * </pre> 
    */
   public static void main(String[] args)
   {
      if (args.length != 2 || args[0].trim().length() == 0 || args[1].trim().length() == 0)
      {
         System.out.println("Syntax: EncryptBase64Value <base64_text_to_encrypt> <password>");
         System.exit(-1);
      }
      
      try
      {
         String pw        = args[1].trim();
         byte[] decoded   = Base64.base64ToByteArray(args[0].trim());
         byte[] encrypted = BCCertFactory.cipherBytes(true, decoded, pw);  
         String recoded   = Base64.byteArrayToBase64(encrypted);
         
         System.out.printf("BASE64 encoded encrypted value: %s\n", recoded);
         System.out.printf("BASE64 encoded password: %s\n", 
                           Base64.byteArrayToBase64(pw.getBytes()));
      }
      
      catch (Exception exc)
      {
         exc.printStackTrace();
      }
   }
}