/*
** Module   : SystemRights.java
** Abstract : Implements Rights interface for the "system" abstract resource. 
**
** Copyright (c) 2004-2017, Golden Code Development Corporation.
**
** -#- -I- --Date----JPRM-- ----------------Description-----------------------
** 001 NVS 20050318  @20425 Created the initial version. This class 
**                          represents instances of access rights to the
**                          "system" abstract resources.
** 002 NVS 20050414  @20709 toString now encloses condition in {}.
** 003 NVS 20090701  @43000 Implements Serializable interface.
** 004 NVS 20090703  @43043 Implemented toDirectoryNode() method, which was
**                          added to the Rights interface.
** 005 SBI 20170224         Changed to be a simple DTO, moved business methods out.
*/ 
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.security;

import java.io.Serializable;
import java.util.LinkedHashMap;
import java.util.Map;

/**
 * Implements the "system" rights objects. This resource currently has one
 * instance named "logon". This instance controls access to the system logon
 * procedure. 
 */
public class SystemRights
implements Rights,
           Serializable
{
   /** access condition (boolean expression) */
   private String condition = null;
   
   /**
    * Default constructor. 
    */
   public SystemRights()
   {
      condition = "false";
   }

   /**
    * Package private constructor. 
    *
    * @param condition
    *        boolean expression that allows access when it evaluates to 
    *        <code>true</code>
    */
   public SystemRights(String condition)
   {
      this.condition = condition;
   }

   /**
    * Converts this object to string. 
    *
    * @return string representation of this object
    */
   public String toString()
   {
      return new String("{" + condition + "}");
   }

   /**
    * Gets the condition. 
    *
    * @return the boolean expression that determines the access condition
    */
   public String getCondition()
   {
      return condition;
   }
   
   /**
    * Returns rights name, a directory known class name.
    * 
    * @return   a directory known class name
    */
   @Override
   public String getRightsName()
   {
      return "systemRights";
   }

   /**
    * Returns its attributes map.
    * 
    * @return   A map of an attribute name to its value
    */
   @Override
   public Map<String, Object[]> getAttributes()
   {
      Map<String, Object[]> map = new LinkedHashMap<>(1);
      
      map.put("check", new Object[] {condition});
      
      return map;
   }
}