/*
** Module   : HintsConstants.java
** Abstract : UAST hints constants used in XML grammar
**
** Copyright (c) 2005-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- --JPRM-- ----------------------------Description-----------------------------
** 001 GES 20050204   @19621 Created initial version with appropriate
**                           constants for both database loading and
**                           passing runtime arguments to the 
**                           preprocessor.
** 002 GES 20050210   @19748 Added support for aliases, variables and
**                           widgets (includes frames too).
** 003 GES 20050314   @20322 Added support for generic UAST hints of
**                           string, string array, boolean, long and
**                           double types.  Each one has a unique name
**                           that is used as a key to lookup the value.
** 004 ECF 20050713   @21698 Added ELEM_SCHEMA and ELEM_TABLE. Used for
**                           the schema hints sub-hierarchy.
** 005 GES 20070313   @32397 Added ELEM_PROPATH.
** 006 GES 20070403   @32715 Added ELEM_ESC.
** 007 GES 20070423   @33208 Added ELEM_MARKER.
** 008 GES 20070816   @34890 Added ELEM_OPSYS, ELEM_BATCH and ELEM_WINSYS.
** 009 GES 20070904   @34949 Added ELEM_E4GL, ELEM_WSOPTIONS, ELEM_MODE
**                           and ELEM_OUTPUT.
** 010 GES 20070919   @35136 Added ATTR_CLASSNAME.
** 011 GES 20080324   @37662 Added ATTR_POS and changed ELEM_ARGS to
**                           ELEM_ARG.
** 012 GES 20081007   @40355 Added ELEM_BASEPATH.
** 013 VMN 20140413          Added ATTR_ESCAPE.
** 014 ECF 20160407          Added call graph elements.
** 015 GES 20170303          Added ELEM_DEBUG.
** 016 HC  20170708          Added source-charset uast preprocessor hint.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.uast;

/**
 * Constant values for the XML grammar used for hints.
 */
public interface HintsConstants
{
   /** Database element tag name. */
   public static final String ELEM_DATABASE = "database";
   
   /** Preprocessor element tag name. */
   public static final String ELEM_PREPROC = "preprocessor";
   
   /** Arguments element tag name. */
   public static final String ELEM_ARG = "argument";
   
   /** Honor UNIX escape sequences element tag name. */
   public static final String ELEM_ESC = "unix-escapes";
   
   /** Preprocessor marker element tag name. */
   public static final String ELEM_MARKER = "marker";
   
   /** Preprocessor BATCH-MODE element tag name. */
   public static final String ELEM_BATCH = "batch";
   
   /** Preprocessor OPSYS element tag name. */
   public static final String ELEM_OPSYS = "opsys";
   
   /** Preprocessor WINDOW-SYSTEM element tag name. */
   public static final String ELEM_WINSYS = "winsys";
   
   /** Preprocessor DEBUG element tag name. */
   public static final String ELEM_DEBUG = "debug";

   /** Preprocessor SOURCE-CHARSET element tag name. */
   public static final String SOURCE_CHARSET = "source-charset";

   /** Alias element tag name. */
   public static final String ELEM_ALIAS = "alias";
   
   /** Variable element tag name. */
   public static final String ELEM_VAR = "variable";
   
   /** <code>PROPATH</code> customization element tag. */
   public static final String ELEM_PROPATH = "propath";      
   
   /** Root element tag name of schema sub-hierarchy */
   public static final String ELEM_SCHEMA = "schema";
   
   /** Table element tag name */
   public static final String ELEM_TABLE = "table";
   
   /** Callgraph element tag name */
   public static final String ELEM_CALLGRAPH = "callgraph";
   
   /** Callgraph rootlist element tag name */
   public static final String ELEM_ROOTLIST = "rootlist";
   
   /** Callgraph include-spec element tag name */
   public static final String ELEM_INCLUDE_SPEC = "include-spec";
   
   /** Generic UAST hint element tag name. */
   public static final String ELEM_UAST = "uast";
   
   /** Generic UAST hint element array value tag name. */
   public static final String ELEM_ARRAY = "array-val";
   
   /** Generic UAST hint element e4gl tag name. */
   public static final String ELEM_E4GL = "e4gl";
   
   /** Generic UAST hint element wsoptions tag name. */
   public static final String ELEM_WSOPTIONS = "wsoptions";
   
   /** Generic UAST hint element mode tag name. */
   public static final String ELEM_MODE = "wsmode";
   
   /** Generic UAST hint element output tag name. */
   public static final String ELEM_OUTPUT = "output";
   
   /** Generic UAST hint element basepath tag name. */
   public static final String ELEM_BASEPATH = "basepath";
   
   /** Name attribute used in multiple elements. */
   public static final String ATTR_NAME = "name";
   
   /** Value attribute used in multiple elements. */
   public static final String ATTR_VAL = "value";   
   
   /** Database attribute used in alias elements. */
   public static final String ATTR_DB = "database";   
   
   /** Token type attribute used in variable/widget elements. */
   public static final String ATTR_TYPE = "type";   
   
   /** Class name attribute used in variable/widget elements. */
   public static final String ATTR_CLASSNAME = "classname";
   
   /** Data type attribute used in UAST elements. */
   public static final String ATTR_DTYPE = "datatype";
   
   /** Escape attribute used in schema/table/field elements. */
   public static final String ATTR_ESCAPE = "escape";
   
   /** Positional number used in preprocessor arguments. */
   public static final String ATTR_POS = "pos";      
}