/*
** Module   : JavaAst.java
** Abstract : A Java-specific AST.
**
** Copyright (c) 2005-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- --JPRM-- ----------------Description-----------------
** 001 GES 20050323   @20487 Created initial version.
** 002 GES 20090518   @42408 Import change.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.uast;

import antlr.*;
import com.goldencode.ast.*;

/**
 * Provides symbolic token name lookups that are specific to the Java
 * AST, all other function is implemented in the super class.
 */
public class JavaAst
extends AnnotatedAst
{
   /**
    * Default constructor.
    */
   public JavaAst()
   {
      super();
   }
   
   /**
    * Constructor which accepts and wrappers token.
    *
    * @param   token
    *          Token upon which this node is based.
    */
   public JavaAst(Token token)
   {
      super(token);
   }
   
   /**
    * Translates a text representation of a token type into the
    * actual integer value.
    *
    * @param    tokenName
    *           The text representation of the token type.
    *
    * @return   A valid integer token type value or -1 if no match was found.
    */
   public int lookupTokenType(String tokenName)
   {   
      return JavaSymbolResolver.lookupTokenType(tokenName);
   }
   
   /**
    * Translates an integer value of a token type into the associated 
    * human readable text representation.
    *
    * @param    type
    *           The integer token type.
    *
    * @return   A valid symbolic token type name or <code>null</code> if
    *           no match was found.
    */
   public String lookupTokenName(int type)
   {   
      return JavaSymbolResolver.lookupTokenName(type);
   }
   
   /**
    * Returns the symbolic representation of this AST's token type.
    *
    * @return  Symbolic token name for this AST's token type.
    */
   public String getSymbolicTokenType()
   {
      return JavaSymbolResolver.lookupTokenName(getType());
   }
}