/*
** Module   : AggregatorElement.java
** Abstract : represents the relationship between a widget and its statistics
**            aggregator
**
** Copyright (c) 2006-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- -T- --JPRM-- ----------------Description-----------------
** 001 GES 20060428 NEW   @25827 Created initial version to provide a
**                               relationship between a widget and its
**                               statistics aggregator.
** 002 ECF 20060503 DEL   @25958 Removed compareTo(), equals() and hashCode().
** 003 GES 20060915 CHG   @29627 Added support for format strings.
** 004 SIY 20071221 CHG   @36557 Added hashCode() and equals() implementation.
**                               Ref: #36556
** 005 CA  20081006 CHG   @40055 Modified equals(). An aggregator is uniquely
**                               identified by its widget and its type.
*/ 
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.ui;

import com.goldencode.p2j.util.*;

/**
 * This class provides an association between a widget and a statistics
 * aggregator.
 */
public class AggregatorElement
extends Element
{
   /** The statistics aggregator. */
   private Accumulator aggr = null;
   
   /**
    * Create an instance.
    *
    * @param    data
    *           The data to display/edit in the given widget.  Must NOT be
    *           <code>null</code>.
    * @param    widget
    *           The widget instance representing the exact frame field and
    *           state of that field.  Must NOT be <code>null</code>.
    * @param    aggr
    *           The statistics aggregator for this widget.
    */
   public AggregatorElement(Accessor      data,
                            GenericWidget widget,
                            Accumulator   aggr)
   {
      super(data, widget);
      this.aggr = aggr;
   }
   
   /**
    * Create an instance.
    *
    * @param    data
    *           The data to display/edit in the given widget.  Must NOT be
    *           <code>null</code>.
    * @param    widget
    *           The widget instance representing the exact frame field and
    *           state of that field.  Must NOT be <code>null</code>.
    * @param    aggr
    *           The statistics aggregator for this widget.
    */
   public AggregatorElement(BaseDataType  data,
                            GenericWidget widget,
                            Accumulator   aggr)
   {
      super(data, widget);
      this.aggr = aggr;
   }
   
   /**
    * Create an instance.
    * <p>
    * <b>WARNING: this form of the constructor allows primative (non-wrapper)
    * types to be copied to the screen buffer BUT any assignment back to such 
    * widgets will silently fail since there is no backing wrapper instance
    * in the calling code to which to assign.</b>
    *
    * @param    data
    *           The data to display/edit in the given widget.  Must NOT be
    *           <code>null</code>.
    * @param    widget
    *           The widget instance representing the exact frame field and
    *           state of that field.  Must NOT be <code>null</code>.
    * @param    aggr
    *           The statistics aggregator for this widget.
    */
   public AggregatorElement(String        data,
                            GenericWidget widget,
                            Accumulator   aggr)
   {
      super(new character(data), widget);
      this.aggr = aggr;
   }
   
   /**
    * Create an instance.
    * <p>
    * <b>WARNING: this form of the constructor allows primative (non-wrapper)
    * types to be copied to the screen buffer BUT any assignment back to such 
    * widgets will silently fail since there is no backing wrapper instance
    * in the calling code to which to assign.</b>
    *
    * @param    data
    *           The data to display/edit in the given widget.
    * @param    widget
    *           The widget instance representing the exact frame field and
    *           state of that field.  Must NOT be <code>null</code>.
    * @param    aggr
    *           The statistics aggregator for this widget.
    */
   public AggregatorElement(int           data,
                            GenericWidget widget,
                            Accumulator   aggr)
   {
      super(new integer(data), widget);
      this.aggr = aggr;
   }
   
   /**
    * Create an instance.
    * <p>
    * <b>WARNING: this form of the constructor allows primative (non-wrapper)
    * types to be copied to the screen buffer BUT any assignment back to such 
    * widgets will silently fail since there is no backing wrapper instance
    * in the calling code to which to assign.</b>
    *
    * @param    data
    *           The data to display/edit in the given widget.
    * @param    widget
    *           The widget instance representing the exact frame field and
    *           state of that field.  Must NOT be <code>null</code>.
    * @param    aggr
    *           The statistics aggregator for this widget.
    */
   public AggregatorElement(double        data,
                            GenericWidget widget,
                            Accumulator   aggr)
   {
      super(new decimal(data), widget);
      this.aggr = aggr;
   }
   
   /**
    * Create an instance.
    * <p>
    * <b>WARNING: this form of the constructor allows primative (non-wrapper)
    * types to be copied to the screen buffer BUT any assignment back to such 
    * widgets will silently fail since there is no backing wrapper instance
    * in the calling code to which to assign.</b>
    *
    * @param    data
    *           The data to display/edit in the given widget.
    * @param    widget
    *           The widget instance representing the exact frame field and
    *           state of that field.  Must NOT be <code>null</code>.
    * @param    aggr
    *           The statistics aggregator for this widget.
    */
   public AggregatorElement(boolean       data,
                            GenericWidget widget,
                            Accumulator   aggr)
   {
      super(new logical(data), widget);
      this.aggr = aggr;
   }
   
   /**
    * Create an instance.
    *
    * @param    data
    *           The data to display/edit in the given widget.  Must NOT be
    *           <code>null</code>.
    * @param    fmt
    *           The format string to associate with the data value. May be
    *           <code>null</code>.
    * @param    widget
    *           The widget instance representing the exact frame field and
    *           state of that field.  Must NOT be <code>null</code>.
    * @param    aggr
    *           The statistics aggregator for this widget.
    */
   public AggregatorElement(Accessor      data,
                            String        fmt,
                            GenericWidget widget,
                            Accumulator   aggr)
   {
      super(data, fmt, widget);
      this.aggr = aggr;
   }
   
   /**
    * Create an instance.
    *
    * @param    data
    *           The data to display/edit in the given widget.  Must NOT be
    *           <code>null</code>.
    * @param    fmt
    *           The format string to associate with the data value. May be
    *           <code>null</code>.
    * @param    widget
    *           The widget instance representing the exact frame field and
    *           state of that field.  Must NOT be <code>null</code>.
    * @param    aggr
    *           The statistics aggregator for this widget.
    */
   public AggregatorElement(BaseDataType  data,
                            String        fmt,
                            GenericWidget widget,
                            Accumulator   aggr)
   {
      super(data, fmt, widget);
      this.aggr = aggr;
   }
   
   /**
    * Create an instance.
    * <p>
    * <b>WARNING: this form of the constructor allows primative (non-wrapper)
    * types to be copied to the screen buffer BUT any assignment back to such 
    * widgets will silently fail since there is no backing wrapper instance
    * in the calling code to which to assign.</b>
    *
    * @param    data
    *           The data to display/edit in the given widget.  Must NOT be
    *           <code>null</code>.
    * @param    fmt
    *           The format string to associate with the data value. May be
    *           <code>null</code>.
    * @param    widget
    *           The widget instance representing the exact frame field and
    *           state of that field.  Must NOT be <code>null</code>.
    * @param    aggr
    *           The statistics aggregator for this widget.
    */
   public AggregatorElement(String        data,
                            String        fmt,
                            GenericWidget widget,
                            Accumulator   aggr)
   {
      super(new character(data), fmt, widget);
      this.aggr = aggr;
   }
   
   /**
    * Create an instance.
    * <p>
    * <b>WARNING: this form of the constructor allows primative (non-wrapper)
    * types to be copied to the screen buffer BUT any assignment back to such 
    * widgets will silently fail since there is no backing wrapper instance
    * in the calling code to which to assign.</b>
    *
    * @param    data
    *           The data to display/edit in the given widget.
    * @param    fmt
    *           The format string to associate with the data value. May be
    *           <code>null</code>.
    * @param    widget
    *           The widget instance representing the exact frame field and
    *           state of that field.  Must NOT be <code>null</code>.
    * @param    aggr
    *           The statistics aggregator for this widget.
    */
   public AggregatorElement(int           data,
                            String        fmt,
                            GenericWidget widget,
                            Accumulator   aggr)
   {
      super(new integer(data), fmt, widget);
      this.aggr = aggr;
   }
   
   /**
    * Create an instance.
    * <p>
    * <b>WARNING: this form of the constructor allows primative (non-wrapper)
    * types to be copied to the screen buffer BUT any assignment back to such 
    * widgets will silently fail since there is no backing wrapper instance
    * in the calling code to which to assign.</b>
    *
    * @param    data
    *           The data to display/edit in the given widget.
    * @param    fmt
    *           The format string to associate with the data value. May be
    *           <code>null</code>.
    * @param    widget
    *           The widget instance representing the exact frame field and
    *           state of that field.  Must NOT be <code>null</code>.
    * @param    aggr
    *           The statistics aggregator for this widget.
    */
   public AggregatorElement(double        data,
                            String        fmt,
                            GenericWidget widget,
                            Accumulator   aggr)
   {
      super(new decimal(data), fmt, widget);
      this.aggr = aggr;
   }
   
   /**
    * Create an instance.
    * <p>
    * <b>WARNING: this form of the constructor allows primative (non-wrapper)
    * types to be copied to the screen buffer BUT any assignment back to such 
    * widgets will silently fail since there is no backing wrapper instance
    * in the calling code to which to assign.</b>
    *
    * @param    data
    *           The data to display/edit in the given widget.
    * @param    fmt
    *           The format string to associate with the data value. May be
    *           <code>null</code>.
    * @param    widget
    *           The widget instance representing the exact frame field and
    *           state of that field.  Must NOT be <code>null</code>.
    * @param    aggr
    *           The statistics aggregator for this widget.
    */
   public AggregatorElement(boolean       data,
                            String        fmt,
                            GenericWidget widget,
                            Accumulator   aggr)
   {
      super(new logical(data), fmt, widget);
      this.aggr = aggr;
   }
   
   /**
    * Determines if the element should cause statistics aggregation to be
    * activated during display operations.
    *
    * @return   Always <code>true</code>;
    */
   public boolean statsColumn()
   {
      return true;
   }
   
   /**
    * Accesses the statistics aggregator associated with this frame element.
    *
    * @return   The accumulator.
    */
   public Accumulator getAccumulator()
   {
      return aggr;
   }

   /**
    * @see java.lang.Object#equals(java.lang.Object)
    */
   public boolean equals(Object obj)
   {
      if (obj instanceof AggregatorElement)
      {
         AggregatorElement e = (AggregatorElement) obj;
         if (getWidget().getId() == e.getWidget().getId())
         {
            return getAccumulator().getClass()
                                   .equals(e.getAccumulator().getClass());
         }
         return false;
      }
      return false;
   }

   /**
    * @see java.lang.Object#hashCode()
    */
   public int hashCode()
   {
      return getWidget().getId();
   }
}