/*
** Module   : ButtonConfig.java
** Abstract : stores button specific widget state
**
** Copyright (c) 2005-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- --JPRM-- ----------------------------Description-----------------------------
** 001 EVL 20050714   @21692 Creation based on general concept. Necessary
**                           modification to get compiled.
** 002 SIY 20050831   @22488 Refactoring (see @@22394-22409).
** 003 EVL 20050908   @22612 Fix for setting action command for auto_go
**                           end auto_end_key events. Changing location
**                           of the string event constants.
** 004 SIY 20060209   @24480 Major refactoring.
** 005 SIY 20060215   @24596 Fixed colors.
** 006 GES 20060803   @28394 Made externalizable for better performance.
**                           Many code standard cleanups.
** 007 SIY 20061102   @30940 Minor changes caused by field encapsulation.
** 008 SIY 20100619          Switching to own implementation of character UI.
** 009 SIY 20100804          Changes in imports, some refactorings.
** 010 SIY 20101001          Added more button options.
** 011 SIY 20101122          Changes related to Button hierarchy cleanup.
**                           Minor cleanups.
** 012 SIY 20110112          Changes related to new color handling scheme.
** 013 SIY 20111101          Moved from CHUI package. Minor 
**                           cleanups/refactorings in preparation to GUI client.
** 014 SIY 20111202          Updated imports.
** 015 GES 20140627          Changed color attribute method names to match naming standards.
** 016 CA  20140926          Added shared widget configuration support, to allow GUI/ChUI
**                           concrete implementation for the same widget ID.  Refactored the
**                           widget configuration classes: all fields were made public; these
**                           classes need to be as dumb as possible, all logic related to
**                           setting/getting a certain field should be at the widget or at the
**                           caller. Refs #2254
** 017 EVL 20150115          Adding constants to differentiate images to be loaded.
** 018 EVL 20150205          Adding support for image phrase.
** 019 EVL 20150225          NO-CONVERT-3D-COLORS option is now processing with CONVERT-3D-COLORS
**                           attribute that is really the same(with boolean inversion).
** 020 EVL 20150227          Adding support for FLAT-BUTTON option/attribute.
** 021 HC  20150323          Removed business logic from config classes.
** 022 EVL 20150417          Adding suffix constant for prepackaged disabled images.
** 023 IAS 20151214          MOUSE-POINTER attribute support.
** 024 IAS 20160322          Changed the constructor visibility.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.ui;

import java.io.*;

/**
 * Stores button-specific state.
 */
public class ButtonConfig
extends ControlConfig
{
   /** Image widget uses this. */   
   public static final int IMAGE_IMG  = -1;
   
   /** Up image to load. */   
   public static final int IMAGE_UP   = 0;
   
   /** Down image to load. */   
   public static final int IMAGE_DOWN = 1;
   
   /** Insensitive image to load. */   
   public static final int IMAGE_INS  = 2;
   
   /** Suffix for prepackaged images insensitive state. */
   public static final String PREPACKAGED_INS_SUFFIX = "-ins";
   
   /** The Progress AUTO-END-KEY attribute. */
   public boolean autoEndKey = false;
  
   /** The Progress AUTO-GO attribute. */
   public boolean autoGo = false;
  
   /** Means whether this button is default one or not. */
   public boolean isDefault = false;

   /** Button image definition. */
   public ButtonImageDefinition imgDef = new ButtonImageDefinition();

   /** Button up image definition. */
   public ButtonImageDefinition imgUpDef = new ButtonImageDefinition();

   /** Button down image definition. */
   public ButtonImageDefinition imgDownDef = new ButtonImageDefinition();

   /** Button insensitive image definition. */
   public ButtonImageDefinition imgInsDef = new ButtonImageDefinition();

   /** Button NO-FOCUS flag. */
   public boolean noFocus = false;

   /** Button FLAT-BUTTON flag. */
   public boolean flatButton = false;

   /** Button NO-CONVERT-3D-COLORS flag and CONVERT-3D-COLORS attribute. */
   public boolean convert3D = true;
   
   /** MOUSE-POINTER attribute */
   public String mousePointer = null;

   /**
    * Default constructor (only used in deserialization).
    */
   public ButtonConfig()
   {
      init();
   }

   /**
    * Create a new config and associate it with the given widget.
    * 
    * @param    id
    *           The ID of the widget to which this configuration belongs.
    */
   public ButtonConfig(WidgetId id)
   {
      super(id);
   }

   /**
    * Create a new config and associate it with the given widget.
    * 
    * @param    id
    *           The ID of the widget to which this configuration belongs.
    */
   protected ButtonConfig(int id)
   {
      super(id);
      init();
   }

   /**
    * Set new values from the provided configuration instance.
    * 
    * @param    config
    *           The instance from which to take values.
    */
   @Override
   public void applyConfig(WidgetConfig config)
   {
      super.applyConfig(config);
      
      ButtonConfig cfg = (ButtonConfig) config;
      
      autoEndKey = cfg.autoEndKey;
      autoGo = cfg.autoGo;
      isDefault = cfg.isDefault;
      imgDef = cfg.imgDef;
      imgUpDef = cfg.imgUpDef;
      imgDownDef = cfg.imgDownDef;
      imgInsDef = cfg.imgInsDef;
      noFocus = cfg.noFocus;
      flatButton = cfg.flatButton;
      convert3D = cfg.convert3D;
      mousePointer = cfg.mousePointer;
   }

   /**
    * Replacement for the default object reading method. The latest
    * state is read from the input source.  The parent's state is
    * read first.
    * 
    * @param    in
    *           The input source from which fields will be restored.
    *
    * @throws   IOException
    *           In case of I/O errors.
    * @throws   ClassNotFoundException
    *           If payload can't be instantiated.
    */
   @Override
   public void readExternal(ObjectInput in)
   throws IOException,
          ClassNotFoundException
   {
      super.readExternal(in);
      autoEndKey = in.readBoolean();
      autoGo     = in.readBoolean();
      isDefault  = in.readBoolean();
      noFocus    = in.readBoolean();
      flatButton = in.readBoolean();
      convert3D  = in.readBoolean();
      imgDef      = (ButtonImageDefinition) in.readObject();
      imgUpDef    = (ButtonImageDefinition) in.readObject();
      imgDownDef  = (ButtonImageDefinition) in.readObject();
      imgInsDef   = (ButtonImageDefinition) in.readObject();
      mousePointer = (String) in.readObject();
  }

   /**
    * Replacement for the default object writing method. The latest
    * state is written to the output destination.  The parent's state is
    * written first.
    * 
    * @param    out
    *           The output destination to which fields will be saved.
    *
    * @throws   IOException
    *           In case of I/O errors.
    */
   @Override
   public void writeExternal(ObjectOutput out)
   throws IOException
   {
      super.writeExternal(out);
      out.writeBoolean(autoEndKey);
      out.writeBoolean(autoGo);
      out.writeBoolean(isDefault);
      out.writeBoolean(noFocus);
      out.writeBoolean(flatButton);
      out.writeBoolean(convert3D);
      out.writeObject(imgDef);
      out.writeObject(imgUpDef);
      out.writeObject(imgDownDef);
      out.writeObject(imgInsDef);
      out.writeObject(mousePointer);
   }
   
   /**
    * Initialize this config.
    */
   private void init()
   {
      dcolor = Color.NORMAL;
   }
}