/*
** Module   : ColorTableEntry.java
** Abstract : color table entry structure.
**
** Copyright (c) 2014-2017, Golden Code Development Corporation.

**
** -#- -I- --Date-- --------------------------------Description-----------------------------------
** 001 MAG 20140812 Created initial version.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.ui;

import java.io.*;

/**
 * Color table entry structure. 
 */
public class ColorTableEntry
extends ColorRgb
implements Externalizable
{
   /** Color index */
   private int index = -1;
      
   /** Dynamic attribute */
   private boolean dynamic = false;
   
   /**
    * Default constructor.
    */
   public ColorTableEntry()
   {
      super(0, 0, 0, null);
   }

   /**
    * Constructor.
    * 
    * @param   index
    *          Color index.
    * @param   red
    *          Red value.
    * @param   green
    *          Green value.
    * @param   blue
    *          Blue value.
    */
   public ColorTableEntry(int index, int red, int green, int blue)
   {
      super(red, green, blue);
      this.index = index;
   }

   /**
    * Constructor.
    * 
    * @param   index
    *          Color index.
    * @param   rgb
    *          ColorRgb instance.
    */
   public ColorTableEntry(int index, ColorRgb rgb)
   {
      super(rgb);
      this.index = index;
   }
   /**
    * Get color index.
    * 
    * @return  see above.
    */
   public int getIndex()
   {
      return index;
   }

   /**
    * Set color index.
    * 
    * @param   index
    *          New index.
    */
   public void setIndex(int index)
   {
      this.index = index;
   }

   /**
    * Check if dynamic color.
    * 
    * @return  see above.
    */
   public boolean isDynamic()
   {
      return dynamic;
   }

   /**
    * Set color as dynamic.
    * 
    * @param   dynamic
    *          <code>true</code> if dynamic.
    */
   public void setDynamic(boolean dynamic)
   {
      this.dynamic = dynamic;
   }   

   /**
    * Replacement for default object reading method. The latest
    * state is read from the input source.
    * 
    * @param    in
    *           The input source from which fields will be restored.
    *
    * @throws   IOException
    *           In case of I/O errors.
    * @throws   ClassNotFoundException
    *           If payload can't be instantiated.
    */   
   public void readExternal(ObjectInput in) 
   throws IOException, 
          ClassNotFoundException
   {
      super.readExternal(in);
      index = in.readInt();
   }

   /**
    * Replacement for default object writing method. The latest
    * state is written to the output destination.
    * 
    * @param    out
    *           The output destination to which fields will be saved.
    *
    * @throws   IOException
    *           In case of I/O errors.
    */
   public void writeExternal(ObjectOutput out) 
   throws IOException
   {
      super.writeExternal(out);
      out.writeInt(index);
   }
}