/*
** Module   : ControlSetItem.java
** Abstract : Label-value pair container.
**
** Copyright (c) 2006-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- -T- --JPRM-- ----------------Description-----------------
** 001 SIY 20060205 NEW  @24502  Created initial version
** 002 IAS 20150205 Added id and selectable fields
** 003 IAS 20150212 Renamed 'selectable' field to 'enabled' to reflect semantics
**
** 004 HC  20150323 Removed business logic from config classes.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.ui;

import java.io.Serializable;

import com.goldencode.p2j.util.*;

/**
 * This class is used to transfer pairs (label and corresponding value)
 * between client and server. At client side this class also used to hold
 * values in the configuration.
 * 
 * @author  SIY
 * @version 1.0.0
 */
public class ControlSetItem
   implements Serializable
{
   /** Item id - required for correct restore of the selection after list modification. */
   private int itemId;

   /** Item label. */
   private character label = null;
   
   /** Item value. */
   private BaseDataType value = null;
   
   /** Item is enabled for RADIO-SET */
   private boolean enabled = true;
   
   /**
    * Constructor for the <code>ControlSetItem</code> instances.
    * 
    * @param   id
    *          Item id.
    * @param   label
    *          Item label.
    * @param   value
    *          Item value.
    */
   public ControlSetItem(int id, character label, BaseDataType value)
   {
      this.itemId = id;
      this.label = label;
      this.value = value;
   }

   /**
    * Constructor for the <code>ControlSetItem</code> instances.
    * 
    * @param   label
    *          Item label.
    * @param   value
    *          Item value.
    */
   public ControlSetItem(character label, BaseDataType value)
   {
      this(0, label, value);
   }
   
   /**
    * Get item label.
    * 
    * @return  Item label.
    */
   public character getLabel()
   {
      return label;
   }
   
   /**
    * Set item label.
    * 
    * @param   label
    *          New item label.
    */
   public void setLabel(character label)
   {
      this.label = label;
   }
   
   /**
    * Get item value.
    * 
    * @return  Item value.
    */
   public BaseDataType getValue()
   {
      return value;
   }
   
   /**
    * Set item value.
    * 
    * @param   value
    *          New item value.
    */
   public void setValue(BaseDataType value)
   {
      this.value = value;
   }

   /**
    * Get item id.
    * 
    * @return  Item id.
    */
   public int getItemId()
   {
      return itemId;
   }
   
   /**
    * Set item id.
    * 
    * @param   value
    *          New item id.
    */
   public void setItemId(int value)
   {
      this.itemId = value;
   }

   /**
    * Check if item is enabled.
    * 
    * @return  value of the 'enabled' flag.
    */
   public boolean isEnabled()
   {
      return enabled;
   }

   /**
    * Set item 'enabled' flag.
    * 
    * @param   enabled
    *          New value of item's 'enabled' flag.
    */
   public void setEnabled(boolean enabled)
   {
      this.enabled = enabled;
   }
}