/*
** Module   : DirectManipulation.java
** Abstract : Direct manipulation methods and attributes definitions. 
**
** Copyright (c) 2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- ---------------------------------Description---------------------------------
** 001 EVL 20170526 Created initial version.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.ui;

import com.goldencode.p2j.util.*;

/**
 * Methods and attributes for widgets supporting direct manipulation. 
 */
public interface DirectManipulation
{
   /**
    * Gets BOX-SELECTABLE attribute.
    * 
    * @return  BOX-SELECTABLE current value.
    */
   @LegacyAttribute(name = "BOX-SELECTABLE")
   public logical isBoxSelectable();
   
   /**
    * Sets BOX-SELECTABLE attribute.
    * 
    * @param   flag
    *          New value for the attribute.
    */
   @LegacyAttribute(name = "BOX-SELECTABLE", setter = true)
   public void setBoxSelectable(boolean flag);
   
   /**
    * Sets BOX-SELECTABLE attribute.
    * 
    * @param   flag
    *          New value for the attribute.
    */
   @LegacyAttribute(name = "BOX-SELECTABLE", setter = true)
   default public void setBoxSelectable(logical flag)
   {
      if (flag != null && !flag.isUnknown())
      {
         setBoxSelectable(flag.booleanValue());
      }
      else
      {
         ErrorManager.recordOrShowError(4083, 
               String.format(
               "Unable to assign UNKNOWN value to attribute BOX-SELECTABLE on %s %s", 
               ((GenericWidget)this).type(), ((GenericWidget)this).widgetName()),
               false);
      }
   }

   /**
    * Gets NUM-SELECTED-WIDGETS attribute.
    * 
    * @return  NUM-SELECTED-WIDGETS current value.
    */
   @LegacyAttribute(name = "NUM-SELECTED-WIDGETS")
   public integer getNumSelectedWidgets();

   /**
    * Gets GRID-FACTOR-HORIZONTAL attribute.
    * 
    * @return  GRID-FACTOR-HORIZONTAL current value.
    */
   @LegacyAttribute(name = "GRID-FACTOR-HORIZONTAL")
   public integer getGridFactorHorizontal();
   
   /**
    * Sets GRID-FACTOR-HORIZONTAL attribute.
    * 
    * @param   value
    *          New value for the attribute.
    */
   @LegacyAttribute(name = "GRID-FACTOR-HORIZONTAL", setter = true)
   public void setGridFactorHorizontal(int value);
   
   /**
    * Sets GRID-FACTOR-HORIZONTAL attribute.
    * 
    * @param   value
    *          New value for the attribute.
    */
   @LegacyAttribute(name = "GRID-FACTOR-HORIZONTAL", setter = true)
   default public void setGridFactorHorizontal(integer value)
   {
      if (value != null && !value.isUnknown())
      {
         setGridFactorHorizontal(value.intValue());
      }
      else
      {
         ErrorManager.recordOrShowError(4083, 
               String.format(
               "Unable to assign UNKNOWN value to attribute GRID-FACTOR-HORIZONTAL on %s %s", 
               ((GenericWidget)this).type(), ((GenericWidget)this).widgetName()),
               false);
      }
   }

   /**
    * Gets GRID-FACTOR-VERTICAL attribute.
    * 
    * @return  GRID-FACTOR-VERTICAL current value.
    */
   @LegacyAttribute(name = "GRID-FACTOR-VERTICAL")
   public integer getGridFactorVertical();
   
   /**
    * Sets GRID-FACTOR-VERTICAL attribute.
    * 
    * @param   value
    *          New value for the attribute.
    */
   @LegacyAttribute(name = "GRID-FACTOR-VERTICAL", setter = true)
   public void setGridFactorVertical(int value);
   
   /**
    * Sets GRID-FACTOR-VERTICAL attribute.
    * 
    * @param   value
    *          New value for the attribute.
    */
   @LegacyAttribute(name = "GRID-FACTOR-VERTICAL", setter = true)
   default public void setGridFactorVertical(integer value)
   {
      if (value != null && !value.isUnknown())
      {
         setGridFactorVertical(value.intValue());
      }
      else
      {
         ErrorManager.recordOrShowError(4083, 
               String.format(
               "Unable to assign UNKNOWN value to attribute GRID-FACTOR-VERTICAL on %s %s", 
               ((GenericWidget)this).type(), ((GenericWidget)this).widgetName()),
               false);
      }
   }

   /**
    * Gets GRID-SNAP attribute.
    * 
    * @return  GRID-SNAP current value.
    */
   @LegacyAttribute(name = "GRID-SNAP")
   public logical isGridSnap();
   
   /**
    * Sets GRID-SNAP attribute.
    * 
    * @param   flag
    *          New value for the attribute.
    */
   @LegacyAttribute(name = "GRID-SNAP", setter = true)
   public void setGridSnap(boolean flag);
   
   /**
    * Sets GRID-SNAP attribute.
    * 
    * @param   flag
    *          New value for the attribute.
    */
   @LegacyAttribute(name = "GRID-SNAP", setter = true)
   default public void setGridSnap(logical flag)
   {
      if (flag != null && !flag.isUnknown())
      {
         setGridSnap(flag.booleanValue());
      }
      else
      {
         ErrorManager.recordOrShowError(4083, 
               String.format(
               "Unable to assign UNKNOWN value to attribute GRID-SNAP on %s %s", 
               ((GenericWidget)this).type(), ((GenericWidget)this).widgetName()),
               false);
      }
   }

   /**
    * Gets GRID-UNIT-HEIGHT-CHARS attribute.
    * 
    * @return  GRID-UNIT-HEIGHT-CHARS current value.
    */
   @LegacyAttribute(name = "GRID-UNIT-HEIGHT-CHARS")
   public decimal getGridUnitHeightChars();
   
   /**
    * Sets GRID-UNIT-HEIGHT-CHARS attribute.
    * 
    * @param   value
    *          New value for the attribute.
    */
   @LegacyAttribute(name = "GRID-UNIT-HEIGHT-CHARS", setter = true)
   public void setGridUnitHeightChars(double value);
   
   /**
    * Sets GRID-UNIT-HEIGHT-CHARS attribute.
    * 
    * @param   value
    *          New value for the attribute.
    */
   @LegacyAttribute(name = "GRID-UNIT-HEIGHT-CHARS", setter = true)
   default public void setGridUnitHeightChars(NumberType value)
   {
      if (value != null && !value.isUnknown())
      {
         setGridUnitHeightPixels(LogicalTerminal.coordinates().rowToPixels(value.doubleValue()));
      }
      else
      {
         ErrorManager.recordOrShowError(4083, 
               String.format(
               "Unable to assign UNKNOWN value to attribute GRID-UNIT-HEIGHT-CHARS on %s %s", 
               ((GenericWidget)this).type(), ((GenericWidget)this).widgetName()),
               false);
      }
   }

   /**
    * Gets GRID-UNIT-HEIGHT-PIXELS attribute.
    * 
    * @return  GRID-UNIT-HEIGHT-PIXELS current value.
    */
   @LegacyAttribute(name = "GRID-UNIT-HEIGHT-PIXELS")
   public integer getGridUnitHeightPixels();
   
   /**
    * Sets GRID-UNIT-HEIGHT-PIXELS attribute.
    * 
    * @param   value
    *          New value for the attribute.
    */
   @LegacyAttribute(name = "GRID-UNIT-HEIGHT-PIXELS", setter = true)
   public void setGridUnitHeightPixels(int value);
   
   /**
    * Sets GRID-UNIT-HEIGHT-PIXELS attribute.
    * 
    * @param   value
    *          New value for the attribute.
    */
   @LegacyAttribute(name = "GRID-UNIT-HEIGHT-PIXELS", setter = true)
   default public void setGridUnitHeightPixels(NumberType value)
   {
      if (value != null && !value.isUnknown())
      {
         setGridUnitHeightPixels(value.intValue());
      }
      else
      {
         ErrorManager.recordOrShowError(4083, 
               String.format(
               "Unable to assign UNKNOWN value to attribute GRID-UNIT-HEIGHT-PIXELS on %s %s", 
               ((GenericWidget)this).type(), ((GenericWidget)this).widgetName()),
               false);
      }
   }
   
   /**
    * Gets GRID-UNIT-WIDTH-CHARS attribute.
    * 
    * @return  GRID-UNIT-WIDTH-CHARS current value.
    */
   @LegacyAttribute(name = "GRID-UNIT-WIDTH-CHARS")
   public decimal getGridUnitWidthChars();
   
   /**
    * Sets GRID-UNIT-WIDTH-CHARS attribute.
    * 
    * @param   value
    *          New value for the attribute.
    */
   @LegacyAttribute(name = "GRID-UNIT-WIDTH-CHARS", setter = true)
   public void setGridUnitWidthChars(double value);
   
   /**
    * Sets GRID-UNIT-WIDTH-CHARS attribute.
    * 
    * @param   value
    *          New value for the attribute.
    */
   @LegacyAttribute(name = "GRID-UNIT-WIDTH-CHARS", setter = true)
   default public void setGridUnitWidthChars(NumberType value)
   {
      if (value != null && !value.isUnknown())
      {
         setGridUnitWidthPixels(LogicalTerminal.coordinates().columnToPixels(value.doubleValue()));
      }
      else
      {
         ErrorManager.recordOrShowError(4083, 
               String.format(
               "Unable to assign UNKNOWN value to attribute GRID-UNIT-WIDTH-CHARS on %s %s", 
               ((GenericWidget)this).type(), ((GenericWidget)this).widgetName()),
               false);
      }
   }

   /**
    * Gets GRID-UNIT-WIDTH-PIXELS attribute.
    * 
    * @return  GRID-UNIT-WIDTH-PIXELS current value.
    */
   @LegacyAttribute(name = "GRID-UNIT-WIDTH-PIXELS")
   public integer getGridUnitWidthPixels();
   
   /**
    * Sets GRID-UNIT-WIDTH-PIXELS attribute.
    * 
    * @param   value
    *          New value for the attribute.
    */
   @LegacyAttribute(name = "GRID-UNIT-WIDTH-PIXELS", setter = true)
   public void setGridUnitWidthPixels(int value);
   
   /**
    * Sets GRID-UNIT-WIDTH-PIXELS attribute.
    * 
    * @param   value
    *          New value for the attribute.
    */
   @LegacyAttribute(name = "GRID-UNIT-WIDTH-PIXELS", setter = true)
   default public void setGridUnitWidthPixels(NumberType value)
   {
      if (value != null && !value.isUnknown())
      {
         setGridUnitWidthPixels(value.intValue());
      }
      else
      {
         ErrorManager.recordOrShowError(4083, 
               String.format(
               "Unable to assign UNKNOWN value to attribute GRID-UNIT-WIDTH-PIXELS on %s %s", 
               ((GenericWidget)this).type(), ((GenericWidget)this).widgetName()),
               false);
      }
   }
   
   /**
    * Gets GRID-VISIBLE attribute.
    * 
    * @return  GRID-VISIBLE current value.
    */
   @LegacyAttribute(name = "GRID-VISIBLE")
   public logical isGridVisible();
   
   /**
    * Sets GRID-VISIBLE attribute.
    * 
    * @param   flag
    *          New value for the attribute.
    */
   @LegacyAttribute(name = "GRID-VISIBLE", setter = true)
   public void setGridVisible(boolean flag);
   
   /**
    * Sets GRID-VISIBLE attribute.
    * 
    * @param   flag
    *          New value for the attribute.
    */
   @LegacyAttribute(name = "GRID-VISIBLE", setter = true)
   default public void setGridVisible(logical flag)
   {
      if (flag != null && !flag.isUnknown())
      {
         setGridVisible(flag.booleanValue());
      }
      else
      {
         ErrorManager.recordOrShowError(4083, 
               String.format(
               "Unable to assign UNKNOWN value to attribute GRID-VISIBLE on %s %s", 
               ((GenericWidget)this).type(), ((GenericWidget)this).widgetName()),
               false);
      }
   }
}