/*
** Module   : FontTableResource.java
** Abstract : Define all APIs available via the FONT-TABLE system handle.
**
** Copyright (c) 2014-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- --------------------------------Description-----------------------------------
** 001 CA  20140701 Created initial version.
** 002 MAG 20140827 Move NUM-ENTRIES methods to NumEntries interface.
** 003 EVL 20160224 Javadoc fixes to make compatible with Oracle Java 8 for Solaris 10.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.ui;

import com.goldencode.p2j.util.*;

/**
 * Exposes the following methods or attributes from the FONT-TABLE handle:
 * <table summary="">
 * <tr>
 *    <th>FONT-TABLE attribute or method</th>
 *    <th>API</th>
 * </tr>
 * <tr>
 *    <td>GET-TEXT-HEIGHT-CHARS</td>
 *    <td>{@link #getTextHeightChars}</td>
 * </tr>
 * <tr>
 *    <td>GET-TEXT-HEIGHT-PIXELS</td>
 *    <td>{@link #getTextHeightPixels}</td>
 * </tr>
 * <tr>
 *    <td>GET-TEXT-WIDTH-CHARS</td>
 *    <td>{@link #getTextWidthChars}</td>
 * </tr>
 * <tr>
 *    <td>GET-TEXT-WIDTH-CHARS</td>
 *    <td>{@link #getTextWidthPixels}</td>
 * </tr>
 * </table>
 * This interface is used as a returned type by the 
 * {@link handle#unwrapFontTable} API, which, depending on the 
 * handle's state, it either returns an {@link FontTable#asHandle()} 
 * instance or a dynamic proxy using {@link handle#invalidAttrAccessProxy} 
 */
@LegacyResource(resource = LegacyResource.PSEUDO_WIDGET)
public interface FontTableResource
extends CommonHandle,
        InstantiatingProcedure,
        NumEntries
{
   /**
    * Get the height in characters unit, of the default font.
    * 
    * @return   See above.
    */
   @LegacyMethod(name = "GET-TEXT-HEIGHT-CHARS")
   public decimal getTextHeightChars();
   
   /**
    * Get the height in characters unit, of the specified font.  If the font number is unknown,
    * the default font is used.
    * 
    * @param    fontNum
    *           The entry in the font-table.
    * 
    * @return   See above.
    */
   @LegacyMethod(name = "GET-TEXT-HEIGHT-CHARS")
   public decimal getTextHeightChars(long fontNum);
   
   /**
    * Get the height in characters unit, of the specified font.  If the font number is unknown,
    * the default font is used.
    * 
    * @param    fontNum
    *           The entry in the font-table.
    * 
    * @return   See above.
    */
   @LegacyMethod(name = "GET-TEXT-HEIGHT-CHARS")
   public decimal getTextHeightChars(int64 fontNum);

   /**
    * Get the height in pixels, of the default font.
    * 
    * @return   See above.
    */
   @LegacyMethod(name = "GET-TEXT-HEIGHT-PIXELS")
   public integer getTextHeightPixels();
   
   /**
    * Get the height in pixels, of the specified font.  If the font number is unknown,
    * the default font is used.
    * 
    * @param    fontNum
    *           The entry in the font-table.
    * 
    * @return   See above.
    */
   @LegacyMethod(name = "GET-TEXT-HEIGHT-PIXELS")
   public integer getTextHeightPixels(long fontNum);

   /**
    * Get the height in pixels, of the specified font.  If the font number is unknown,
    * the default font is used.
    * 
    * @param    fontNum
    *           The entry in the font-table.
    * 
    * @return   See above.
    */
   @LegacyMethod(name = "GET-TEXT-HEIGHT-PIXELS")
   public integer getTextHeightPixels(int64 fontNum);

   /**
    * Get the width of the specified text, in character units, using the default font.
    * 
    * @param    txt
    *           The text.
    *           
    * @return   See above.
    */
   @LegacyMethod(name = "GET-TEXT-WIDTH-CHARS")
   public decimal getTextWidthChars(String txt);

   /**
    * Get the width of the specified text, in character units, using the default font.
    * 
    * @param    txt
    *           The text.
    *           
    * @return   See above.
    */
   @LegacyMethod(name = "GET-TEXT-WIDTH-CHARS")
   public decimal getTextWidthChars(character txt);

   /**
    * Get the width of the specified text, in character units, using the specified font.  
    * If the font number is unknown, the default font is used.
    * 
    * @param    txt
    *           The text.
    * @param    fontNum
    *           The entry in the font-table.
    *           
    * @return   See above.
    */
   @LegacyMethod(name = "GET-TEXT-WIDTH-CHARS")
   public decimal getTextWidthChars(String txt, long fontNum);

   /**
    * Get the width of the specified text, in character units, using the specified font.  
    * If the font number is unknown, the default font is used.
    * 
    * @param    txt
    *           The text.
    * @param    fontNum
    *           The entry in the font-table.
    *           
    * @return   See above.
    */
   @LegacyMethod(name = "GET-TEXT-WIDTH-CHARS")
   public decimal getTextWidthChars(character txt, long fontNum);

   /**
    * Get the width of the specified text, in character units, using the specified font.  
    * If the font number is unknown, the default font is used.
    * 
    * @param    txt
    *           The text.
    * @param    fontNum
    *           The entry in the font-table.
    *           
    * @return   See above.
    */
   @LegacyMethod(name = "GET-TEXT-WIDTH-CHARS")
   public decimal getTextWidthChars(String txt, int64 fontNum);

   /**
    * Get the width of the specified text, in character units, using the specified font.  
    * If the font number is unknown, the default font is used.
    * 
    * @param    txt
    *           The text.
    * @param    fontNum
    *           The entry in the font-table.
    *           
    * @return   See above.
    */
   @LegacyMethod(name = "GET-TEXT-WIDTH-CHARS")
   public decimal getTextWidthChars(character txt, int64 fontNum);

   /**
    * Get the width of the specified text, in pixels, using the default font.
    * 
    * @param    txt
    *           The text.
    *           
    * @return   See above.
    */
   @LegacyMethod(name = "GET-TEXT-WIDTH-PIXELS")
   public integer getTextWidthPixels(String txt);

   /**
    * Get the width of the specified text, in pixels, using the default font.
    * 
    * @param    txt
    *           The text.
    *           
    * @return   See above.
    */
   @LegacyMethod(name = "GET-TEXT-WIDTH-PIXELS")
   public integer getTextWidthPixels(character txt);

   /**
    * Get the width of the specified text, in pixels, using the specified font.  
    * If the font number is unknown, the default font is used.
    * 
    * @param    txt
    *           The text.
    * @param    fontNum
    *           The entry in the font-table.
    *           
    * @return   See above.
    */
   @LegacyMethod(name = "GET-TEXT-WIDTH-PIXELS")
   public integer getTextWidthPixels(String txt, long fontNum);

   /**
    * Get the width of the specified text, in pixels, using the specified font.  
    * If the font number is unknown, the default font is used.
    * 
    * @param    txt
    *           The text.
    * @param    fontNum
    *           The entry in the font-table.
    *           
    * @return   See above.
    */
   @LegacyMethod(name = "GET-TEXT-WIDTH-PIXELS")
   public integer getTextWidthPixels(character txt, long fontNum);

   /**
    * Get the width of the specified text, in pixels, using the specified font.  
    * If the font number is unknown, the default font is used.
    * 
    * @param    txt
    *           The text.
    * @param    fontNum
    *           The entry in the font-table.
    *           
    * @return   See above.
    */
   @LegacyMethod(name = "GET-TEXT-WIDTH-PIXELS")
   public integer getTextWidthPixels(String txt, int64 fontNum);

   /**
    * Get the width of the specified text, in pixels, using the specified font.  
    * If the font number is unknown, the default font is used.
    * 
    * @param    txt
    *           The text.
    * @param    fontNum
    *           The entry in the font-table.
    *           
    * @return   See above.
    */
   @LegacyMethod(name = "GET-TEXT-WIDTH-PIXELS")
   public integer getTextWidthPixels(character txt, int64 fontNum);
}