/*
** Module   : Group.java
** Abstract : API to group widgets
**
** Copyright (c) 2008-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- --JPRM-- ----------------------------Description------------------------------
** 001 GES 20080319  @37588 Created initial version to group and traverse
**                          widgets.
** 002 CA  20130323 This must be a CommonWidget too.
** 003 CA  20130927 getFirst/LastChild must return a handle instance.
** 004 CA  20140926 Added shared widget configuration support, to allow GUI/ChUI concrete
**                  implementation for the same widget ID.  Refactored the widget configuration
**                  classes: all fields were made public; these classes need to be as dumb as
**                  possible, all logic related to setting/getting a certain field should be at
**                  the widget or at the caller. Refs #2254
** 005 CA  20150416 Moved FIRST/LAST-CHILD APIs to separate interface (CommonHandleTree).
*/ 
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.ui;

import com.goldencode.p2j.util.*;

/**
 * Groups widgets together into a list of siblings that can be accessed
 * via the first or last entry in the list.
 */
public interface Group
extends CommonWidget,
        CommonHandleTree
{
   /**
    * Accesses the next widget in the list of contained widgets.
    *
    * @param    child
    *           The widget in the list for which to find the following child.
    *
    * @return   The next widget or <code>null</code> if there is no subsequent
    *           widget.
    */
   public GenericWidget<?> getNextChild(GenericWidget<?> child);

   /**
    * Accesses the previous widget in the list of contained widgets.
    *
    * @param    child
    *           The widget in the list for which to find the preceding child.
    *
    * @return   The previous widget or <code>null</code> if there is no
    *           preceding widget.
    */
   public GenericWidget<?> getPrevChild(GenericWidget<?> child);
}