/* 
** Module   : RadioSetConfig.java
** Abstract : stores radio set specific widget state
**
** Copyright (c) 2005-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- --JPRM-- ----------------Description-----------------
** 001 EVL 20050714   @21692 Creation based on general concept. Necessary
**                           modification to get compiled.
** 002 EVL 20050724   @21804 Adding the number of initially checked radio
**                           button
** 003 SIY 20050831   @22511 Refactoring (see @@22394-22409)
** 004 SIY 20050908   @22647 Fixed setting of width.
** 005 SIY 20050909   @22669 Fixed double initialization of items.
** 006 NVS 20051116   @23344 Class access changed to public. 
** 007 SIY 20060209   @24448 Major refactoring.
** 008 GES 20060803   @28411 Made externalizable for better performance.
**                           Many code standard cleanups.
** 009 NVS 20061214   @31693 Added the getIndex() method.
** 010 SIY 20100619          Switching to own implementation of character UI.
** 011 SIY 20100804          Changes in imports, some refactorings.
** 012 SIY 20111101          Moved from CHUI package. Minor 
**                           cleanups/refactorings in preparation to GUI client.
** 013 SIY 20111202          Updated imports.
** 014 SIY 20111121          Cleaned up imports. 
** 015 HC  20140911          GUI coordinate support - data types refactoring.
** 016 CA  20140926          Added shared widget configuration support, to allow GUI/ChUI
**                           concrete implementation for the same widget ID.  Refactored the
**                           widget configuration classes: all fields were made public; these
**                           classes need to be as dumb as possible, all logic related to
**                           setting/getting a certain field should be at the widget or at the
**                           caller. Refs #2254
** 017 HC  20141021          Implemented window move/position support.
** 018 CA  20150105          Changes for SIDE-LABEL-HANDLE runtime support.
** 019 HC  20150323          Removed business logic from config classes.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.ui;

import java.io.*;

/**
 * Stores radio set specific state.
 */
public class RadioSetConfig
extends ControlSetConfig
{
   /** Initially checked radio button's number. */
   public int initial = 0;

   /** The Progress EXPAND attribute. */
   public boolean expand = false;

   /** The radio-set orientation. */
   public boolean horizontal = false;
   
   /**
    * Default constructor (only used in deserialization).
    */
   public RadioSetConfig()
   {
      possibleSideLabel = true;
   }

   /**
    * Create a new config and associate it with the given widget.
    * 
    * @param    id
    *           The ID of the widget to which this configuration belongs.
    */
   protected RadioSetConfig(int id)
   {
      super(id);

      possibleSideLabel = true;
   }

   /**
    * Create a new config and associate it with the given widget.
    * 
    * @param    id
    *           The ID of the widget to which this configuration belongs.
    */
   protected RadioSetConfig(WidgetId id)
   {
      super(id);

      possibleSideLabel = true;
   }

   /**
    * Set new values from the provided configuration instance.
    * 
    * @param    config
    *           The instance from which to take values.
    */
   @Override
   public void applyConfig(WidgetConfig config)
   {
      super.applyConfig(config);
      
      RadioSetConfig cfg = (RadioSetConfig) config;
      
      initial = cfg.initial;
      expand = cfg.expand;
      horizontal = cfg.horizontal;
   }

   /**
    * Replacement for the default object reading method. The latest
    * state is read from the input source.  The parent's state is
    * read first.
    * 
    * @param    in
    *           The input source from which fields will be restored.
    *
    * @throws   IOException
    *           In case of I/O errors.
    * @throws   ClassNotFoundException
    *           If payload can't be instantiated.
    */
   @Override
   public void readExternal(ObjectInput in)
   throws IOException,
          ClassNotFoundException
   {
      super.readExternal(in);
      initial    = in.readInt();
      expand     = in.readBoolean();
      horizontal = in.readBoolean();
   }

   /**
    * Replacement for the default object writing method. The latest
    * state is written to the output destination.  The parent's state is
    * written first.
    * 
    * @param    out
    *           The output destination to which fields will be saved.
    *
    * @throws   IOException
    *           In case of I/O errors.
    */
   @Override
   public void writeExternal(ObjectOutput out)
   throws IOException
   {
      super.writeExternal(out);
      out.writeInt(initial);
      out.writeBoolean(expand);
      out.writeBoolean(horizontal);
   }
}