/*
** Module   : AlertBox.java
** Abstract : AlertBox implementation
**
** Copyright (c) 2005-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- --JPRM-- ----------------------------Description-----------------------------
** 001 EVL 20050427   @20916 Creation based on CHARVA JDialog class.
**                           Necessary modification to get compiled.
** 002 EVL 20050428   @20939 Adding return value feature.
** 003 EVL 20050531   @21370 Changing the inner variable names to sync
**                           with the rest of the project and preparing
**                           to the javadoc processing.
** 004 EVL 20050607   @21435 Syntax for comments checking.
** 005 EVL 20050611   @21478 Changing in button creation.
** 006 SIY 20050831   @22484 Refactoring (see @@22394-22409).
** 007 SIY 20050907   @22619 Fixed use of @inheritDoc tag.
** 008 SIY 20050922   @22817 Using ColorTable colors for drawing.
** 009 SIY 20060215   @24594 Significant rewrite to make it working 
**                           properly.
** 010 SIY 20060226   @24764 Fixed initial selection display in some 
**                           cases.
** 011 NVS 20060301   @24861 KeyInput is injected into the event queue
**                           instead of KeyEvent.
** 012 NVS 20060630   @27765 Replaced requestFocus() calls in nextFocus()
**                           and previousFocus() methods with tryFocus-
**                           Change() method.
** 013 NVS 20060705   @27811 Backed out the change #012 since the callers
**                           of nextFocus() and previousFocus() methods
**                           now take care of the Progress semantics.
** 014 GES 20060808   @28486 Fixed 3 button Progress semantic of "cancel"
**                           yielding unknown value as a result. Properly
**                           handle the initial value as a logical and
**                           use it to set the default button. Fixed the
**                           contents list processing to properly 
**                           concatenate sequential text elements and to
**                           insert an extra space whenever a text element
**                           is not the last element in the array.
** 015 GES 20060825   @28915 Avoid call to server at result assignment.
**                           Externalized the exit processing.
** 016 NVS 20061108   @31028 Implemented proper handling of embedded new
**                           line characters in message box strings.
** 017 ECF 20061207   @31599 Removed call to ComponentConfig.setSelected()
**                           method. This method has been removed, since
**                           the selected flag is now managed only on the
**                           server.
** 018 SIY 20070524   @33746 Fixed focus handling.
** 019 SIY 20071025   @35538 Fixed width calculation.
**                           Ref: #35535.
** 020 SIY 20071106   @35717 Fixed width calculation regression from #019.
**                           Ref: #35686.
** 021 SIY 20071206   @36230 Added support for long messages which require 
**                           splitting.
**                           Ref: #36229
** 022 SIY 20080701   @39014 Fixed handling of SKIP, including case of 
**                           SKIP-text-SKIP sequence, which affects 
**                           alert box width. 
**                           Ref: #39013
** 023 CA  20081114   @40450 Changes due to new clipping implementation.
** 024 SIY 20090206   @41263 Reworked size calculation in order to match  
**                           current knowledge about relevant 4GL behavior.
** 025 SIY 20090206   @41290 Minor change in size calculation necessary  
**                           for proper handling of some edge cases.
** 026 SIY 20090206   @41305 Updated #25 to handle properly all sizes.  
** 027 SIY 20090223   @41355 Fixed handling of string wrapping in some 
**                           edge cases.
** 028 SIY 20090508   @42119 Fixed width calculation in one edge case. 
** 029 CA  20091207   @44489 Fixed message width calculation in case when a CR
**                           character is encountered. Fixed title display 
**                           when its size is greater then the dialog size.
**                           TODO: Conversion issues related remain, related 
**                           to complex TITLE expressions; also, there is one 
**                           edge case when the message is a "c SKIP c"
**                           expression and c's length is 20.
** 030 SIY 20100619          Switching to own implementation of character UI.
** 031 SIY 20100804          Changes in imports, some refactorings.
** 032 SIY 20100822          Refactored AttributeContainer support.
** 033 SIY 20101122          Switching from KeyEvent to KeyInput.
** 034 SIY 20110112          Changes related to new color handling scheme.
** 035 SIY 20111202          Refactored and moved from CHUI package.
** 036 SIY 20111221          Switched to widget factory.
** 037 GES 20140627          Changed color attribute method names to match naming standards.
** 038 CA  20140805          Added generics to force the abstract widget classes to not use
**                           implementation-specific code: the abstract widgets will have access 
**                           only to the base APIs provided by the OutputManager.
** 039 HC  20140911          GUI coordinate support - data types refactoring.
** 040 CA  20140926          Added shared widget configuration support, to allow GUI/ChUI
**                           concrete implementation for the same widget ID.  Refactored the
**                           widget configuration classes: all fields were made public; these
**                           classes need to be as dumb as possible, all logic related to
**                           setting/getting a certain field should be at the widget or at the
**                           caller. Refs #2254
** 041 HC  20141021          Implemented window move/position support.
** 042 CA  20150323          Pass the frame ID when creating a label (to properly resolve the font
**                           in GUI).
** 043 HC  20150612          Implemented GUI ALERT-BOX.
** 044 IAS 20160322          DEBUG-ALERT support.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.ui.client;

import com.goldencode.p2j.ui.client.event.KeyInput;
import com.goldencode.p2j.ui.client.widget.*;
import com.goldencode.p2j.util.logical;

/**
 * AlertBox provides exactly the same functionality
 * as the ALERT-BOX widget in the Progress language. This is the alternative
 * way to represent the message statement with a simple dialog box with one or
 * more buttons and the possible return value.
 * <p>
 * The interface provides a common API to the client widgets implementing the
 * alert box functionality. 
 */
public interface AlertBox<O extends OutputManager<?>>
extends Container<O>
{
   /**
    * Find the button that matches a passed key event.
    * 
    * @param    keyEvent
    *           Event to check.
    * 
    * @return   <code>true</code> if matching button is found and event is
    *           processed.
    */
   boolean findButton(KeyInput keyEvent);

   /**
    * Set the result code, hide the dialog and flag input as complete.
    *
    * @param    rc
    *           The result code for the alert box.
    */
   void finished(logical rc);
   
   /**
    * Get status of the input processing.
    * 
    * @return  current status of the input processing.
    */
   boolean inputDone();
   
   /**
    * Helper method to get the edited result when finished.
    * 
    * @return   The result of the editing operation which was set depending
    *           on the pressed button.
    */
   logical getResult();
   
   /**
    * Create window for the stack trace
    * 
    * @param title
    *        window title
    * 
    * @return window for the stack trace
    */
   TitledWindow createStackTraceWindow(String title);
   
   /**
    * Create effectively invisible widget (the gap) for the stack trace dialog
    * to avoid explicit coordinates' usage 
    * TODO: at this moment the layout manager doesn't support this approach.
    *   
    * @param registry 
    *        widget registry
    * 
    * @return the gap widget
    */
   Widget<O> createGap(WidgetRegistry<O> registry);
}