/*
** Module   : ComboBox.java
** Abstract : combo-box implementation
**
** Copyright (c) 2005-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- --JPRM-- ----------------------------Description-----------------------------
** 001 EVL 20050211   @19831 Creation based on CHARVA JComboBox class.
**                           Necessary modification to get compiled
** 002 EVL 20050406   @20655 Adding explicit declaration of CHARVA's Frame
**                           class to avoid interference with our CHUI
**                           Frame for the inner class PopUp. Changing
**                           import charva.awt.* statement to the
**                           importing only classes we really need.
** 003 EVL 20050421   @20842 Adding implementation of ItemListener
**                           interface which just repaint the parent
**                           which contains this widget
** 004 EVL 20050522   @21256 Adding Progress specific methods
**                           implementation
** 005 EVL 20050603   @21405 Changing the inner variable names to sync
**                           with the rest of the project. The reference
**                           to the CHARVA's Frame class was moved to the
**                           usage in PopUp inner class instead of import
**                           statement
** 006 EVL 20050603   @21406 Adding the Progress specific methods to work
**                           with Progress combo-box attributes
** 007 EVL 20050607   @21435 Syntax for comments checking
** 008 EVL 20050607   @21438 Removing the visible member. It is completely
**                           duplicated in CHARVA's Component superclass
**                           so we do not need to double this work. Also
**                           the methods to work with the HIDDEN attribute
**                           are modified to reflect this change
** 009 EVL 20050723   @21805 Properly handling of the change the
**                           LIST_ITEMS attribute. Default delimiter is 
**                           comma.
** 010 EVL 20050822   @22199 Using the P2JFrame/Container interfaces to
**                           detect the class.
** 011 SIY 20050831   @22490 Refactoring (see @@22394-22409) 
** 012 SIY 20050907   @22626 Fixed use of @inheritDoc tag in javadoc
** 013 EVL 20050912   @22678 Adding Validateable interface to this widget.
** 014 EVL 20050913   @22696 Adding focus listener interface.
** 015 EVL 20050916   @22737 Returned value class is changed to be the
**                           BaseDataType.
** 016 EVL 20050919   @22748 Adding going down through the classes via key
**                           processing events.
** 017 EVL 20050920   @22760 Adding the implementation of the new method
**                           isModified() for the Validateable interface.
** 018 EVL 20050921   @22765 Adding method to be able to undo the widget's
**                           value.
** 019 SIY 20050920   @22830 Added support for SENSITIVE and SELECTED 
**                           attributes.
** 020 SIY 20051129   @23562 Moved item list member to config.
** 021 GES 20060114   @24202 Implemented DataContainer and changes to
**                           Validatable.
** 022 SIY 20060131   @24203 Use ThinClient special method instead of
**                           direct calls to Window.message().
** 023 SIY 20060209   @24484 Major refactoring.
** 024 SIY 20060209   @24495 Fixed display value formatting.
** 025 SIY 20060210   @24522 Added getSelection() method, added 
**                           ControlSetComponent interface.
** 026 SIY 20060222   @24632 Added necessary methods to support changes in
**                           DataContainer interface.
** 027 NVS 20060227   @24864 Key actions KA_* are used instead of virtual
**                           keys VK_* for all editing operations.
** 028 SIY 20060322   @25217 Added setNoLabels() to match new 
**                           LabeledWidget interface.
** 029 SIY 20060515   @26145 Encapsulated access to help field.
** 030 SIY 20060515   @26147 Added getColumnLabelText().
** 031 NVS 20060703   @27767 Reviewed the uses of requestFocus(). They are
**                           safe.
** 032 NVS 20060703   @27800 getParent.nextFocus() replaced with
**                           ThinClient.nextFocus() call to maintain
**                           Progress semantics. Similar replacement has
**                           been done to getParent.previousFocus().
** 033 GES 20060803   @28395 Major cleanup and refactoring.
** 034 EVL 20060809   @28497 Fix for DropDown line mismatch.
** 035 EVL 20060809   @28513 Adding drawCaret() method to force cursor
**                           setting to the first char of the widget.
** 036 SIY 20060824   @28846 Removed getP2JType() method.
** 037 GES 20061107   @31000 Many code cleanups and fixes for key 
**                           processing, drop-down list width and getting
**                           the correct initial value.
** 038 GES 20061108   @31022 Fix to constrain drop-down box size to a
**                           maximum height that is the same as the 
**                           distance from the top of the frame (which
**                           contains the combo-box) to the bottom of the
**                           screen.
** 039 GES 20061108   @31024 Default to disabled during the constructor.
** 040 GES 20061109   @31048 Fix for the main field drawing width and the 
**                           drop-down width calculation.
** 041 NVS 20061204   @31554 Switched to using a private widgetFocused
**                           flag instead of config.selected
** 042 NVS 20061206   @31593 Fixed draw() method for uninitialized widget
**                           and added alignment of the shorter strings.
** 043 SIY 20061211   @31671 Fixed size calculation (affected layout 
**                           processing), some refactoring and cleanups.
** 044 SIY 20072011   @33156 Force setting of SCREEN-VALUE on ENTRY event.
** 045 NVS 20070426   @33250 Enabled state is checked instead of SENSITIVE
**                           attribute.
** 046 SIY 20070614   @34037 Fixed focus handling.
** 047 SIY 20070615   @34057 Fixed repainting after exit from drop down.
** 048 SIY 20070730   @34731 Fixed handling of RETURN. Ref: #34730.
** 049 SIY 20070802   @34761 Fixed drawing regression. Ref: #34760.
** 050 NVS 20070802   @34780 Removed obsoleted comments and javadoc.
** 051 SIY 20070807   @34808 Added isColumnFixed() method.
** 052 SVL 20071221   @36529 Label is shown/hidden together with the
**                           widget. 
** 053 SIY 20080320   @37606 Fixed handling of the SCREEN-VALUE and data
**                           value.
** 054 NVS 20080320   @37620 Fixed drawing of combo boxes in focus when
**                           they are disabled.
** 055 NVS 20080401   @37773 Added isFocusable() method implementation.
** 056 NVS 20080501   @38190 Fixed drop-down hiding related repainting
**                           issue. Fixed empty list handling and painting
**                           issues.
** 057 SIY 20080505   @38207 Fixed alignment of numeric values.
** 058 SIY 20080709   @39083 Removed processing of RETURN as "AUTO-GO" 
**                           because this contradicts observed 4GL 
**                           behavior.
** 059 SIY 20081107   @40370 Mark VALUE-CHANGED as special to simplify 
**                           separation of internally generated events 
**                           and application generated ones.
** 060 CA  20090326   @41672 Added DropDown.getComboBox - it will return
**                           the widget to which the drop-down is attached.
** 061 GES 20090424   @41960 Import change.
** 062 SIY 20100619          Switching to own implementation of character UI.
** 063 SIY 20100804          Changes in imports, some refactorings.
** 064 SIY 20100822          Refactored AttributeContainer support.
** 065 SIY 20101122          Reduced class hierarchy. Switching from KeyEvent
**                           to KeyInput.
** 066 SIY 20110112          Changes related to new color handling scheme.
** 067 SIY 20111101          Updated imports, minor cleanups/refactorings.
** 068 SIY 20111202          Refactored and moved from CHUI package.
** 069 SIY 20111221          Minor cleanups.
** 070 EVL 20140528          Adding implementation stubs for two new methods.
** 071 GES 20140627          Changed color attribute method names to match naming standards.
** 072 CA  20140805          Added generics to force the abstract widget classes to not use
**                           implementation-specific code: the abstract widgets will have access
**                           only to the base APIs provided by the OutputManager.
** 073 HC  20140911          GUI coordinate support - data types refactoring.
** 074 CA  20140926          Added shared widget configuration support, to allow GUI/ChUI
**                           concrete implementation for the same widget ID.  Refactored the
**                           widget configuration classes: all fields were made public; these
**                           classes need to be as dumb as possible, all logic related to
**                           setting/getting a certain field should be at the widget or at the
**                           caller. Refs #2254
** 075 HC  20141021          Implemented window move/position support.
** 076 CA  20150105          Changes for SIDE-LABEL-HANDLE runtime support.
** 077 CA  20150114          Fixed support for changing the ROW/COLUMN attrs from server-side, 
**                           for a side-label.
** 078 HC  20150128          Added support for frame widget position changes induced by
**                           setting ROW/COL/X/Y attributes.
** 079 CA  20150202          Side-labels needs to track the HIDDEN state, too.
** 080 HC  20150214          Improved positioning of side labels.
** 081 CA  20150212          Allow events to be captured and processed by the widgets without
**                           server-side or main event-loop notification.  This fixes problems in
**                           the EDITING and WAIT-FOR event loops, for i.e. COMBO-BOX and EDITOR
**                           widgets.
** 082 CA  20150223          Allow the label to be removed.
** 083 IAS 20150224          Fixed logic for duplicated and empty/unknown labels
** 084 IAS 20150305          Changed 'pairs' type to Boolean
** 085 CA  20150308          Added isSelected API.
** 086 HC  20150323          Removed business logic from config classes.
** 087 EVL 20150428          Changing drop-down creation approach to be UI neutral.  Making
**                           activate() as protected to be called from GUI implementation too.
** 088 IAS 20150604          Added INNER-LINES and SELECTED-TEXT attributes' support
** 089 EVL 20150514          Exit event processing for event loop is a bit different in GUI, put
**                           the UI spectific calls in respective implementation.
** 090 CA  20150722          Added postprocessConfig - required additional processing, as layout  
**                           is performed only on the first frame definition push.
** 091 EVL 20150720          Changing approach for selected flag modification.  As soon as the
**                           value is changed the selected flag need to be true even before the
**                           drop-down become closed.
** 092 IAS 20150801          TAB-STOP attribute support
** 093 HC  20150831          Widgets are automatically detached from their parents when destroyed.
** 094 CA  20150910          Move the drop-down to top, to allow correct mouse source resolution.
**                           The inner WAIT-FOR which handles the drop-down events must 
**                           save/restore the invalidate, as this might be called from an existing
**                           event/draw processing loop. 
** 095 IAS 20150924          Added AUTO-RESIZE attribute support
** 096 CA  20150925          Made exitDropDown() public, it needs to be called if the drop-down
**                           is notified that is losing focus. Fixed multi-window drawing issues 
**                           caused by concurrent drawing access to the GuiDriver: all access to
**                           the driver is done exclusively, each thread taking and releasing
**                           ownership for the duration of the drawing operation or other GUI API
**                           invocation which requires access to the underlying physical window.
** 097 HC  20151013          Changes to extend GUI multi-window focus management and ACTIVE-
**                           WINDOW system handle processing to modal windows.
** 098 HC  20151103          Improved GUI COMBO-BOX layout and fixes.
**     EVL 20151118          Making the getDataEntryReturn() protected.  In GUI it must always
**                           return false.  Making selected member as protected to be able to
**                           change it in GUI version of the combo-box.
** 099 EVL 20151217          Drop-down approach rework.  Separation focus requesting calls for
**                           different UI types.
** 100 EVL 20160119          Adding focus move postsprocessing.
** 101 IAS 20160130          LOAD-MOUSE-POINTER support.
** 102 IAS 20160303          Fixed RETURN processing.
** 103 HC  20160406          Fixed COMBO-BOX drop-down resource cleanup.
** 104 HC  20160413          Comment fix.
** 105 HC  20160511          Fixed COMBO-BOX focusing issues and related changes.
** 106 SBI 20160708          Changed to move the code that resets the active combo box reference
**                           and the locked pointer to the concrete GUI subclass, ComboBoxGuiImpl.
** 107 CA  20160930          Fixed setValue to properly select and display the value set via the
**                           SCREEN-BUFFER attribute.
** 108 CA  20161014          setValue() compares the given value and the item's value using their
**                           toStringMessage() representation, as the item's value and the new
**                           value may be of different data types.
** 109 SBI 20161004          Added new method isFormatHonored() to get config.honorFormat.
** 110 HC  20161018          Added an initial support for rendering of mnemonic labels to all
**                           labeled widgets.
** 111 CA  20161123          Convert the item's value to the combo-box datatype in refreshItems(),
**                           if the type differs.
** 112 OM  20170320          ThinClient.setHelp() signature change. Made isFormatHonored() public.
** 113 CA  20170404          exitDropDown must set focus back to the combo-box.
** 114 OM  20170511          Added support for EDIT-CLEAR method.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.ui.client;

import java.util.*;

import com.goldencode.p2j.ui.*;
import com.goldencode.p2j.ui.ComboBoxConfig.Mode;
import com.goldencode.p2j.ui.chui.ThinClient;
import com.goldencode.p2j.ui.client.event.*;
import com.goldencode.p2j.ui.client.event.listener.*;
import com.goldencode.p2j.ui.client.format.*;
import com.goldencode.p2j.ui.client.model.*;
import com.goldencode.p2j.ui.client.widget.*;
import com.goldencode.p2j.util.*;

/**
 * Allows the user to select an item from a drop-down list of choices.
 * <p>
 * When the combo-box is in a non-dropped-down state, it looks like a Fill-in
 * with a [V] character on the right. To make the drop-down menu appear, the
 * user positions the cursor over the combo-box and presses DOWN key.
 * <p>
 * When the user selects an item in the drop-down menu (by positioning the
 * cursor on it and pressing ENTER), the drop-down menu disappears
 * and only the selected item is shown.  Any use of the <code>END-ERROR</code>
 * key will cause the drop-down to be closed with no change in the selected
 * value.  Any alphabetic or numeric character will select the next item
 * that starts with that character.  All other keys are ignored.
 * <p>
 * Note that this class provides a Progress combo-box that matches the
 * <code>DROP-DOWN-LIST</code> mode.  The <code>SIMPLE</code> and
 * <code>DROP-DOWN</code> modes are not supported.
 */
public abstract class ComboBox<O extends OutputManager<?>>
extends AbstractWidget<O>
implements WidgetWithConfig<ComboBoxConfig>,
           SelectionListener,
           FocusListener,
           ItemListener,
           DataContainer,
           LabeledWidget<O>,
           ControlSetComponent,
           MnemonicTrait<O>,
           TextSelection
{
   /** Contains our attributes. */
   protected ComboBoxConfig config = null;

   /** The drop-down list object to select one item from the list. */
   protected DropDown<O> dropDown;

   /** The maximum size of the data portion of the main combo-box. */
   protected double limit = 0;

   /** Track if a VALUE-CHANGED event needs to be raised. */
   protected boolean valueChanged;

   /** A helper field for drop-down related event handling. */
   protected boolean endConditionRaised;
   
   /** Component label. */
   private Label<O> label = null;
   
   /** Maximal label length */
   private int maxlen = 0;

   /** Original value assigned to the widget. */
   private BaseDataType initValue = null;

   /** Track selection. */
   protected boolean selected;

   /** Combo-box model. */
   private ComboBoxModel<String> model = new ComboBoxModel<String>();

   /** Mnemonic information */
   private MnemonicInfo mnemonic = null;

   /** Column mnemonic information */
   private MnemonicInfo columnMnemonic = null;

   /**
    * Special constructor used to restore the component using a given
    * configuration.
     * <p>
    * Call {@link #initialize} to finish the widget initialization.
   * 
    * @param    id
    *           The widget's ID.
    * @param    cfg
    *           Configuration data.
    */
   public ComboBox(WidgetId id, ComboBoxConfig cfg)
   {
   }
   
   /**
    * Replacement API for the logic which should exist in the widget's c'tor, but the logic 
    * can't be executed as it depends on the widget being registered into the official registry.
    * <p>
    * This API is called after the widget is created and added to the registry.
    * 
    * @param    id
    *           The widget's ID.
    * @param    cfg
    *           The config used to initialize this widget.
    */
   @Override
   public void initialize(WidgetId id, ComboBoxConfig cfg)
   {
      disable();
      addFocusListener(this);
      
      config = ConfigManager.getInstance().resolveWidgetConfig(id, cfg);
      ConfigManager.getInstance().replaceConfig(config, cfg);
   }

   /**
    * Draws the combo box as a list item.
    */
   @Override
   public abstract void draw();
   
   /**
    * Sets position of terminal's cursor to the first char of the combo-box.
    */
   @Override
   public abstract void drawCaret();
   
   /**
    * Check if the ComboBox mode is SIMPLE
    * 
    * @return <code>true</code> if the ComboBox mode is SIMPLE
    */
   public boolean isSimpleMode()
   {
      return config.mode == Mode.SIMPLE;
   }
   
   /**
    * Format source string using current settings and make it suitable for
    * display.
    * <p>
    * There are 2 formatting modes:
    * <p>
    * <pre>
    * 1. Item List - this is used when the list being represented is
    *                a simple list of values rather than a set of
    *                label + value pairs, in this case the explicit format
    *                string (or the default format) will be used to format
    *                the label text for display in both the drop-down box
    *                and in the main field
    * 2. Item Pair - used when the list is representing label/value pairs,
    *                in this case the entire text of the label will be
    *                returned (padded on the right to reach the same
    *                length as the largest of the labels)
    * </pre>
    * <p>
    * If the <code>honor format</code> flag is <code>true</code> then the 
    * widget is in "item list" mode.
    * 
    * @param    selectedItem
    *           The source string to format.
    * 
    * @return   The formatted value suitable for display.
    */
   String formatValue(String selectedItem)
   {
      String display = "?";
      
      if (isFormatHonored())
      {
         // static item-list mode
         try
         {
            BaseDataType var = DisplayFormat.instanceOfType(config.dataType);
            DisplayFormat format;
            
            if (config.format != null)
            {
               format = DisplayFormatFactory.makeFormat(var, config.format);
            }
            else
            {
               format = DisplayFormatFactory.makeDefaultFormat(var);
            }
            
            boolean right = format.isRightPositioning();
            
            // ignore alignment if we're in item-pair mode because 
            // label is always aligned left
            if (Boolean.TRUE.equals(config.pairs))
            {
               right = false;
            }
            else  
            {
               // but numeric format are always aligned right
               if (format instanceof NumberFormat)
               {
                  right = true;
               }
            }
            
            int maxWidth = getMaxLabelLen();
            int fmtWidth = format.getScreenWidth();
            int width    = (maxWidth > fmtWidth) ? maxWidth : fmtWidth; 
            
            display = DisplayFormat.formatScreenValue(selectedItem,
                                                      width,
                                                      0,
                                                      false,
                                                      right);
         }
         catch (Exception e) 
         {
            // do nothing and return unknown
         }
      }
      else
      {
         // static item-pair mode
         
         if (selectedItem == null)
            selectedItem = "";
         
         StringBuffer buf = new StringBuffer(selectedItem);

         int maxlen = getMaxLabelLen();
         
         for (int i = 0; i < (maxlen - selectedItem.length()); i++)
            buf.append(' ');

         display = buf.toString();
      }
      
      return display;
   }

   /**
    * Provide access to the attribute of the widget.
    * 
    * @return   The reference to the attribute container.
    */
   @Override
   public ComboBoxConfig config()
   {
      return config;
   }
   
   /**
    * The method is called after the configuration associated with 
    * the implementor changes. The parameter points to a reference
    * representing the original configuration state. This reference
    * can be used to detect what configuration fields changed and so
    * optimize any processing related to the configuration change.
    * 
    * @param   beforeUpdate
    *          Config reference capturing the config state before 
    *          the owner's config was modified.
    */
   @Override
   public void afterConfigUpdate(ComboBoxConfig beforeUpdate)
   {
      super.afterConfigUpdateBase(beforeUpdate);

      updateMnemonic(beforeUpdate.clabel, config.clabel, true);
      updateMnemonic(beforeUpdate.label, config.label, false);
      refreshItems();
   }

   /**
    * Notifies this component that the state of the combo-box has changed.
    * 
    * @param    itemEvent
    *           The item related event to handle.
    */
   @Override
   public void onSelect(ItemEvent itemEvent)
   {
      if (itemEvent == null)
         return;
      
      parent().repaint();
   }
   
   /**
    * Returns the index of the item with the specified text.
    * 
    * @param    item
    *           The string entry whose index must be returned.
    * 
    * @return   The index of the given string value within the list.
    */
   public int lookup(String item)
   {
      return model().indexOf(item);
   }

   /**
    * Process key events that have been generated by this component.
    * 
    * @param    keyEvent
    *           The key event to be handled.
    */
   @Override
   public void processKeyEvent(KeyInput keyEvent)
   {
      int key = keyEvent.actionCode();

      if (key == Keyboard.KA_TAB)
      {
         // the following call must be conditional
         ThinClient.nextTabStop(this);
         return;
      }
      else if (key == Keyboard.KA_BACK_TAB)
      {
         // the following call must be conditional
         ThinClient.previousTabStop(this);
         return;
      }
      else if (key == Keyboard.KA_RETURN)
      {
         return;
      }
      else if (key == Keyboard.SE_ENTRY)
      {
         // force setting of SCREEN-VALUE on entry
         ConfigHelper.setModified(config, true);
      }
      else
         processKeyListeners(keyEvent);
   }

   /**
    * Obtain the input focus.
    */
   @Override
   public void requestFocus()
   {
      if (!isEnabled())
         return;
      
      // generate the FOCUS_GAINED event
      super.requestFocus();
   }
   
   /**
    * Return <code>true</code> if a drop-down value has been explicitly selected.
    * 
    * @return   The state of the {@link #selected} flag.
    */
   public boolean isSelected()
   {
      return selected;
   }
   
   /**
    * Gets the currently selected item.
    *
    * @return   The character representing the current value of the combo-box.
    */
   @Override
   public BaseDataType getValue()
   {
      if (!selected)
         return initValue;
      
      return getScreenValue();
   }
   
   /**
    * Sets the instance representing the current value.
    *
    * @param    value
    *           The value representing the current widget value.
    */
   @Override
   @SuppressWarnings("unchecked")
   public void setValue(BaseDataType value)
   {
      ControlSetItem[] items = config.items;

      initValue = null;
      selected  = false;
      
      ComboBoxModel<String> model = model();
      
      for (int i = 0; i < items.length; i++)
      {
         if (items[i].getValue().toStringMessage().compareToIgnoreCase(value.toStringMessage()) == 0)
         {
            initValue = items[i].getValue();

            String text = items[i].getLabel().toStringMessage();
            
            model.select(formatValue(text));
            model.setSelectedIndex(i);
            
            break;
         }
      }
   }

   /**
    * Get text representation of the currently selected value.
    * 
    * @return   The screen representation of the value or <code>null</code>
    *           if no value is selected.
    */
   @Override
   public String getText()
   {
      String selected = model().selected();
      
      ControlSetItem[] items = config.items;
      
      int ndx = model().indexOf(selected);
      
      if (ndx < 0 || ndx >= items.length)
         return "";
      
      return items[ndx].getLabel().toStringMessage();
   }
   
   /**
    * Getting the modification state of the widget.
    * 
    * @return  <code>true</code> if the widget value was modified,
    *          <code>false</code> otherwise.
    */
   @Override
   public boolean isModified()
   {
      return config.modified;
   }

   /**
    * This method is invoked when selection is changed.
    */
   @Override
   public void changed()
   {
      this.valueChanged = true;
      // we made the selection, set the flag accordingly
      selected = true;
   }

   /**
    * Value changed handler.
    */
   public void valueChangedImpl()
   {
      // special case of empty list
      ControlSetItem[] items = config.items;
      
      int idx = dropDown.getSelectedIndex();
      model.setSelectedIndex(idx);
      config.selectedIndex = idx;
      
      if (0 <= idx && idx < items.length)
      {
         character label = items[idx].getLabel();
         model.select(label == null ? null : label.toStringMessage());
         model.setSelectedValue(items[idx].getValue());
         model.setSelectedId(items[idx].getItemId());
      }

      // post the value changed event 
      valueChangedCommon();
      
      repaint();
      requestSync();
   }
   
   /**
    * Get widget dimension.
    *
    * @return  widget dimension.
    */
   @Override
   public Dimension dimension()
   {
      return new Dimension(width(), height());
   }
   
   /**
    * @see com.goldencode.p2j.ui.client.widget.Widget#width()
    */
   @Override
   public double width()
   {
      double w = limit + 4;
      
      if (config.widthChars < w)
      {
         return w;
      }
      else
      {
         return config.widthChars;
      }
   }

   /**
    * @see com.goldencode.p2j.ui.client.widget.Widget#height()
    */
   @Override
   public double height()
   {
      return 1;
   }
   
   /**
    * Properly dereference the text of an object, handling Progress data type
    * formatting if needed.
    *
    * @param    item
    *           The instance to be rendered as text.
    *
    * @return   The full text of the item.
    */
   public String getItemText(Object item)
   {
      String txt = null;
      
      if (item != null)
      {
         if (item instanceof BaseDataType)
         {
            BaseDataType bdt = (BaseDataType) item;
            txt = bdt.toStringMessage();
         }
         else
         {
            txt = item.toString();
         }
      }
      
      return txt;
   }
   
   /**
    * Refresh items in the combo-box after assignment.
    */
   @Override
   public void refreshItems()
   {
      // clean up old content if it exists
      if (model().size() > 0)
         model().clear();
      
      ControlSetItem[] items = config.items;

      maxlen = 0;
      
      String cbDataType = config.dataType;
      
      for (ControlSetItem item : items)
      {
         BaseDataType itemValue = item.getValue();
         if (!itemValue.getTypeName().equals(cbDataType))
         {
            item.setValue(DisplayFormat.instanceOfType(cbDataType, itemValue.toStringMessage()));
         }
         
         String text  = formatValue(item.getLabel().toStringMessage());
         
         if (text.length() > maxlen)
            maxlen = text.length();
         
         model().add(text);
      }

      limit = calcLimit(true);

      model().setSelectedIndex(-1); // reset selected index
      model().setSelection(items);

      int idx = model().getSelectedIndex();
      config.selectedIndex = idx;
      if (idx > 0) 
      {
         character label = items[idx].getLabel();
         model().select(label == null ? null : label.toStringMessage().trim());
      }

      repaint();
   }

   /**
    * Obtain the list of indexes for all selected items.
    * <p>
    * Multiple selection mode is not supported in this class, so the returned
    * array will always have a single element. 
    * 
    * @return   The single element array of the 0-based index of the selected
    *           item.  The element's value may be <code>null</code> but a
    *           valid array will always be returned.
    */
   @Override
   public int[] getSelection()
   {
      String selected = model().selected();
      
      if (selected == null)
         return new int[0];
      
      return new int[] { model().indexOf(selected) };
   }
   
   /**
    * This is called when the combo-box gains the input focus. When the help string is defined for
    * the combo-box will be displayed in the status line.
    *
    * @param    event
    *           The focus event to be processed.
    */
   @Override
   public void onFocusGained(FocusEvent event)
   {
      ThinClient.setHelp(config.help, this);
   }
   
   /**
    * This is called when the combo-box loses the input focus.
    *
    * @param    focusEvent
    *           The focus event to be processed.
    */
   @Override
   public void onFocusLost(FocusEvent focusEvent)
   {
   }
   
   /**
    * Return the reference to the instance of the <code>Label</code> attached
    * to the widget.
    * 
    * @return   Reference to <code>Label</code> instance.
    */
   @Override
   public Label<O> getLabelInstance()
   {
      return label;
   }
   
   /**
    * Assign a new instance of <code>Label</code> to the widget.
    * 
    * @param    label
    *           Reference to the new <code>Label</code> instance.
    */
   @Override
   public void setLabelInstance(Label<O> label)
   {
      if (label != this.label && this.label != null)
      {
         this.label.destroy();
      }
      
      if (label == null)
      {
         this.label = null;
         return;
      }

      int frameId = ThinClient.frameIdFromWidgetId(this.config.id.asInt());
      
      // link the label to the frame of this widget
      label.linkTo(frameId, config().sideLabelId);

      this.label = label;
      this.label.setVisible(isVisible());
      this.label.setHidden(hidden());
   }
   
   /**
    * Return text of the label.
    * 
    * @return   The text of the label.
    */
   @Override
   public String getLabelText()
   {
      return getMnemonicLabel(false).orElse(config.label);
   }

   /**
    * Return text of the column label.
    * 
    * @return   The text of the column label.
    */
   @Override
   public String getColumnLabelText()
   {
      return getMnemonicLabel(true).orElse(config.clabel);
   }
   
   /**
    * Return TO column.
    * 
    * @return   The column assigned via TO option.
    */
   @Override
   public double getTo()
   {
      return ConfigHelper.getTo(config);
   }

   /**
    * Get current state of the flag which is forces presence of the label
    * regardless of the frame's NO-LABELS option.
    * 
    * @return   The current state of the flag.
    */
   @Override
   public boolean isForceLabel()
   {
      return config.forceLabel;
   }

   /**
    * Get current state of the NO-LABELS flag.
    * 
    * @return   The current state of the flag.
    */
   @Override
   public boolean isNoLabels()
   {
      return !config.labels;
   }

   /**
    * Force NO-LABELS flag.
    * 
    * @param    labels
    *           The new value of the flag.
    */
   @Override
   public void setNoLabels(boolean labels)
   {
      config.labels = !labels;
   }
   
   /**
    * Return current value of COLON option.
    * 
    * @return   The current value of the option column.
    */
   @Override
   public double getColon()
   {
      return ConfigHelper.getColon(config);
   }

   /**
    * Get widget alignment policy.
    * 
    * @return   <code>true</code> if the widget is formatted for right
    *           alignment.
    */
   @Override
   public boolean isRightAligned()
   {
      return false;
   }
   
   /**
    * Calculates the minimum size of the of the combo-box by returning the 
    * largest label width of all elements in the list.
    *
    * @return   The minimum width of the combo-box or 0 if there are no
    *           elements.
    */
   public int getMaxLabelLen()
   {
      return maxlen;
   }

   /**
    * Shows or hides the combo-box and its label.
    *
    * @param    visible_
    *           <code>true</code> to make the widget visible, 
    *           <code>false</code> to hide the widget.
    */
   @Override
   public void setVisible(boolean visible_)
   {
      super.setVisible(visible_);

      if (label != null)
         label.setVisible(visible_);
   }
   
   /**
    * Tracks the hidden state of the widget and attached label.
    * 
    * @param    hidden
    *           New hidden state.
    */
   @Override
   public void setHidden(boolean hidden)
   {
      super.setHidden(hidden);
      
      if (label != null)
         label.setHidden(hidden);
   }

   /**
    * Get DCOLOR attribute.
    * 
    * @return  The color definition.
    */
   public Color getDColor()
   {
      return UiUtils.getDColor(this);
   }
   
   /**
    * Posts value changed event to the queue.  This should be done in ChUI and GUI mode after
    * closing drop-down or in GUI mode after pressing UP/DOWN keys with focused combo-box.
    */
   protected void valueChangedCommon()
   {
      // generate value changed event
      String item = model().selected();

      // put an ActionEvent onto the system event queue
      EventManager.postEvent(new ActionEvent(this, item));
      
      // honor any Progress triggers
      KeyInput vchng = new KeyInput(this, EventType.KEY_PRESSED, Keyboard.SE_VALUE_CHANGED, true);
      EventManager.postEvent(vchng);
      
      ConfigHelper.setModified(config, true);
   }
   
   /**
    * Exit the drop-down (the "selection" mode of the combo-box).
    */
   public void exitDropDown()
   {
      if (!isDropDownActive())
      {
         return;
      }
      
      // this also destroys and detaches the drop down from its parent
      dropDown.hide();

      // put focus back on the combo-box, but no ENTRY event generated
      parent().setFocus(this);
   }
   
   /**
    * Get list of the drop-down
    * 
    * @return list of the drop-down
    */
   public Widget<?> getDropDownList()
   {
      return dropDown == null ? null : dropDown.getScrollPane();
   }

   /**
    * Starts the drop-down part of the life cycle. The user selects
    * one item in the drop-down list or presses END-ERROR to exit without
    * making a selection. Then the list is closed and if a selection was
    * made, the new selection is returned.
    */
   @SuppressWarnings({ "unchecked", "rawtypes" })
   protected void activate()
   {
      if (config.innerLinesAssignments != 0)
      {
         config.innerLinesAssignments = 0;
      }
      if (config.innerLinesLimitDelta != 0)
      {
         int newLimit = config.innerLinesLimit - config.innerLinesLimitDelta;
         config.innerLinesLimit = Math.max(newLimit,  config.innerLines);
         config.innerLinesLimitDelta = Math.max(0, config.innerLines - newLimit);
      } 

      final ThinClient tc = ThinClient.getInstance();
      tc.independentEventDrawingBracket(ComboBox.this, () ->
      {
         // create the drop-down box
         dropDown = screen().getFactory().createDropDown(ComboBox.this);
            
         // register it as a new frame
         registerDropDownFrame();
            
         // enable and show the drop-down
         dropDown.show();
         dropDown.setEnabled(true);
         
         if (!tc.isChui())
         {
            // this is needed in case combo-box is clicked and drop-down
            // activated while another drop-down active 
            requestFocus();
         }
         
         dropDownRequestFocus();
         ThinClient.getInstance().setActiveComboBox(ComboBox.this, 
                                          getScrollPaneId(), getEffectiveMousePointer());
      });
      
      Widget<O> focus = dropDown.getScrollPane();
      
      int fid = UiUtils.getWidgetIdAsInt(focus);

      // add the exit event
      EventList list = new EventList();
      Set<String> events = initEventsSet();
      Set<Integer> widgets = new HashSet<>();
      widgets.add(fid);
      EventDefinition event = new EventDefinition(events, widgets, null, false);
      event.setTriggerId(-1);
      list.addEvent(event);
      // adding UI specifics
      addUISpecificEvents(list);

      Widget<O> oldPending = tc.getPendingFocus();
      tc.setPendingFocus(focus);
      
      this.valueChanged = false;
      int save = tc.getOutputManager().saveInvalidate(focus);
      
      EventManager.pushEventQueue();
      try
      {
         // Do not let waitForWorker to mangle with focus. In GUI there are multiple top-level
         // windows, so focus will be naturally preserved and moved to the combo when drop-down 
         // closed. In ChUI we explicitly restore focus below in the finally block. Also when 
         // user clicks into an unrelated widget while drop-down active, we want the focus to 
         // go directly to that clicked widget.
         tc.waitForWorker(list, fid, -1, null, true, false, false, 
                          BlockingOperation.NO_BLOCKING_OPERATION, false);
      }
      catch (EndConditionException |
             ErrorConditionException ce)
      {
         // consume this
      }
      catch (ConditionException ce)
      {
         // all others propagate
         throw ce;
      }
      finally
      {
         // we have exited the waitFor event loop, no need to raise end condition
         endConditionRaised = true;
         
         tc.getOutputManager().restoreInvalidate(save);

         tc.setPendingFocus(oldPending);
         
         tc.independentEventDrawingBracket(ComboBox.this, deactivateCode());
         focus = (Widget<O>) UiUtils.getCurrentFocus();
         
         // if no specific widget was chosen as focused by triggers
         // force the focus back to the combo-box, validity of the focus
         // will be evaluated just below
         if (focus == null)
         {
            parent().setFocus(this);
            focus = (Widget<O>) UiUtils.getCurrentFocus();
         }
         
         if (focus == this)
         {
            int state = FocusManager.focusState(this);
            
            if (state != FocusManager.FOCUS_VALID)
            {
               tc.getFocusManager().adjustFocus();
            }
         }

         EventManager.popEventQueue();
         
         // make postprocessing if required
         postprocessDropDown();
      }
   }
   
   /**
    * Get drop down deactivation Runnable
    * 
    * @return drop down deactivation Runnable
    */
   public Runnable deactivateCode()
   {
      return () -> 
      {
         exitDropDown();
         if (valueChanged)
         {
            valueChangedImpl();
         }
      };
   }
   /**
    * Sometimes it is required to do several action the was deferred during drop-down life cycle.
    */
   protected void postprocessDropDown()
   {
      // do nothing in general case
   }
   
   /**
    * Registers drop-down frame in current window system.
    */
   protected abstract void registerDropDownFrame();

   /**
    * Requesting focus to opened drop-down.
    */
   protected abstract void dropDownRequestFocus();
   
   /**
    * Initializes the event set to return.  Common version.
    * 
    * @return  The keyboard event to be used to exit drop-down.
    */
   protected Set<String> initEventsSet()
   {
      Set<String> setResult = new HashSet<>();
      setResult.add("RETURN");
      
      return setResult;
   }
   
   /**
    * Adding mouse clicks as exit events.  Common version.
    * 
    * @param   listToChange
    *          The event list to modify.
    */
   protected void addUISpecificEvents(EventList listToChange)
   {
      // in common case do nothing for now
      return;
   }
   
   /**
    * Calculate width limit.
    * <p>
    * This method is ChUI specific, do not call from GUI code.
    * 
    * @param   body
    *          Perform calculation for body part of widget.
    * 
    * @return  Calculated width limit.
    */
   public double calcLimit(boolean body)
   {
      // always honor an explicit width first
      double maxwid = config.widthChars - 4;
      
      // calculate the size based on the format 
      String       type = config.dataType;
      BaseDataType bdt  = DisplayFormat.instanceOfType(type);
      
      int fmtlen = bdt.formatLength(config.format); 
      
      // safe, comparison of scaled doubles
      if (maxwid < 0)
      {
         if (isFormatHonored())
            maxwid = fmtlen;
         else
            maxwid = getMaxLabelLen();
      }
      
      // safe, comparison of scaled doubles
      if (body && fmtlen < maxwid  && config.autoResize)
         maxwid = fmtlen;
      
      return maxwid;
   }

   /**
    * Get widget screen value.
    * 
    * @return  Current widget screen value.
    */
   public BaseDataType getScreenValue()
   {
      ControlSetItem[] items = config.items;
      int ndx = model().getSelectedIndex();
      
      if (ndx < 0 || ndx >= items.length)
      {
         BaseDataType var = DisplayFormat.instanceOfType(config.dataType);
         var.setUnknown();
         
         return var;
      }

      ControlSetItem item = items[ndx];

      return (item.getLabel() == null) || "".equals(item.getLabel().toStringMessage()) ? 
            new character() : item.getValue();
   }

   /**
    * Get widget minimum size.
    * 
    * @return  widget minimum size.
    */
   @Override
   public Dimension minimumSize()
   {
      return dimension();
   }

   /**
    * Get underlying model.
    *  
    * @return  reference to model.
    */
   public ComboBoxModel<String> model()
   {
      return model;
   }

   /**
    * Process event and dispatch it to specific method.
    * 
    * @param   event
    *          Event to process.
    */
   @Override
   public void processEvent(Event event)
   {
      super.processEvent(event);
      
      if (event instanceof ActionEvent)
      {
         onSelect(new ItemEvent(this));
      }
   }
   
   /**
    * Checking if the widget was made visible at least once.
    * 
    * @return  <code>true</code> if the widget was visible,
    *          <code>false</code> otherwise.
    */
   @Override
   public boolean wasVisible()
   {
      return true;
   }
   
   /**
    * Remember the widget was made visible at least once.
    */
   @Override
   public void setWasVisible()
   {
      return;
   }
   
   /**
    * Check if the current widget can manage non-global system actions on its own, and not via the
    * enclosing event loop.
    * 
    * @return   <code>true</code> as the widget has its own event management.
    */
   @Override
   public boolean manageSystemActions()
   {
      return true;
   }

   /**
    * Process the action associated with the given key in an independent drawing bracket.
    * <p>
    * This must be called only in the context of widget-level event processing.
    * 
    * @param    key
    *           The key to be processed.
    */
   @Override
   public void processSystemKey(final KeyInput key)
   {
      if (key.actionCode() == Keyboard.KA_CURSOR_DOWN)
      {
         activate();
      }
      else
      {
         // let the widget process this key
         ThinClient.getInstance().independentEventDrawingBracket(this, 
                                                                 () -> processKeyEvent(key));
      }
   }
   
   /**
    * Additional processing required when widget configuration is updated AFTER the frame layout
    * has been performed.
    */
   @Override
   public void postprocessConfig()
   {
      super.postprocessConfig();
      
      if (label != null)
      {
         label.postprocessConfig(this);
      }
   }
   
   /**
    * Destroy this widget.
    */
   @Override
   public void destroy()
   {
      if (label != null)
      {
         label.destroy();
         label = null;
      }
      
      super.destroy();
   }
   
   /**
    * Get the id of the drop-down scroll pane
    * 
    * @return the id of the drop-down scroll pane
    */
   public int getScrollPaneId()
   {
      return dropDown == null ? -1 : dropDown.getScrollPane().getId().asInt();
   }

   /**
    * MnemonicInfo instance getter.
    * <p>
    * The implementors should simply return the value of the mnemonic class field. No
    * additional processing logic is advised.
    *
    * @return   See above.
    */
   public MnemonicInfo getMnemonic()
   {
      return mnemonic;
   }

   /**
    * MnemonicInfo instance setter.
    * <p>
    * The implementors should simply assign the parameter value to the mnemonic class
    * field. No additional processing logic is advised.
    */
   public void setMnemonic(MnemonicInfo mnemonic)
   {
      this.mnemonic = mnemonic;
   }

   /**
    * Column {@link MnemonicInfo} instance getter.
    * <p>
    * The implementors should simply return the value of the mnemonic class field. No
    * additional processing logic in the implemented method is advised.
    * <p>
    * The method will return a valid value when the implementing widget supports column label.
    *
    * @return   See above.
    */
   @Override
   public MnemonicInfo getColumnMnemonic()
   {
      return columnMnemonic;
   }

   /**
    * Column {@link MnemonicInfo} instance setter.
    * <p>
    * The implementors should simply assign the parameter value to the mnemonic class
    * field. No additional processing logic in the implemented method is advised.
    */
   public void setColumnMnemonic(MnemonicInfo mnemonic)
   {
      this.columnMnemonic = mnemonic;
   }

   /**
    * Returns <code>true</code> when drop-down is visible.
    * 
    * @return   See above.
    */
   protected boolean isDropDownActive()
   {
      return dropDown != null && dropDown.isDisplayed();
   }
   
   /**
    * Returns true if the format settings are used to format values.
    * 
    * @return   True if the format settings are in use.
    */
   public boolean isFormatHonored()
   {
      return config.honorFormat;
   }
}
