/*
** Module   : ScreenData.java
** Abstract : Container for entire screen data.
**
** Copyright (c) 2011-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- -----------------------Description------------------------
** 001 SIY 20110112 Created initial version
** 002 GES 20110914 Added asArray() method.
** 003 GES 20111202 Updated imports.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.ui.client;

import java.util.List;
import java.util.*;

/**
 * Entire screen data container.
 *
 * @author   SIY
 * @version  1.0
 */
public class ScreenData
{
   /** Screen image. */
   private final Cell[][] data;

   /** Required cursor X coordinate */
   private final int cursorX;

   /** Required cursor Y coordinate */
   private final int cursorY;

   /**
    * @param   data
    *          Screen content data.
    * @param   cursorX
    *          Desired cursor X.
    * @param   cursorY
    *          Desired cursor Y.
    */
   public ScreenData(Cell[][] data, int cursorX, int cursorY)
   {
      this.data = data;
      this.cursorX = cursorX;
      this.cursorY = cursorY;
   }

   /**
    * Get cell at specified row and column.
    *
    * @param   row
    *          Row.
    * @param   col
    *          Column.
    *
    * @return  instance holding cell data for specified location.
    */
   public Cell getCell(int row, int col)
   {
      return data[row][col];
   }

   /**
    * Get cursor X.
    *
    * @return cursor X
    */
   public int getCursorX()
   {
      return cursorX;
   }

   /**
    * Get cursor Y.
    *
    * @return cursor Y
    */
   public int getCursorY()
   {
      return cursorY;
   }

   /**
    * Get screen data as a multi-dimensional array of cells.
    *
    * @return   Multi-dimensional array of cells.
    */
   public Cell[][] asArray()
   {
      Cell[][] result = new Cell[data.length][data[0].length];

      for (int i = 0; i < data.length; i++)
      {
         for (int j = 0; j < data[i].length; j++)
         {
            // the Cell class is immutable so it is OK to copy the reference
            result[i][j] = data[i][j];
         }
      }

      return result;
   }
   
   /**
    * Get screen data as list of cells.
    *
    * @return  list of cells.
    */
   public List<Cell> list()
   {
      ArrayList<Cell> result = new ArrayList<Cell>();

      for (int i = 0; i < data.length; i++)
      {
         for (int j = 0; j < data[i].length; j++)
         {
            result.add(data[i][j]);
         }
      }

      return result;
   }
}