/* 
** Module   : List.java
** Abstract : drop-down list helper implementation for CHUI
**
** Copyright (c) 2005-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- --JPRM-- ----------------------------Description------------------------------
** 001 EVL 20050214   @19831 Creation based on CHARVA JList class.
**                           Necessary modification to get compiled
** 002 EVL 20050523   @21257 Eat the mouse events.
** 003 EVL 20050603   @21405 Changing the inner variable names to sync
**                           with the rest of the project and preparing
**                           to javadoc processing;
** 004 EVL 20050607   @21435 Syntax for comments checking.
** 005 EVL 20050729   @21915 Removing the ugly tabulation from source.
** 006 SIY 20050831   @22507 Refactoring (see @@22394-22409). 
** 007 SIY 20050907   @22641 Fixed use of @inheritDoc tag in javadoc.
** 008 SIY 20050926   @22843 Selected proper attribute container, using 
**                           ColorTable for drawing.
** 009 SIY 20060209   @24457 Cleanups and updates to accommodate changes in
**                           xxxConfig classes, handling of empty list.
** 010 SIY 20060209   @24497 Added getCurrentRow() method. 
** 011 NVS 20060227   @24865 Key actions KA_* are used instead of virtual
**                           keys VK_* for all editing operations.
** 012 EVL 20060810   @28543 Adding ability to search the item in combo
**                           box widget as reaction for alphabetical key
**                           pressed when the DropDown is opened.
** 013 SIY 20060824   @28854 Removed getP2JType() method.
** 014 GES 20061107   @30999 Removed dead code and made fixes to key
**                           event processing.
** 015 NVS 20061204   @31555 Switched to using a private widgetFocused
**                           flag instead of config.selected
** 016 SIY 20061212   @31672 Fixed drawing of the drop-down list. Cleanups
**                           and refactoring.
** 017 SIY 20080411   @37938 Fixed drawing of the drop-down list.
** 018 NVS 20080501   @38191 Fixed drawing of the drop-down list one more
**                           time, this time for the empty list. Added
**                           drawCaret() method.
** 019 GES 20090424   @41964 Import change.
** 020 CA  20100507   @44849 Added getBox(), to retrieve the drop-down to
**                           which the list is associated with.
** 021 SIY 20100619          Switching to own implementation of character UI.
** 022 SIY 20100804          Changes in imports, some refactorings.
** 023 SIY 20101122          Switching from KeyEvent to KeyInput.
** 024 SIY 20110112          Changes related to new color handling scheme.
** 024 SIY 20111121          Moved from CHUI and renamed in order to avoid 
**                           conflicts with java.util.List
** 025 SIY 20111202          Refactored and moved from CHUI package.
** 026 CA  20140805          Added generics to force the abstract widget classes to not use
**                           implementation-specific code: the abstract widgets will have access
**                           only to the base APIs provided by the OutputManager.
** 027 HC  20140911          GUI coordinate support - data types refactoring.
** 028 CA  20150212          Allow events to be captured and processed by the widgets without
**                           server-side or main event-loop notification.  This fixes problems in
**                           the EDITING and WAIT-FOR event loops, for i.e. COMBO-BOX and EDITOR
**                           widgets.
** 029 EVL 20150517          Adding api to set up the top row index.  Adding method to select
**                           item from first letter from outside thjis class, combo-box GUI
**                           implementation needs this ability.
** 030 HC  20150831          Scrolling support overhaul.
** 031 HC  20151029          Fixed drop-down scroll list width calculation, it must not be
**                           smaller than the parent's width.
**         20151110          GUI combo-box fixes.
** 032 VIG 20160420          Added isScrollPopupSupported implementation.
** 033 EVL 20161102          Fix fir SIMPLE mode combo-box issues.  Adding worker to search the
**                           item from the list by first located substring.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.ui.client;


import com.goldencode.p2j.ui.*;
import com.goldencode.p2j.ui.client.event.*;
import com.goldencode.p2j.ui.client.model.*;
import com.goldencode.p2j.ui.client.widget.*;

/**
 * A component that allows the user to select one or more objects from a
 * list.
 * <p>
 * This class does not provide scrolling capability.  Instances must be
 * inserted into a {@link ScrollPane} to provide scrolling.
 * <p>
 * This class is intended to only be used as the container of the actual
 * "contents" inside the drop-down portion of a {@link ComboBox}. For the
 * general purpose selection list please use the {@link SelectionList} class.
 */
public abstract class ScrollableList<O extends OutputManager<?>>
extends DefaultList<String, O>
{
   /** ComboBox instance which this instance services. */
   protected ComboBox<O> box = null;

   /**
    * Create component using given configuration and data model.
    * 
    * @param    box
    *           The instance with which this object must be associated.
    * @param    model
    *           The data model that represents the contents of the list.
    */
   public ScrollableList(ComboBox<O> box, ListModel<String> model)
   {
      super(model);
      allowDeselect(false);
      this.box = box;
   }

   /**
    * Get widget minimum size.
    * 
    * @return  widget minimum size.
    */
   @Override
   public Dimension minimumSize()
   {
      // super.minimumSize() calculates columns only based on the list model,
      // instead we need to return the columns value calculated and set by DropDown 
      double columns = getColumns();
      return new Dimension(columns, visibleRows());
   }

   /**
    * Draws this component. Overrides draw() in JList.
    */
   @Override
   public abstract void draw();
   
   /**
    * Sets position of terminal's cursor to the first char of the combo-box.
    */
   @Override
   public abstract void drawCaret();
   
   /**
    * Set item number to be displayed on the top of the drop-down list.
    * 
    * @param   index
    *          The index of the top item.
    */
   protected abstract void setTopRow(int index);

   /**
    * Get item text at specified index.
    * 
    * @param   i
    *          The item index.
    *          
    * @return  The item text.
    */
   protected String getItemText(int i)
   {
      return model().get(i).toString();
   }

   /**
    * Intercept key events and handle some special cases which are not
    * handled by base class.
    * 
    * @param    kvt
    *           Key event to process.
    */
   @Override
   public void processKeyEvent(KeyInput kvt)
   {
      int key = kvt.actionCode();
      
      // disable tab and back-tab (we don't give the parent classes a chance
      // to operate on these since they are disabled in Progress)
      if (key == Keyboard.KA_TAB || key == Keyboard.KA_BACK_TAB)
      {
         kvt.consume();
         screen().beep();
         return;
      }

      // END-ERROR action exits the drop down without making any change
      if (key == Keyboard.KA_END_ERROR)
      {
         return;
      }
      
      // Implementation of the item searching by first char. Launches for non
      // functional keys - alphanumerical and for existed combo-box.
      if (box != null)
      {
         int newIndx = getFirstCharSelection(kvt, model(), currentRow());
         // We found the new item to highlight and it is not the same as
         // the current one
         if (newIndx >= 0)
         {
            // select it and ensure it will be visible in the list
            ensureRowVisible(newIndx);
            kvt.consume();
            return;
         }
      }
      
      super.processKeyEvent(kvt);
      
      if (key == Keyboard.SE_LEAVE)
      {
         // ignore
      }
      else
      {
         // bypass all further processing
         kvt.consume();
      }
   }

   /**
    * Gets item index for the given model according to the first item char and key code from
    * the key event.  If the new item is the same as the curently selected in the given model
    * -1 will be returned.
    * 
    * @param   ki
    *          The event to be tested.
    * @param   model
    *          The list model to search the new item.
    * @param   currIndx
    *          The currently selected item for th emode to search in.
    *          
    * @return  The new index if the pressed key match the first char of the item or -1 if
    *          there is no match or new item is the same as the currently selected one.
    */
   public static int getFirstCharSelection(KeyInput ki, ListModel<String> model, int currIndx)
   {
      // the index to return if no item found
      int newIndx = -1;
      
      if (ki.keyCode() < 256)
      {
         char keyChar = ki.keyChar();
         // Only letter or digit is interesting here
         if (keyChar != 0 && Character.isLetterOrDigit(keyChar))
         {
            int listSize = model.size();
            // Scan complete list model
            for (int i = 0; i < listSize; i++)
            {
               // Retrieve next item in String representation
               String item = model.get(i).toString();
               // Check equality only for the first char but
               // ignoring the letter case
               if (Character.toUpperCase(item.charAt(0)) == Character.toUpperCase(keyChar))
               {
                  // Found the candidate, no need to continue search
                  newIndx = i;
                  break;
               }
            }

            // we found the new item to highlight and it is not the same as
            // the current one, otherwise undo search result
            if (newIndx == currIndx)
            {
               newIndx = -1;
            }
         }
      }
      
      return newIndx;
   }
   
   /**
    * Gets item index for the given model according to the item list name and given substring
    * considering it at the beginning of the item name.  If the new item is the same as the
    * curently selected it is up to caller to make proper action further.
    * 
    * @param   text
    *          The text to be tested for beginning part.
    * @param   model
    *          The list model to search the new item.
    *          
    * @return  The new index if the given text match the first part of the item or -1 if
    *          there is no match.
    */
   public static int getStartStringSelection(String text, ListModel<String> model)
   {
      // the index to return if no item found
      int newIndx = -1;
      
      if (text != null && !text.isEmpty())
      {
         // prepare to case-insensitive search
         text = text.toUpperCase();
         int listSize = model.size();
         // Scan complete list model
         for (int i = 0; i < listSize; i++)
         {
            // Retrieve next item in String representation
            String item = model.get(i).toString();
            // Check equality the item text start with incoming text, no case sensitivity
            if (item.toUpperCase().indexOf(text) == 0)
            {
               // Found the candidate, no need to continue search
               newIndx = i;
               break;
            }
         }
      }
      
      return newIndx;
   }
   
   /**
    * Get current cursor row.
    * 
    * @return   The current cursor row.
    */
   public int getCurrentRow()
   {
      return currentRow();
   }
   
   /**
    * Get the combo-box instance to which this drop-down list is attached to.
    * 
    * @return   the {@link ComboBox} instance to which this drop-down 
    *           list is attached to.
    */
   public ComboBox<O> getBox()
   {
      return box;
   }
   
   /**
    * Check if this widget that supports scrolling also supports scroll popup.
    *
    * @return  <code>true</code> if this widget supports scroll popup.
    */
   public boolean isScrollPopupSupported()
   {
      return box.isSimpleMode();
   }
}