/*
** Module   : TerminalOptions.java
** Abstract : configuration container
**
** Copyright (c) 2011-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- ---------------------------------Description---------------------------------
** 001 GES 20110921 Created initial version.
** 002 CA  20141029 Added flag to identify if the client must run in windows compatibility mode.
** 003 GES 20150424 Eliminated AWT dependency and moved to a generic location for better reuse.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.ui.client.chui.driver;

import com.goldencode.p2j.ui.*;

/**
 * Simple configuration container.
 */
public class TerminalOptions
{
   /** The default number of rows of text on the terminal screen. */
   public static int DEF_ROWS = 24;
   
   /** The default number of columns of text on the terminal screen. */
   public static int DEF_COLUMNS = 80;
   
   /** The default background color for each character (0x000000 - black). */
   public static ColorRgb DEF_BACKGROUND = new ColorRgb(0, 0, 0);
   
   /** The default foreground color for each character (0xFFA500 - amber). */
   public static ColorRgb DEF_FOREGROUND = new ColorRgb(255, 165, 0);
   
   /** The default color of the selection box (0x0000FF - blue). */
   public static ColorRgb DEF_SELECTION = new ColorRgb(0, 0, 255);
   
   /** The default font used for all text. */
   public static String DEF_FONTNAME = "monospaced";
   
   /** The default point size used for all text. */
   public static int DEF_FONTSIZE = 12;
   
   /** The number of rows of text on the terminal screen. */
   public int rows = DEF_ROWS;
   
   /** The number of columns of text on the terminal screen. */
   public int columns = DEF_COLUMNS;
   
   /** The background color for each character. */
   public ColorRgb background = DEF_BACKGROUND;
   
   /** The foreground color for each character. */
   public ColorRgb foreground = DEF_FOREGROUND;
   
   /** The color of the box when the user selects a rectangle of text. */
   public ColorRgb selection = DEF_SELECTION;
   
   /** The font used for all text. */
   public String fontname = DEF_FONTNAME;
   
   /** The font's point size used for all text. */
   public int fontsize = DEF_FONTSIZE;
   
   /** Flag indicating if the client is running in windows compatibility mode. */
   public boolean underWindows = false;
   
   /**
    * Compare the given instance to the current instance and report if the
    * two are equivalent. This implementation compares the data stored in
    * each instance member-wise.
    *
    * @param    options
    *           The instance with which the current data is compared.
    *
    * @return   <code>true</code> if both instances contain equivalent data.
    */
   public boolean equals(TerminalOptions options)
   {
      return (rows                == options.rows           &&
              columns             == options.columns        &&
              fontsize            == options.fontsize       &&
              underWindows        == options.underWindows   &&
              background.equals(options.background)         &&
              foreground.equals(options.foreground)         &&
              selection.equals(options.selection)           &&
              fontname.equalsIgnoreCase(options.fontname));
   }
}