/*
** Module   : PaintPrimitives.java
** Abstract : paint operations in the GUI interface
**
** Copyright (c) 2014-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- ---------------------------------Description---------------------------------
** 001 MAG 20140508 First version.
** 002 CA  20140805 Added CLEAR, CLEAR_AREA, NO_CLIP, CLIP primitives.
** 003 MAG 20140912 Added FILL_POLYGON.
** 004 CA  20141021 Added SET_LOCATION, RESIZE_WINDOW, START_BATCH, END_BATCH.
** 005 HC  20141031 Implemented support for drawing highlighted areas.
** 006 EVL 20141116 Adding round rectangles draw and fill.
** 007 EVL 20141125 Adding dotted line draw capability.
** 008 CA  20141220 Added DRAW_STRING_SCALED.
** 009 CA  20150408 Added SET_TITLE.
** 010 GES 20150422 Moved this class to a more generic location (not specific to Swing).
** 011 HC  20150612 Implemented support for paragraph layout.
** 012 GES 20150720 Added a warning about future modifications.
** 013 HC  20150831 Improved GUI window resize performance.
** 014 CA  20150821 Added SET_ICON.
** 015 SVL 20151013 Added SET_XOR_COMPOSITE and RESET_COMPOSITE.
** 016 CA  20160216 Added primitives for drawing operations recording and caching.
**     SBI 20160324 Added SET_DESKTOP_BGCOLOR.
** 017 EVL 20170706 Adding SET_XOR_MODE, DRAW_SELECTION, RESET_SELECTION.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.ui.client.gui.driver;

/**
 * This is the list of possible GUI painting primitives. The values are used as id's for paint
 * operations.
 * <p>
 * WARNING: It is safest and easiest to add to the end of this list. The reason is that the
 * enum ordinal is being used directly at the driver level for encoding drawing operations.
 * If you add in the middle of the list of values, then that value and all following ones will
 * have their ordinals changed and the downstream code will break.  You may add to the middle
 * so long as YOU fix up the downstream usage!
 * <p>
 * WARNING: If there are ever more than 256 values in this enum, it will break downstream code
 * that assumes the ordinal will fit in an octet.
 */
public enum PaintPrimitives
{
   DRAW_STRING,
   DRAW_STRING_SCALED,
   DRAW_PARAGRAPH,   
   DRAW_RECT,
   DRAW_ROUND_RECT,
   DRAW_IMAGE,
   FILL_RECT,
   FILL_ROUND_RECT,
   FILL_POLYGON,
   DRAW_LINE,
   DRAW_3D_RECT,
   FILL_3D_RECT,
   SET_COLOR,
   SET_LINE_STROKE,
   SET_FONT,
   SET_FONT_STYLE,
   TRANSLATE_PUSH,
   TRANSLATE_POP,
   CLEAR,
   CLEAR_AREA, 
   NO_CLIP,
   CLIP,
   FLUSH,
   SET_LOCATION,
   RESIZE_WINDOW,
   SET_WINDOW_BOUNDS,
   DRAW_HIGHLIGHT,
   CLEAR_HIGHLIGHT,
   START_BATCH,
   END_BATCH,
   SET_TITLE,
   SET_ICON,
   SET_XOR_COMPOSITE,
   RESET_COMPOSITE, 
   START_RECORDING,
   STOP_RECORDING,
   DRAW_CACHED,
   SET_DESKTOP_BGCOLOR,
   SET_XOR_MODE,
   DRAW_SELECTION,
   RESET_SELECTION;
}