/*
** Module   : HandleCommon.java
** Abstract : A massive interface which groups together all attributes and methods available
**            via a handle.
**
** Copyright (c) 2013-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- --------------------------------Description----------------------------------
** 001 CA  20130112 Created initial version.
** 002 CS  20130204 Added support for socket and server-socket. Moved sensitive related methods 
**                  from CommonWidget in the Sensitive interface.
** 003 OM  20130129 Added TempTable and Buffer interface.
** 004 CS  20130213 Added conversion support for app server.
** 005 CA  20130221 Added missing interfaces.
** 006 OM  20130129 Added Dereferenceable interface.
** 007 OM  20130320 Added ResultListHandler interface. Sorted all interfaces alphabetically.
** 008 CS  20130403 Added ConectableServer and BufferHandle interfaces.
** 009 CA  20130813 Added support for the async request reosurce.
** 010 GES 20141110 Added LegacyLogManager, BrowseInterface, BrowseElement, AutoZapElement,
**                  MaxCharsElement and ScrollbarVerticalElement.
** 011 GES 20141120 Added EditorInterface, InnerLines, QueryAssociable, FrameInterface, NumItems
**                  and ScrollbarHorizontalElement.
** 012 EVL 20150128 Added ImageButtonSupport, ImageSupport.
** 013 VIG 20150131 Added interface MenuInterface to implements clause.
** 014 EVL 20150223 Adding ImageWidgetSupport.  ImageSupport now defines features common to button
**                  and image widgets while ImageWidgetSupport only has the image widget specific.
** 015 IAS 20150224 Added RadioSetInterface.
** 016 EVL 20150227 Changing ImageButtonSupport to ButtonInterface.
** 017 ECF 20150315 Replaced FieldInterface with BufferField.
** 018 IAS 20150330 Added BoxInterface.
** 019 CA  20150416 Added CommonHandleTree.
** 020 IAS 20150604 Added ReplaceInterface and SelectableText.
** 021 SVL 20150804 Added Down and Multiple.
** 022 IAS 20150805 Added DeblankInterface, BlankInterface, PasswordFieldInterface 
**                  and IndexedInterface.
** 023 VIG 20150818 Added MenuItemInterface.
** 024 EVL 20150825 Adding SelectionListInterface class.
** 025 EVL 20150911 Adding OrientationInterface and SliderInterface.
** 026 IAS 20150926 Adding TopOnlyInterface class.
** 027 SVL 20151130 Added Labels.
** 028 CA  20160327 Added support for LAST-EVENT:X and LAST-EVENT:Y attributes.
** 029 SBI 20160608 HandleCommon must support Clipboard.
** 030 GES 20160909 Added RectangleInterface.
** 031 SVL 20170126 Added RcodeInfo.
** 032 EVL 20170526 Adding drag-and-drop related intefraces.
** 033 OM  20170530 Added support for TIMER FWD extension and ControlFrames.
*/

/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.util;

import com.goldencode.p2j.extension.*;
import com.goldencode.p2j.persist.*;
import com.goldencode.p2j.ui.*;
import com.goldencode.p2j.xml.*;

/**
 * This interface groups together all attributes and methods available via a
 * handle.  No explicit implementation exists for it, its purpose is only to
 * provide runtime error support, when the handle is either invalid or not
 * of the expected type. Thus, if one of the handle's unwrap APIs find the
 * handle invalid or of a different type than expected, the
 * {@link handle#invalidAttrAccessProxy} API is used to obtain a dynamic proxy,
 * with its implementation in the {@link handle} class.
 */
public interface HandleCommon
extends AsyncRequest,
        AutoZapElement,
        BlankInterface,
        BoxInterface,
        BrowseColumnInterface,
        BrowseElement,
        BrowseInterface,
        Buffer,
        BufferField,
        BufferHandle,
        ButtonInterface,
        Clipboard,
        CommonErrorStatus,
        CommonField,
        CommonFileInfo,
        CommonHandle,
        CommonHandleChain,
        CommonHandleTree,
        CommonLastEvent,
        CommonSession,
        CommonWidget,
        CommonListWidget,
        CommonWindow,
        ControlFrame,
        Connectable,
        ConnectableServer,
        DeblankInterface,
        Dereferenceable,
        DirectManipulation,
        Down,
        Droppable,
        EditorInterface,
        Errorable,
        FirstLastProcedureAttribute,
        FrameInterface,
        ImageSupport,
        ImageWidgetSupport,
        IndexedInterface,
        InnerLines,
        Labels,
        LegacyLogManager,
        MaxCharsElement,
        MenuInterface,
        MenuItemInterface,
        Multiple,
        NumItems,
        OrientationInterface,
        PasswordFieldInterface,
        P2JQuery,
        PersistentProcedure,
        QueryAssociable,
        RadioSetInterface,
        RcodeInfo,
        RectangleInterface,
        ReplaceInterface,
        ResultListHandler,
        SaxAttributes,
        SaxEntity,
        SaxReader,
        SaxWriter,
        ScrollbarHorizontalElement,
        ScrollbarVerticalElement,
        SelectableText,
        SelectionListInterface,
        Sensitive,
        Server,
        SliderInterface,
        SOAPEntity,
        SOAPFault,
        SOAPHeader,
        SOAPHeaderEntry,
        Socket,
        SocketListener,
        SortedElement,
        SslServerAttribute,
        SubTypeAttribute,
        TempTable,
        TopOnlyInterface,
        XCommon,
        XDocument,
        XEntity,
        XNodeRef,
        XYCoordinates,
        
        // FWD extension interfaces are listed from this point forward
        FWDTimer 
{
}
