/*
** Module   : LegacyResource.java
** Abstract : Annotation definition for a legacy resource.
**
** Copyright (c) 2013-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- --------------------------------Description-----------------------------------
** 001 CA  20130926 Created initial version.
** 002 VIG 20141129 Added RULE resource.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.util;

import java.lang.annotation.*;

/**
 * Annotation definition to mark the Progress 4GL resource type implemented by a P2J
 * class. These resources are referenced by handle in the 4GL and these constants
 * correspond to the TYPE attribute that would be returned in the 4GL.
 */
@Target(ElementType.TYPE)
@Retention(RetentionPolicy.RUNTIME)
public @interface LegacyResource
{
   String resource();

   /** Constant identifying the ASYNC-REQUEST resource. */
   public static String ASYNC_REQUEST = "ASYNC-REQUEST";

   /** Constant identifying the BROWSE-COLUMN resource. */
   public static String BROWSE_COLUMN = "BROWSE-COLUMN";

   /** Constant identifying the BUFFER resource. */
   public static String BUFFER = "BUFFER";

   /** Constant identifying the BUFFER-FIELD resource. */
   public static String BUFFER_FIELD = "BUFFER-FIELD";

   /** Constant identifying the PROCEDURE resource. */
   public static String PROCEDURE = "PROCEDURE";

   /** Constant identifying the PSEUDO-WIDGET resource.  All system handles have this type. */
   public static String PSEUDO_WIDGET = "PSEUDO-WIDGET";
   
   /** Constant identifying the QUERY resource. */
   public static String QUERY = "QUERY";

   /** Constant identifying the SAX-ATTRIBUTES resource. */
   public static String SAX_ATTRIBUTES = "SAX-ATTRIBUTES";
   
   /** Constant identifying the SAX-READER resource. */
   public static String SAX_READER = "SAX-READER";
   
   /** Constant identifying the SAX-WRITER resource. */
   public static String SAX_WRITER = "SAX-WRITER";
   
   /** Constant identifying the SERVER resource. */
   public static String SERVER = "SERVER";
   
   /** Constant identifying the SERVER-SOCKET resource. */
   public static String SERVER_SOCKET = "SERVER-SOCKET";

   /** Constant identifying the SOAP-FAULT resource. */
   public static String SOAP_FAULT = "SOAP-FAULT";

   /** Constant identifying the SOAP-HEADER resource. */
   public static String SOAP_HEADER = "SOAP-HEADER";

   /** Constant identifying the SOAP-HEADER-ENTRYREF resource. */
   public static String SOAP_HEADER_ENTRYREF = "SOAP-HEADER-ENTRYREF";

   /** Constant identifying the SOAP-FAULT-DETAIL resource. */
   public static String SOAP_FAULT_DETAIL = "SOAP-FAULT-DETAIL";
   
   /** Constant identifying the SOCKET resource. */
   public static String SOCKET = "SOCKET";
   
   /** Constant identifying the TEMP-TABLE resource. */
   public static String TEMP_TABLE = "TEMP-TABLE";
   
   /** Constant identifying the X-DOCUMENT resource. */
   public static String X_DOCUMENT = "X-DOCUMENT";
   
   /** Constant identifying the X-NODEREF resource. */
   public static String X_NODEREF = "X-NODEREF";

   // start of widget resources

   /** Constant identifying the BROWSE resource. */
   public static String BROWSE = "BROWSE";

   /** Constant identifying the BROWSE-CELL resource. */
   public static String BROWSE_CELL = "BROWSE-CELL";

   /** Constant identifying the BUTTON resource. */
   public static String BUTTON = "BUTTON";

   /** Constant identifying the COMBO-BOX resource. */
   public static String COMBO_BOX = "COMBO-BOX";

   /** Constant identifying the CONTROL-FRAME resource. */
   public static String CONTROL_FRAME = "CONTROL-FRAME";

   /** Constant identifying the DIALOG-BOX resource. */
   public static String DIALOG_BOX = "DIALOG-BOX";

   /** Constant identifying the  resource. */
   public static String EDITOR = "EDITOR";

   /** Constant identifying the FIELD-GROUP resource. */
   public static String FIELD_GROUP = "FIELD-GROUP";

   /** Constant identifying the FILL-IN resource. */
   public static String FILL_IN = "FILL-IN";

   /** Constant identifying the FRAME resource. */
   public static String FRAME = "FRAME";

   /** Constant identifying the IMAGE resource. */
   public static String IMAGE = "IMAGE";

   /** Constant identifying the LITERAL resource. */
   public static String LITERAL = "LITERAL";

   /** Constant identifying the MENU resource. */
   public static String MENU = "MENU";

   /** Constant identifying the MENU-ITEM resource. */
   public static String MENU_ITEM = "MENU-ITEM";

   /** Constant identifying the RECTANGLE resource. */
   public static String RECTANGLE = "RECTANGLE";

   /** Constant identifying the RADIO-SET resource. */
   public static String RADIO_SET = "RADIO-SET";

   /** Constant identifying the RULE resource. */
   public static String RULE = "RULE";

   /** Constant identifying the SELECTION-LIST resource. */
   public static String SELECTION_LIST = "SELECTION-LIST";

   /**
    * Constant identifying a SKIP or SPACE widget; doesn't have a 4GL equivalent, this is a 
    * a synthetic resource in P2J.
    */
   public static String SKIP_OR_SPACE = "SKIP-OR-SPACE";

   /** Constant identifying the SLIDER resource. */
   public static String SLIDER = "SLIDER";

   /** Constant identifying the SUB-MENU resource. */
   public static String SUB_MENU = "SUB-MENU";

   /** Constant identifying the TEXT resource. */
   public static String TEXT = "TEXT";

   /** Constant identifying the TOGGLE-BOX resource. */
   public static String TOGGLE_BOX = "TOGGLE-BOX";

   /** Constant identifying the WINDOW resource. */
   public static String WINDOW = "WINDOW";
}