/*
** Module   : OutputParameterAssigner.java
** Abstract : API called by BlockManager to set output parameter values into
**            database fields
**
** Copyright (c) 2004-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- --JPRM-- -----------------------------Description-----------------------------
** 001 ECF 20090816   @43654 Created initial version. API called by BlockManager to set output
**                           parameter values into database fields.
** 002 OM  20150709          Added parameters to processAssignments.
** 003 EVL 20160224          Javadoc fixes to make compatible with Oracle Java 8 for Solaris 10.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.util;

/**
 * API called from BlockManager to set output parameter values into database
 * fields.
 */
public interface OutputParameterAssigner
{
   /**
    * Invoked by <code>BlockManager</code> when output parameters should have
    * their values assigned into the DMO properties with which they are associated.
    * <p>
    * Implementors should clean up their current batch of parameters before
    * this method returns, even if it returns abnormally.
    *
    * @param   endingRoutine
    *          The legacy name of the routine that is ending when these back assignments are
    *          performed. 
    * @param   function
    *          {@code true} if the called routine is a function.
    *
    * @throws  ErrorConditionException
    *          if validation of an output parameter value fails when the
    *          attempt is made to store it in the DMO.
    */
   public void processAssignments(String endingRoutine, boolean function);
   
   /**
    * Invoked by <code>BlockManager</code> when the current batch of output
    * parameters should <i>not</i> be assigned back to the corresponding DMO
    * properties.
    * <p>
    * Implementors should clean up their current batch of parameters upon
    * invocation of this method.
    */
   public void abort();
}