/*
** Module   : SocketFactory.java
** Abstract : A class for instantiating SOCKET handles.
**
** Copyright (c) 2013-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- --------------------------------Description-----------------------------------
** 001 CS  20130204 Created initial version.
** 002 CA  20130916 Do not allow server sockets on appserver calls.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.util;

/**
 * A class for instantiating socket handles.
 */
public class SocketFactory
{
   /**
    * Method for assigning a <code>SocketImpl</code> to the handle defining it 
    * as a socket.
    * 
    * @param socketHandle
    *        The handle to which we will assign the socket
    */
   public static void createSocket(handle socketHandle)
   {
      socketHandle.assign(getSocket());
   }
   
   /**
    * Method for assigning a <code>SocketListenerImpl</code> to the handle 
    * defining it as a server-socket.
    * 
    * @param socketHandle
    *        The handle to which we will assign the socket
    */
   public static void createServerSocket(handle socketHandle)
   {
      if (AppServerManager.isRemote())
      {
         final String msg1 = "CREATE SERVER-SOCKET can not be called from an Appserver";
         final String msg2 = "Cannot create widget of type SERVER-SOCKET";
         
         ErrorManager.recordOrThrowError(new int[]    { 9353, 3181 },
                                         new String[] { msg1, msg2},
                                         false);
         
         return;
      }
      
      socketHandle.assign(getServerSocket());
   }
   
   /**
    * Method which returns a new SocketListenerImpl.
    * 
    * @return a new SocketListenerImpl as a WrappedResource.
    */
   private static WrappedResource getServerSocket()
   {
      return new SocketListenerImpl();
   }
   
   /**
    * Method which returns a new SocketImpl.
    * 
    * @return a new SocketImpl as a WrappedResource.
    */
   private static WrappedResource getSocket()
   {
      return new SocketImpl();
   }
}