/*
** Module   : comhandle.java
** Abstract : Progress 4GL compatible COM handle object
**
** Copyright (c) 2010-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- ---------------------------------Description---------------------------------
** 001 GES 20101005 Placeholder version with some stubs for attributes and methods.
** 002 SIY 20101014 Changed signatures.
** 003 OM  20170523 Added new methods and completed implementation of existing ones.
*/

/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.util;

import com.goldencode.p2j.comauto.*;
import java.io.*;
import java.lang.reflect.*;
import java.util.*;

/**
 * A class that represents a Progress 4GL compatible COM handle data type. It provides a mechanism
 * to group the functionality that is specific to a COM handle in Progress.
 */
public class comhandle
extends BaseDataType
{
   /** The object stored by this special handle. */
   protected ComObject value = null;
   
   /** 
    * Special handler that shows an invalid access error when a attribute or method member is
    * accessed for an unknown/invalid handle.
    */
   private final ControlFrameComObject invalidAccessHandler = new ControlFrameComObject()
   {
      /**
       * Shows an invalid access error.
       *
       * @param   ocx
       *          Ignored.
       * @param   ctrl
       *          Ignored.
       *
       * @return  always {@code unknown}.
       */
      @Override
      public logical loadControls(character ocx, character ctrl)
      {
         showInvalidAccessError("LoadControls");
         return new logical();
      }
      
      /**
       * Shows an invalid access error.
       *
       * @param   ocx
       *          Ignored.
       * @param   ctrl
       *          Ignored.
       *
       * @return  always {@code unknown}.
       */
      @Override
      public logical loadControls(String ocx, character ctrl)
      {
         showInvalidAccessError("LoadControls");
         return new logical();
      }
      
      /**
       * Shows an invalid access error.
       *
       * @param   ocx
       *          Ignored.
       * @param   ctrl
       *          Ignored.
       *
       * @return  always {@code unknown}.
       */
      @Override
      public logical loadControls(character ocx, String ctrl)
      {
         showInvalidAccessError("LoadControls");
         return new logical();
      }
      
      /**
       * Shows an invalid access error.
       *
       * @param   ocx
       *          Ignored.
       * @param   ctrl
       *          Ignored.
       *
       * @return  always {@code unknown}.
       */
      @Override
      public logical loadControls(String ocx, String ctrl)
      {
         showInvalidAccessError("LoadControls");
         return new logical();
      }
      
      /**
       * Shows an invalid access error.
       *
       * @return  always {@code unknown}.
       */
      @Override
      public comhandle getControlName()
      {
         showInvalidAccessError("CONTROL-NAME");
         return new comhandle();
      }
      
      /**
       * Shows an invalid access error.
       *
       * @return  always {@code unknown}.
       */
      @Override
      public comhandle getControls()
      {
         showInvalidAccessError("CONTROLS");
         return new comhandle();
      }
      
      /**
       * Shows an invalid access error.
       *
       * @return  always {@code unknown}.
       */
      @Override
      public handle getWidgetHandle()
      {
         showInvalidAccessError("WIDGET-HANDLE");
         return new handle();
      }
      
      /**
       * Shows an invalid access error.
       *
       * @return  always {@code unknown}.
       */
      @Override
      public integer getWidth()
      {
         showInvalidAccessError("WIDTH");
         return new integer();
      }
      
      /**
       * Shows an invalid access error.
       *
       * @param   width
       *          Ignored.
       */
      @Override
      public void setWidth(long width)
      {
         showInvalidAccessError("WIDTH");
      }
      
      /**
       * Shows an invalid access error.
       *
       * @param   width
       *          Ignored.
       */
      @Override
      public void setWidth(NumberType width)
      {
         showInvalidAccessError("WIDTH");
      }
      
      /**
       * Shows an invalid access error.
       *
       * @return  always {@code unknown}.
       */
      @Override
      public integer getHeight()
      {
         showInvalidAccessError("HEIGHT");
         return new integer();
      }
      
      /**
       * Shows an invalid access error.
       *
       * @param   height
       *          Ignored.
       */
      @Override
      public void setHeight(long height)
      {
         showInvalidAccessError("HEIGHT");
      }
      
      /**
       * Shows an invalid access error.
       *
       * @param   height
       *          Ignored.
       */
      @Override
      public void setHeight(NumberType height)
      {
         showInvalidAccessError("HEIGHT");
      }
      
      /**
       * Shows an invalid access error.
       *
       * @return  always {@code unknown}.
       */
      @Override
      public integer getLeft()
      {
         showInvalidAccessError("LEFT");
         return new integer();
      }
      
      /**
       * Shows an invalid access error.
       *
       * @param   left
       *          Ignored.
       */
      @Override
      public void setLeft(long left)
      {
         showInvalidAccessError("LEFT");
      }
      
      /**
       * Shows an invalid access error.
       *
       * @param   left
       *          Ignored.
       */
      @Override
      public void setLeft(NumberType left)
      {
         showInvalidAccessError("LEFT");
      }
      
      /**
       * Shows an invalid access error.
       *
       * @return  always {@code unknown}.
       */
      @Override
      public integer getTop()
      {
         showInvalidAccessError("TOP");
         return new integer();
      }
      
      /**
       * Shows an invalid access error.
       *
       * @param   top
       *          Ignored.
       */
      @Override
      public void setTop(long top)
      {
         showInvalidAccessError("LEFT");
      }
      
      /**
       * Shows an invalid access error.
       *
       * @param   top
       *          Ignored.
       */
      @Override
      public void setTop(NumberType top)
      {
         showInvalidAccessError("LEFT");
      }
      
      /**
       * Shows an invalid access error.
       *
       * @return  always {@code unknown}.
       */
      @Override
      public integer getWidgetId()
      {
         showInvalidAccessError("WIDGET-ID");
         return new integer();
      }
      
      /**
       * Shows an invalid access error.
       *
       * @return  always {@code unknown}.
       */
      @Override
      public handle getFrameHandle()
      {
         showInvalidAccessError("FRAME");
         return new handle();
      }
      
      /**
       * Shows an invalid access error.
       *
       * @return  always {@code unknown}.
       */
      @Override
      public comhandle getActiveX(String axName)
      {
         showInvalidAccessError(axName);
         return new comhandle();
      }
      
      /**
       * Shows an invalid access error.
       *
       * @return  always {@code unknown}.
       */
      @Override
      public character name()
      {
         showInvalidAccessError("NAME");
         return new character();
      }
      
      /**
       * Shows an invalid access error.
       *
       * @param   name
       *          Ignored.
       */
      @Override
      public void name(String name)
      {
         showInvalidAccessError("NAME");
      }
      
      /**
       * Shows an invalid access error.
       *
       * @param   name
       *          Ignored.
       */
      @Override
      public void name(character name)
      {
         showInvalidAccessError("NAME");
      }
   };
   
   /**
    * Default constructor, creates an instance that represents the unknown value.
    */
   public comhandle()
   {
   }
   
   /**
    * Constructs an instance that has the contained object reference and {@code unknown} that 
    * exactly matches that of the passed-in instance.
    *
    * @param    that
    *           The instance to pattern from.
    */
   public comhandle(comhandle that)
   {
      if (that.isUnknown())
      {
         this.setUnknown();
      }
      else
      {
         this.value = that.value;
      }
   }
   
   /**
    * Constructs an instance that wraps the given object reference as a {@link WrappedResource}.
    *
    * @param    value
    *           The instance to contain.
    */
   public comhandle(Object value)
   {
      if (value instanceof comhandle)
      {
         comhandle that = (comhandle) value;
         if (that.isUnknown())
         {
            this.setUnknown();
         }
         else
         {
            this.value = that.value;
         }
      }
      else if (value instanceof ComObject)
      {
         this.value = (ComObject) value;
      }
      else
      {
         setUnknown();
         System.err.println("Invalid construct!");
      }
   }
   
   /**
    * Modify the contained data of the instance to copy the instance data from the given instance.
    * Generally, the class of the given instance must match the class upon which the 
    * {@code assign} method is called, although this is not a strict requirement.
    *
    * @param   newVal
    *          The backup instance from which to copy data.
    */
   @Override
   public void assign(Undoable newVal)
   {
      if (newVal instanceof comhandle)
      {
         this.value = ((comhandle) newVal).value;
      }
      else if (newVal instanceof ComObject)
      {
         this.value = (ComObject) newVal;
      }
      else
      {
         System.err.println("Invalid construct!");
      }
   }
   
   /**
    * Dedicated assignment method for loading a {@code ComObject} into this {@code comhandle}.
    * This method is only called by {@link ComServer#create(String, comhandle)}. Stores a newly
    * created {@code ComObject} into this handle.
    * 
    * @param   newVal
    *          The new value to be stored by this handle. 
    */
   public void assign(ComObject newVal)
   {
      // TODO: check undoable ?
      this.value = newVal;
   }
   
   /**
    * Sets the state (data and unknown value) of this instance based on the state of the passed
    * instance.
    *
    * @param    value
    *           The instance from which to copy state.
    * @param    force
    *           Flag to indicate if a pending error should be ignored (see 
    *           {@link ErrorManager#isPendingError} and the assignment forced regardless.  Note
    *           that normal assignment silently exits with no change if there is a pending error.
    *           If this flag is {@code true}, that check is bypassed and the assignment will
    *           always be done.
    */
   @Override
   public void assign(BaseDataType value, boolean force)
   {
      if (value == null || value.isUnknown())
      {
         setUnknown();
      }
      else if (value instanceof comhandle)
      {
         comhandle that = (comhandle) value;
         this.value = that.value;
      }
      else if (value instanceof handle)
      {
         // special case when ZERO_RESOURCE is passed in, convert it to local ZERO_RESOURCE
         WrappedResource res = ((handle) value).getResource();
         if (res != null && res.id() == 0)
         {
            this.value = ZERO_RESOURCE; 
         }
      }
//      else if (value instanceof character)
//      {
//         assign(fromString((character) value), force);
//      }
      else
      {
         String err = "Incompatible data types in expression or assignment.";
         ErrorManager.recordOrThrowError(223, err);
      }
   }
   
   /**
    * Determines if this instance is valid (FWD internal access).
    *
    * @return  boolean {@code true} if this instance can be used.
    */
   public boolean _isValid()
   {
      return (value != null) && value.valid();
   }
   
   /**
    * Determines if this instance is valid.
    *
    * @return  {@code true} if this instance can be used.
    */
   public logical isValid()
   {
      return new logical(_isValid());
   }
   
   /**
    * Creates a new instance of the same type that represents the {@code unknown} value.
    *
    * @return   An instance that represents the {@code unknown} value.
    */
   @Override
   public BaseDataType instantiateUnknown()
   {
      return new comhandle();
   }
   
   /**
    * Does the same as standard {@code clone()} method but returns an instance of
    * {@code BaseDataType} and doesn't throw the {@code CloneNotSupportedException}.
    *
    * @return   A clone of this instance.  
    */
   @Override
   public BaseDataType duplicate()
   {
      return new comhandle(this);
   }
   
   /**
    * Reports if this instance represents the {@code unknown}.
    *
    * @return   {@code true} if this instance is set to the {@code unknown} value.
    */
   @Override
   public boolean isUnknown()
   {
      return value == null;
   }
   
   /**
    * Custom implementation of {@link Object#hashCode}.
    *  
    * @return  a hashcode for current object.
    */
   @Override
   public int hashCode()
   {
      if (isUnknown()) // Ie. value == null
      {
         return ResourceIdHelper.UNKNOWN_STRING.hashCode();
      }
      
      return value.hashCode();
   }
   
   /**
    * Sets the state of this instance's {@code unknown value} flag to {@code true}.
    * <p>
    * <b>Warning: a separate call is needed to ensure that the data of this instance is set to the
    * correct value.</b>
    */
   @Override
   public void setUnknown()
   {
      if (!isUnknown())
      {
         checkUndoable(true);
      }
      
      value = null;
   }
   
   /**
    * Compares this instance with the specified instance and returns a -1 if this instance is less
    * than the specified, 0 if the two instances are equal and 1 if this instance is greater than
    * the specified instance.  This is the implementation of the {@code Comparable} interface.
    * <p>
    * The algorithm will fail to give meaningful results in the case where one tries to sort
    * against other objects that do not represent compatible values.
    *
    * @param    obj
    *           The instance to compare against.
    *
    * @return   0 or 1 depending on if the passed instance is or is not {@code unknown}.
    */
   @Override
   public int compareTo(Object obj)
   {
      // handle is also a BDT type
      if (obj instanceof comhandle)
      {
         comhandle that = (comhandle) obj;
         
         // two unknown values are equals
         if (isUnknown() || that.isUnknown())
         {
            // an unknown handle is greater than a known handle
            return Boolean.compare(isUnknown(), that.isUnknown());
         }
         
         return (handle.resourceId(this.value) == handle.resourceId(that.value)) ? 0 : 1;
      }
      return -1;
   }
   
   /**
    * Creates a string representation of the instance data using the format 
    * {@link ResourceIdHelper#DEFAULT_FORMAT}. If the instance represents the {@code unknown}
    * value, a '?' will be returned.
    *
    * @return  The formatted string.
    */
   public String toString()
   {
      return toString(null);
   }
   
   /**
    * Creates a string representation of the instance data using the user specified format string
    * (Progress 4GL compatible) or if no such string is provided, the default format of 
    * {@link ResourceIdHelper#DEFAULT_FORMAT}. If the instance represents the {@code unknown}
    * value, a '?' will be returned.
    *
    * @param   fmt
    *          The Progress 4GL format string or {@code null} if the default format is to be used.
    *
    * @return  The formatted string.
    */
   public String toString(String fmt)
   {
      if (isUnknown())
      {
         return ResourceIdHelper.UNKNOWN_STRING;
      }
      
      Long id = handle.resourceId(value);
      
      return ResourceIdHelper.idToString(fmt, id);
   }
   
   /**
    * Creates a string representation of the instance data in a form that is compatible with the
    * {@code MESSAGE} language statement.  If the instance represents the {@code unknown value},
    * a '?' will be returned.
    *
    * @return  The 'message' formatted string.
    */
   public String toStringMessage()
   {
      return toString();
   }
   
   /**
    * Creates a string representation of the instance data using the 'export' format.  If the
    * instance represents the {@code unknown value}, a '?' will be returned.
    *
    * @return   The 'export' formatted string.
    */
   public String toStringExport()
   {
      return toString();
   }
   
   /**
    * Return the default display format string for this type.
    * <p>
    * Warning: this is larger than the Progress version because the range of handle values can be
    * a 64-bit integer in this implementation.
    *
    * @return   The default format string.
    */
   public String defaultFormatString()
   {
      return ResourceIdHelper.DEFAULT_FORMAT;
   }   
   
   /**
    * Replacement for the default object reading method. The latest state is read from the input
    * source.
    * 
    * @param    in
    *           The input source from which fields will be restored.
    *
    * @throws   IOException
    *           In case of I/O errors.
    * @throws   ClassNotFoundException
    *           If payload can't be instantiated.
    */
   @Override
   public void readExternal(ObjectInput in)
   throws IOException,
          ClassNotFoundException
   {
      throw new NotSerializableException("comhandle");
   }
   
   /**
    * Replacement for the default object writing method. The latest state is written to the output
    * destination.
    * 
    * @param    out
    *           The output destination to which fields will be saved.
    *
    * @throws   IOException
    *           In case of I/O errors.
    */
   @Override
   public void writeExternal(ObjectOutput out)
   throws IOException
   {
      throw new NotSerializableException("comhandle");
   }
   
   /**
    * Unwrap this com-handle to the {@link ControlFrameComObject} instance of the parent 
    * {@link ControlFrame}. If the comhandle is not obtained from a Control Frame using 
    * {@code COM-HANDLE} attribute handle or is invalid, a special {@link ControlFrameComObject}
    * is returned that handles invalid accesses.
    *
    * @return  See above. 
    */
   public ControlFrameComObject unwrapComObject()
   {
      // to the best effort to identify the object 
      if (value instanceof ControlFrameComObject)
      {
         return (ControlFrameComObject) value;
      }
      
      return invalidAccessHandler;
   }
   
   /**
    * Sets a COM property for the COM object stored by the handle.
    * 
    * @param   prop
    *          The legacy property name. Case insensitive. If no property is found then a warning
    *          is displayed and method returns without altering the stored object.
    * @param   newVal
    *          The new value for the property. It must be of a compatible type with the property.  
    */
   public void setProperty(String prop, Object newVal)
   {
      Method setter = getComAccessor(value, prop, true);
      if (setter == null)
      {
         showAccessError(prop.toUpperCase(), "0x80020006", "Unknown name.");
         return;
      }
      
      try
      {
         // prepare params
         BaseDataType[] params = new BaseDataType[1]; 
         if (newVal instanceof BaseDataType)
         {
            params[0] = (BaseDataType) newVal;
         }
         else 
         {
            // setter is guaranteed to have exactly one parameter because of getComAccessor()
            params[0] = marshal(newVal, setter.getParameterTypes()[0]);
         }
         
         try
         {
            setter.invoke(value, (Object[]) params);
         }
         catch (IllegalAccessException | InvocationTargetException e)
         {
            // res remains unchanged 
         }
      }
      catch (PropertyAccessException pae)
      {
         showAccessError(prop.toUpperCase(), pae.getErrorCode(), pae.getMessage());
      }
   }
   
   /**
    * Obtain the COM object held by this comhandle.
    * 
    * @return  the COM object held by this comhandle.
    */
   public ComObject getResource()
   {
      return value;
   }
   
   /**
    * Obtain a property value from the COM object stored in this handle. 
    * 
    * @param   prop
    *          The legacy name of the property. Case insensitive. If no property is found then a
    *          warning is displayed and method returns {@code unknown} value.
    *
    * @return  The value of the requested property or {@code unknown} on exceptions.
    */
   public BaseDataType getProperty(String prop)
   {
      if (value == null)
      {
         showInvalidAccessError(prop.toUpperCase());
         return new unknown();
      }
      
      // check this is a top-level ActiveX container
      if (value instanceof ControlFrameComObject)
      {
         comhandle axObject = ((ControlFrameComObject) value).getActiveX(prop);
         // sometimes a "0" is return character (C++ NULL perhaps?) if no such property is found
         // not sure if this is the same ZERO_RESOURCE object like the one from handle.
         return (axObject == null || axObject.isUnknown()) ? new handle(ZERO_RESOURCE) : axObject;
      }
      
      Method getter = getComAccessor(value, prop, false);
      if (getter != null)
      {
         Object ret = null;
         try
         {
            ret = getter.invoke(value, (Object[]) null);
         }
         catch (IllegalAccessException | InvocationTargetException e)
         {
            // res remains [null] and won't be cast to BDT and returned 
         }
         
         if (ret instanceof BaseDataType)
         {
            return (BaseDataType) ret;
         }
         else if (ret instanceof ComObject)
         {
            return new comhandle(ret);
         }
      }
      
      showAccessError(prop.toUpperCase(), "0x80020006", "Unknown name.");
      return new unknown();
   }
   
   /**
    * Chains access to COM property/methods. This call is generated when a chained construct is
    * encountered in legacy code. For example, {@code ch:prop1:prop2:prop-or-method} will be
    * converted to something like: {@code ch.chain("prop1").chain("prop2").get("prop-or-method")} 
    * 
    * @param   property
    *          The property to access. It should normally be a com-object property of the object
    *          stored in this {@code comhandle}. 
    * 
    * @return  A {@code comhandle} to the COM object property of currently referred resource. It
    *          normally should be chained with another access for a property of method. 
    */
   public comhandle chain(String property)
   {
      return new comhandle(getProperty(property));
   }
   
   /**
    * Calls a COM-method without parameters on the COM-object stored in this com-handle.
    *
    * @param   methodName
    *          The method name. Case insensitive. If the COM-object does not declare such method
    *          an error message is displayed and this method returns {@code unknown} value.
    * @param   params
    *          The list of actual parameters. Their types must be compatible with the parameters
    *          of called method.
    *          TODO: marshal to BDT (?) When code is generated [param] is already BDT.
    *
    * @return  The result of the called method, if any.
    */
   public BaseDataType call(String methodName, Object... params)
   {
      if (value == null)
      {
         showInvalidAccessError(methodName.toUpperCase());
         //what is the error
         return new unknown();
      }
      
      Class<?> clazz = value.getClass();
      while (clazz != Object.class)
      {
         List<Method> allMethods = new ArrayList<>(Arrays.asList(clazz.getDeclaredMethods()));
         for (Method method : allMethods)
         {
            ComMethod ann = method.getAnnotation(ComMethod.class);
            if (ann != null && ann.name().equalsIgnoreCase(methodName))
            {
               // check parameter compatibility / marshal params to DBT (?)
               try
               {
                  Object ret = method.invoke(value, params);
                  if (ret instanceof BaseDataType)
                  {
                     return (BaseDataType) ret;
                  }
                  else 
                  {
                     return marshal(ret, null);
                  }
               }
               catch (IllegalAccessException | InvocationTargetException e)
               {
                  e.printStackTrace();
               }
            }
         }
         // move to the upper class in the hierarchy in search for more methods
         clazz = clazz.getSuperclass();
      }
      
      return new comhandle();
   }
   
   /**
    * Calls a COM-method without parameters on the COM-object stored in this com-handle.
    *
    * @param   methodName
    *          The method name. Case insensitive. If the COM-object does not declare such method
    *          an error message is displayed and this method returns {@code unknown} value.
    * @param   params
    *          The list of actual parameters. Their types must be compatible with the parameters
    *          of called method.
    *          TODO: marshal to BDT (?) When code is generated [param] is already BDT.
    *
    * @return  The result of the called method, in a {@code comhandle}, so it can be used in a
    *          chained call or property access. 
    */
   public comhandle chainCall(String methodName, Object... params)
   {
      if (value == null)
      {
         showInvalidAccessError(methodName.toUpperCase());
         //what is the error
         return new comhandle();
      }
      
      Class<?> clazz = value.getClass();
      while (clazz != Object.class)
      {
         List<Method> allMethods = new ArrayList<>(Arrays.asList(clazz.getDeclaredMethods()));
         for (Method method : allMethods)
         {
            ComMethod ann = method.getAnnotation(ComMethod.class);
            if (ann != null && ann.name().equalsIgnoreCase(methodName))
            {
               // check parameter compatibility / marshal params to DBT (?)
               try
               {
                  Object ret = method.invoke(value, params);
                  // the result must be some king of ComObject so that the chained child
                  // call/property could be executed
                  if (ret instanceof ComObject)
                  {
                     return new comhandle(ret);
                  }
                  else 
                  {
                     // otherwise a NPE will occur because of invalid result 
                     return new comhandle();
                  }
               }
               catch (IllegalAccessException | InvocationTargetException e)
               {
                  e.printStackTrace();
               }
            }
         }
         // move to the upper class in the hierarchy in search for more methods
         clazz = clazz.getSuperclass();
      }
      
      return new comhandle();
   }
   
   /**
    * Calls a COM-method without parameters on the COM-object stored in this com-handle.
    *
    * @param   method
    *          The method name. Case insensitive. If the COM-object does not declare such method
    *          an error message is displayed and this method returns {@code unknown} value.
    *
    * @return  The result of the called method, if any.
    */
   public BaseDataType call(String method)
   {
      return call(method, (Object[]) null);
   }
   
   /**
    * Converts a value to a BDT object. The method does not check whether the value is already a
    * {@code BaseDataType}, in fact is assume it is not. Note that, because this method uses the
    * best matching constructor, this method may do some implicit conversions if the passed in
    * value is compatible with the class required (like from a {@code String}).
    * 
    * @param   aVal
    *          The value to be converted.
    * @param   bdtClass
    *          The type of the returned result (as a subclass of {@code BaseDataType}).
    *
    * @return  A {@code BaseDataType} instance that represent the same value as {@code aVal}.
    */
   private BaseDataType marshal(Object aVal, Class<?> bdtClass)
   {
      if (bdtClass != null)
      {
         try
         {
            Constructor<?> ctor = bdtClass.getConstructor(aVal.getClass());
            return (BaseDataType) ctor.newInstance(aVal);
         }
         catch (NoSuchMethodException  |
                IllegalAccessException |
                InstantiationException |
                InvocationTargetException e)
         {
            // TODO: log this
         }
      }
      else
      {
         if (aVal != null)
         {
            if (aVal instanceof BaseDataType)
            {
               return (BaseDataType) aVal;
            }
            else if (aVal instanceof String)
            {
               return new character((String) aVal);
            }
            else if (aVal instanceof Boolean)
            {
               return new logical((Boolean) aVal);
            }
            else if (aVal instanceof Integer)
            {
               return new integer((Integer) aVal);
            }
            else if (aVal instanceof Long)
            {
               return new int64((Long) aVal);
            }
            else if (aVal instanceof Double)
            {
               return new decimal((Double) aVal);
            }
            else if (aVal instanceof Float)
            {
               return new decimal((Float) aVal);
            }
            else if (aVal instanceof Date)
            {
               return new date((Date) aVal);
            }
         }
      }
      return new unknown();
   }
   
   /**
    * Internal utility method that selects an accessor for a property from a COM object.
    * 
    * @param   comObject
    *          The object being investigated.
    * @param   prop
    *          The property being searched. The accessor method (setter or getter) must be
    *          annotated with proper {@code ComProperty} annotation.
    * @param   setter
    *          {@code true} for setter methods and {@code false} for getters. In either case a
    *          quick check is performed to verify whether the eventually found method are the
    *          requested kind of accessors.
    * 
    * @return  A method for accessing the respective property or {@code null} if no such property
    *          was found or it does not meet the minimum syntactic conditions.
    */
   private Method getComAccessor(ComObject comObject, String prop, boolean setter)
   {
      if (comObject == null)
      {
         return null;
      }
      
      Class<?> clazz = comObject.getClass();
      while (clazz != Object.class)
      {
         List<Method> allMethods = new ArrayList<>(Arrays.asList(clazz.getDeclaredMethods()));
         for (Method method : allMethods)
         {
            ComProperty ann = method.getAnnotation(ComProperty.class);
            if (ann != null                        && 
                ann.name().equalsIgnoreCase(prop)  &&
                ann.setter() == setter)
            {
               // check parameter compatibility for setter
               if (setter)
               {
                  Class<?>[] params = method.getParameterTypes();
                  if (params == null || params.length != 1)
                  {
                     return null;
                  }
               }
               else
               {
                  Class<?>[] params = method.getParameterTypes();
                  if (params != null && params.length != 0)
                  {
                     return null;
                  }
               }
               return method;
            }
         }
         // move to the upper class in the hierarchy in search for more methods
         clazz = clazz.getSuperclass();
      }
      
      return null;
   }
   
   /**
    * Shows a standard error for accessing an property of method of a com object stored in a
    * {@code com-handle}.
    *
    * @param   prop
    *          The legacy name of the property/method being accessed. 
    */
   public static void showAccessError(String prop, String errCode, String errMsg)
   {
      String msg = "Error occurred while accessing component property/method: " +
                   prop + ".\n" + errMsg + "\nError code: " + errCode + " " +
                   ProcedureManager.getInstantiatingExternalProgram();
      ErrorManager.displayError(5890, msg, false);
   }
   
   /**
    * Shows a standard error for accessing an invalid {@code com-handle}.
    *
    * @param   prop
    *          The legacy name of the property or method being accessed. 
    */
   private static void showInvalidAccessError(String prop)
   {
      String msg = "Invalid component-handle referenced while processing method/statement: " +
                   prop + ".\n\n" + ProcedureManager.getInstantiatingExternalProgram();
      ErrorManager.displayError(5884, msg, false);
   }
   
   /**
    * A "zero" pseudo-resource. A "zero" resource is used for handles created by the 
    * {@link handle#fromString(String)} method when a formally valid ID is provided, but the ID 
    * doesn't denote a valid resource.
    * <p>
    * A zero resource is not valid but is not unknown.
    */
   private static final ComObject ZERO_RESOURCE = new ComObject()
   {
      @Override
      public boolean valid()
      {
         return false;
      }
      
      @Override
      public boolean unknown()
      {
         return false;
      }
      
      @Override
      public Long id()
      {
         return 0L;
      }
      
      @Override
      public void id(long id)
      {
         // no-op
      }
      
      @Override
      public String getName()
      {
         return null;
      }
      
      @Override
      public void release()
      {
      }
   };
}
