#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You may find a copy of the GNU Affero GPL version 3 at the following
# location: https://www.gnu.org/licenses/agpl-3.0.en.html
# 
# Additional terms under GNU Affero GPL version 3 section 7:
# 
#   Under Section 7 of the GNU Affero GPL version 3, the following additional
#   terms apply to the works covered under the License.  These additional terms
#   are non-permissive additional terms allowed under Section 7 of the GNU
#   Affero GPL version 3 and may not be removed by you.
# 
#   0. Attribution Requirement.
# 
#     You must preserve all legal notices or author attributions in the covered
#     work or Appropriate Legal Notices displayed by works containing the covered
#     work.  You may not remove from the covered work any author or developer
#     credit already included within the covered work.
# 
#   1. No License To Use Trademarks.
# 
#     This license does not grant any license or rights to use the trademarks
#     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
#     of Golden Code Development Corporation. You are not authorized to use the
#     name Golden Code, FWD, or the names of any author or contributor, for
#     publicity purposes without written authorization.
# 
#   2. No Misrepresentation of Affiliation.
# 
#     You may not represent yourself as Golden Code Development Corporation or FWD.
# 
#     You may not represent yourself for publicity purposes as associated with
#     Golden Code Development Corporation, FWD, or any author or contributor to
#     the covered work, without written authorization.
# 
#   3. No Misrepresentation of Source or Origin.
# 
#     You may not represent the covered work as solely your work.  All modified
#     versions of the covered work must be marked in a reasonable way to make it
#     clear that the modified work is not originating from Golden Code Development
#     Corporation or FWD.  All modified versions must contain the notices of
#     attribution required in this license.
#
#!/bin/bash
#
# This script MUST be always executed having root privileges:
#	$ sudo ./postbuild.sh <spawn-install-dir> <spawn-tool> [ <srv-certs.store location> ]
#
# In order to avoid to provide the root password whenever execute the 
# sudo postbuild.sh command (used to make spawner, see makefile.spawner)
# please do the following:
#
# - create a text file (example sudoers-ext) inside the etc/sudoers.d folder.
#   basically this file extends the /etc/sudoers file which is recommended 
#   to remain unchanged.
# - open to edit this file using visudo and add the following line:
#
# username ALL=(ALL) NOPASSWD: /p2j_path/src/native/postbuild.sh
#
# - replace username with your Linux account name and change the path to
#   the location of your p2j project folder. 
#   Example for user mag:
#
# mag ALL=(ALL) NOPASSWD: /home/mag/p2j/src/native/postbuild.sh
#
# - save your changes.
# - the mode of the file sudoers-ext should be 0440.
# - please check and change if necessary (chmod 0440 sudoers-ext)
#
# - open etc/sudoers file using visudo editor and check if you already 
#   have a directive like:
#	#includedir /etc/sudoers.d
# - if not please add this line at the end of your etc/sudoers file.
#
# - if this does not work on your Linux distribution you may edit
#   directly the /etc/sudoers file. (not recommended if you already 
#   have an /etc/sudores.d folder alternative)
#

if [ $# -eq 0 ]
then
   echo "Usage: $0 <spawn-install-dir> <spawn-tool> [ <srv-certs.store location> ]"
   exit 1
fi

if [ $# -eq 2 ]
then
   echo "Installing the spawn tool from $2 into $1/spawn ..."
else
   echo "Installing the spawn tool from $2 into $1/spawn using $3 as srv-certs.store file..."
fi

if [ ! -d "$1" ]
then
   echo "$1 does not exist!"
   exit 1
fi

if [ ! -f $2 ]
then
   echo "$2 does not exist!"
   exit 1
fi

P2J_JAR="../lib/p2j.jar"
if [ ! -f $P2J_JAR ]
then
   echo "$P2J_JAR does not exist!"
   exit 1
fi

if [ $# -eq 3 ]
then
   if [ ! -f "$3" ]
   then
      echo "$3 does not exist!"
      exit 1
   fi

   cp -f $3 $1/srv-certs.store
else
   echo "WARNING! The srv-certs.store file was not specified."
fi

cp -f $2 $1/spawn
cp -f $P2J_JAR $1/p2j.jar

sudo chown root:root $1/spawn $1/p2j.jar 
sudo chmod 4755 $1/spawn
sudo chmod 0550 $1/p2j.jar

if [ -f "$1/srv-certs.store" ]
then
   sudo chown root:root $1/srv-certs.store
   sudo chmod 0550 $1/srv-certs.store
fi