/*
** Module   : process.c
** Abstract : implementation for process creation/waiting logic in libp2j.so or p2j.dll 
**            native library which is called via JNI
**
** Copyright (c) 2010-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- ---------------------------------Description----------------------------------
** 001 LMR 20101123 Initial revision of this file. The code was moved from 
**                  Toolkit.c in the charva package (built as libTerminal.so).
**                  It was an original work created by Golden Code and was 
**                  never a part of the Rob Pitman charva distribution.  For
**                  this reason, it is fully owned by Golden Code and has been
**                  moved here to keep it separate from the LGPL charva code.
** 002 GES 20110629 Removed unused var reference that was causing a compile
**                  warning.
** 003 EVL 20130329 Making changes to compile Windows DLL.
** 004 EVL 20140117 Moving the init steps to the special location.
** 005 ECF 20150126 Added flag for background mode to pseudoTerminalLaunch. Do not check whether
**                  STDIN_FILENO is a TTY when in background mode.
*/ 
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

#include "process_launch.h"

#include "com_goldencode_p2j_util_LaunchManager.h"

/**
 * Java_com_goldencode_p2j_util_LaunchManager_init - 
 *                         set up signal handling to ignore job control
 *                         signals (reflecteds in the child process as well)
 *
 * @param   env   pointer to an object providing access to JNI environment
 *
 * @param   cls   reference to the Java class who called this function
 *                (from static context)
 *
 */ 
/*JNIEXPORT void JNICALL Java_com_goldencode_p2j_util_LaunchManager_init(
                                                                JNIEnv* env,
                                                                jclass class)
{
   init(env, class);
}*/

/**
 * Java_com_goldencode_p2j_util_LaunchManager_pseudoTerminalLaunch - 
 *      spawns a child process redirecting its standard input, output and
 *      error to particular file descriptors (FileDescriptor Java instances) 
 *      
 *
 * @param   env         pointer to a structure that provides access to JNI
 *                      environment
 *
 * @param   cls         reference to the Java class who called this function
 *                      (from static context)
 *
 * @param   cmdline     the command line used for launching the child process  
 *
 * @param   in          FileDescriptor Java object for stdin
 *
 * @param   out         FileDescriptor Java object for stdout
 *
 * @param   err         FileDescriptor Java object for stderr
 *
 * @param   silent      boolean flag controlling interactive mode
 *
 * @param   background  boolean flag indicating client is running in the
 *                      background (disconnected from its controlling terminal)
 *
 * @return  > 1  PID of spawned child process if successful
 *          < 0  on error
 */ 

JNIEXPORT jint JNICALL
   Java_com_goldencode_p2j_util_LaunchManager_pseudoTerminalLaunch(JNIEnv*       env,
                                                                   jclass        cls,
                                                                   jobjectArray  cmdline,
                                                                   jobject       in,
                                                                   jobject       out,
                                                                   jobject       err,
                                                                   jboolean      silent,
                                                                   jboolean      background)
   
{
   return pseudoTerminalLaunch(env, cls, cmdline, in, out, err, silent, background);
}

/**
 * Java_com_goldencode_p2j_util_LaunchManager_pseudoTerminalWait - 
 *          waits for a child process to terminate. 
 *
 * @param   env     pointer to a structure that provides access to JNI environment
 *
 * @param   cls     reference to the Java class who called this function from
 *                  a static context
 *
 * @param   pid     PID of the process to be waited
 *
 * @param   silent  boolean flag to control interactive mode
 *
 */ 
JNIEXPORT void JNICALL
   Java_com_goldencode_p2j_util_LaunchManager_pseudoTerminalWait(JNIEnv*  env,
                                                                 jclass   cls,
                                                                 jint      pid,
                                                                 jboolean silent)
{
   pseudoTerminalWait(env, cls, pid, silent);
}
